*=======================================================================*
*                                                                       *
*                     S T E P   B O O T D I S T                         *
*                                                                       *
*  Bootstrap Analysis of Genetic similarity and distance coefficients   *
*                                                                       *
*  Similarity and distance coefficients are referred to throughout the  *
*  program by the following indices:                                    *
*                                                                       *
*        1 = Nei (1978) I (unbiased)                                    *
*        2 = Nei (1978) D (unbiased)                                    *
*        3 = Nei (1972) I                                               *
*        4 = Nei (1972) D                                               *
*        5 = Nei (1978) minimum D (unbiased)                            *
*        6 = Nei (1972) minimum D                                       *
*        7 = Rogers (1972) S                                            *
*        8 = Rogers (1972) D                                            *
*        9 = modified Rogers distance (Wright, 1978)                    *
*       10 = Prevosti distance (Wright, 1978)                           *
*       11 = Cavalli-Sforza and Edwards (1967) chord distance           *
*       12 = Cavalli-Sforza and Edwards (1967) arc distance             *
*       13 = Edwards (1971,1974) "E" distance                           *
*                                                                       *
*  Subprograms required by step SIMDIS: BOOTDIST  READER  ALLCNT        *
*                                                                       *
*=======================================================================*

      SUBROUTINE BOOTDIST
     +   (BDIST,SDIST,ADIST,FREQ2,MINI2,MAXI2,N2,JX,GX,SMJX2,SMGX2,
     +    EDJX2,POPNUM,ML,MA,MT)

*  Calculates similarity and distance coefficients for pairwise
*  comparisons between populations

      CHARACTER LINE*80,DATE*12,TIME*12,TITLE*72,FMT*80,
     +   INTST*64,REALST*8,LOGST*8,COEF*48,PHYF*64,POPNUM*4
      REAL BDIST,SDIST,ADIST,JX,GX,SMJX2,SMGX2,EDJX2,FREQ2,
     +   JXY1,JX1,JY1,IUN,ID,REALVL,SEED*8
      INTEGER MINI2,MAXI2,N2,INTVL,DCOEF,POP1,POP2
      PARAMETER (ZERO=1.E-6, PI=3.14159, CFACT=2.*1.41421/PI,
     +    EFACT=CFACT*2.)
      COMMON/GBPARM/NOTU,NOTU2,NLOC,NALL,NLET,CRT,NDATYP
      COMMON/MAT/MATFIL(13),NOCOEF,NMAT,NSING
      COMMON/COEFNM/COEF(13)
      COMMON/CHARV/LINE,DATE,TIME,TITLE,FMT
      COMMON/PAGES/FULSKP

      DIMENSION INTST(4),BDIST(ML,MT,MT),SDIST(MT,MT),POPNUM(MT),
     +   ADIST(MT,MT),JX(MT),GX(MT),SMJX2(MT),SMGX2(MT),EDJX2(MT),
     +   FREQ2(MA,MT),MINI2(MT),MAXI2(MT),N2(MT),INTVL(3),REALVL(1),
     +   LOGVL(7),REALST(1)
* below are where the valid tokens are declared...
      LOGICAL FULSKP
* added 'SEED' and 'PHYF'
      DATA INTST/'COEF','NREPS','SEED','PHYF'/
     
      COEF(1)='Nei (1978) unbiased genetic identity'
      COEF(2)='Nei (1978) unbiased genetic distance'
      COEF(3)='Nei (1972) genetic identity'
      COEF(4)='Nei (1972) genetic distance'
      COEF(5)='Nei (1978) unbiased minimum distance'
      COEF(6)='Nei (1972) minimum distance'
      COEF(7)='Rogers (1972) genetic similarity'
      COEF(8)='Rogers (1972) genetic distance'
      COEF(9)='Modified Rogers distance (Wright,1978)'
      COEF(10)='Prevosti distance (Wright,1978)'
      COEF(11)='Cavalli-Sforza & Edwards (1967) chord distance'
      COEF(12)='Cavalli-Sforza & Edwards (1967) arc distance'
      COEF(13)='Edwards (1971,1974) "E" distance'

      CALL PAGSKP
      WRITE (6,600) TITLE,DATE,TIME

*     Defaults:

         INTVL(1)=0
         INTVL(2)=0
         INTVL(3)=IRand(0)


         CALL READER (4,0,0,INTST,REALST,LOGST,INTVL,REALVL,LOGVL,
     +      IERROR)
         IF (IERROR.EQ.1) RETURN
         DCOEF=INTVL(1)
         NBOOT=INTVL(2)
         SEED=INTVL(3)
* kludge: PHYF filename is put in the INTST array at READER (MISC.FOR)
         PHYF=INTST(4)

      IF(NBOOT.GE.1) THEN
*      WRITE(*,'(//////////////////////)')
*      WRITE(*,'(A)')' Distance/Similarity measure:'
*      WRITE(*,'(A64)')COEF(DCOEF)
*      WRITE(*,'(A)')' Name of the file to contain the bootstrapped PHYLI
*     +P matrix file?'
*      READ(*,'(A)') ONAME
*      OPEN(UNIT=6,FILE=ONAME)

* default is 'out.phy'/maj
      OPEN(UNIT=8,FILE=PHYF)
* SEED is now set in the data file/maj
*      WRITE(*,'(A)')' Random seed?'
*      READ(*,'(F10.0)')SEED
      ENDIF

      IF(NBOOT.LT.1) THEN
*      WRITE(*,'(//////////////////////)')
*      WRITE(*,'(A)')' Distance/Similarity measure:'
*      WRITE(*,'(A64)')COEF(DCOEF)
*      WRITE(*,'(A)')' Name of the file to contain the PHYLIP matrix?'
*      READ(*,'(A)')ONAME
      OPEN(UNIT=8,FILE=PHYF)
      ENDIF

      DO 100 J=1,NLOC
      DO 110 K=1,NOTU
         JX(K)=0.
         GX(K)=0.

         IF(DCOEF.EQ.13) THEN
            BDIST(J,K,K)=99.E9
            GO TO 110
         ELSE IF(DCOEF.EQ.12) THEN
            BDIST(J,K,K)=99.E9
            GO TO 110
         ELSE IF(DCOEF.EQ.10) THEN
            BDIST(J,K,K)=99.E9
            GO TO 110
         ELSE
            BDIST(J,K,K)=0.
            GO TO 110
         ENDIF

110   CONTINUE
100   CONTINUE

      DO 120 J=1,NLOC
       DO 130 K1=2,NOTU
         DO 140 K2=1,K1-1
            BDIST(J,K1,K2)=0.
            SDIST(K1,K2)=0
140      CONTINUE
130    CONTINUE
120   CONTINUE

*  Calculate distances:

      REWIND 16
      DO 200 J=1,NLOC
         READ (16) FREQ2,MINI2,MAXI2,N2
      IF (DCOEF.EQ.13) CALL ALLCNT (FREQ2,MINI2,MAXI2,NALLS,NOTU,MA,MT)
         DO 210 K1=1,NOTU
            MINIX=MINI2(K1)
            MAXIX=MAXI2(K1)
            NX=N2(K1)
            SMJX=0.
            EDJX=0.
            DO 220 I=MINIX,MAXIX
               QFREQ=FREQ2(I,K1)
               SMJX=SMJX+QFREQ*QFREQ
               IF (DCOEF.EQ.13) EDJX=EDJX+SQRT(QFREQ/NALLS)
220         CONTINUE
            SMJX2(K1)=SMJX
            SMGX2(K1)=(2.*NX*SMJX-1.)/(2*NX-1)
            IF (SMGX2(K1).LE.ZERO) SMGX2(K1)=0.5
            EDJX2(K1)=EDJX
            JX(K1)=JX(K1)+SMJX
            GX(K1)=GX(K1)+SMGX2(K1)
            IF (K1.NE.1) THEN
               DO 230 K2=1,K1-1
                  MINIY=MINI2(K2)
                  MAXIY=MAXI2(K2)
                  IMIN=MIN(MINIX,MINIY)
                  IMAX=MAX0(MAXIX,MAXIY)
                  IF (MINIX .EQ. MAXIX .AND. MINIY .EQ. MAXIY) THEN
                     IF (IMIN .EQ. IMAX) THEN
*                       ( Populations K1 and K2 fixed identically )
                        SUMXY=1.
                        COSTHE=1.
                        SUM1=0.
                        SUM2=0.
                     ELSE
*                       ( Populations K1 and K2 fixed differently )
                        SUMXY=0.
                        COSTHE=0.
                        SUM1=2.
                        SUM2=2.
                     ENDIF
                  ELSE
*                    ( Populations K1 and/or K2 polymorphic )
                     SUMXY=0.
                     COSTHE=0.
                     SUM1=0.
                     SUM2=0.
                     DO 240 I=IMIN,IMAX
                        XI=FREQ2(I,K1)
                        YI=FREQ2(I,K2)
                        DIFFXY=ABS(XI-YI)
                        SUMXY=SUMXY+XI*YI
                        COSTHE=COSTHE+SQRT(XI*YI)
                        SUM1=SUM1+DIFFXY*DIFFXY
                        SUM2=SUM2+DIFFXY
240                  CONTINUE
                  ENDIF

                  DJROG=SQRT(SUM1/2.)
                  DJPREV=SUM2/2.
                  IF (COSTHE.GT.1.) COSTHE=1.
                  THETA=ACOS(COSTHE)
                  DJARC=2.*THETA/PI
                  DJCHRD=CFACT*SQRT(1.-COSTHE)
                  DJEDW=EFACT*SQRT((1.-COSTHE)/((1.+EDJX2(K1))*
     +                  (1.+EDJX2(K2))))
                  IUN=SUMXY/SQRT(SMGX2(K1)*SMGX2(K2))
                  IF (IUN.GT.1) IUN=1.
                  ID=SUMXY/SQRT(SMJX2(K1)*SMJX2(K2))
*                 Totals:
         IF(DCOEF.LE.6) THEN
            BDIST(J,K1,K2)=BDIST(J,K1,K2)+SUMXY
            GO TO 230
         ELSE IF(DCOEF.LE.8) THEN
            BDIST(J,K1,K2)=BDIST(J,K1,K2)+DJROG
            GO TO 230
         ELSE IF(DCOEF.EQ.9) THEN
            BDIST(J,K1,K2)=BDIST(J,K1,K2)+(DJROG*DJROG)
            GO TO 230
         ELSE IF(DCOEF.EQ.10) THEN
            BDIST(J,K1,K2)=BDIST(J,K1,K2)+DJPREV
            GO TO 230
         ELSE IF(DCOEF.EQ.11) THEN
            BDIST(J,K1,K2)=BDIST(J,K1,K2)+(DJCHRD*DJCHRD)
            GO TO 230
         ELSE IF(DCOEF.EQ.12) THEN
            BDIST(J,K1,K2)=BDIST(J,K1,K2)+(DJARC*DJARC)
            GO TO 230
         ELSE IF(DCOEF.EQ.13) THEN
            BDIST(J,K1,K2)=BDIST(J,K1,K2)+(DJEDW*DJEDW)
            GO TO 230
         ENDIF

230            CONTINUE
            ENDIF
210      CONTINUE

200   CONTINUE

*  Distances randomly sampled and summed across loci:
      IF(NBOOT.EQ.0) NBOOT=1
      DO 290 IBOOT=1,NBOOT
      DO 350 K1=2,NOTU
         DO 360 K2=1,K1-1
            SDIST(K1,K2)=0.
            ADIST(K1,K2)=0.
360      CONTINUE
350   CONTINUE

      IF(NBOOT.GT.1)THEN
      RINT=100000/NLOC
      DO 300 L=1,NLOC
         RJ=RANDS(SEED)/RINT
         J=RJ+1
         IF(J.GT.NLOC)J=NLOC
      DO 310 K1=2,NOTU
         DO 320 K2=1,K1-1
            SDIST(K1,K2)=SDIST(K1,K2)+BDIST(J,K1,K2)
320      CONTINUE
310    CONTINUE
300   CONTINUE
      ENDIF

      IF(NBOOT.LE.1)THEN
      DO 800 L=1,NLOC
      DO 810 K1=2,NOTU
         DO 820 K2=1,K1-1
            SDIST(K1,K2)=SDIST(K1,K2)+BDIST(L,K1,K2)
820      CONTINUE
810    CONTINUE
800   CONTINUE
      ENDIF

*  Distances averaged across loci:

      DO 330 K1=2,NOTU
         DO 340 K2=1,K1-1
            IF(DCOEF.EQ.9) THEN
              ADIST(K1,K2)=SQRT(SDIST(K1,K2)/NLOC)
              ADIST(K2,K1)=ADIST(K1,K2)
              GO TO 340
            ELSE IF(DCOEF.GE.11) THEN
              ADIST(K1,K2)=SQRT(SDIST(K1,K2)/NLOC)
              ADIST(K2,K1)=ADIST(K1,K2)
              GO TO 340
            ELSE
              ADIST(K1,K2)=SDIST(K1,K2)/NLOC
              ADIST(K2,K1)=ADIST(K1,K2)
              GO TO 340
            ENDIF
340      CONTINUE
330   CONTINUE

      IF (DCOEF.EQ.7) THEN
         DO 400 K1=2,NOTU
            DO 410 K2=1,K1-1
               ADIST(K1,K2)=1.-ADIST(K1,K2)
               ADIST(K2,K1)=ADIST(K1,K2)
410         CONTINUE
400      CONTINUE
      ENDIF

      IF(DCOEF.GE.7) THEN
*      WRITE(6,611)NOTU
      WRITE(8,611)NOTU
      DO 500 POP1=1,NOTU
*      WRITE(6,610)POPNUM(POP1),(ADIST(POP1,POP2),POP2=1,NOTU)
      WRITE(8,610)POPNUM(POP1),(ADIST(POP1,POP2),POP2=1,NOTU)
500   CONTINUE
      ENDIF

      IF(DCOEF.LE.6) THEN
         DO 510 K1=2,NOTU
            GX1=GX(K1)/NLOC
            JX1=JX(K1)/NLOC
            DO 520 K2=1,K1-1
               JXY1=ADIST(K1,K2)
               GY1=GX(K2)/NLOC
               JY1=JX(K2)/NLOC

              IF(DCOEF.LE.2) THEN
                ADIST(K1,K2)=JXY1/SQRT(GX1*GY1)
                ADIST(K2,K1)=ADIST(K1,K2)
                IF (ADIST(K1,K2).GT.1.) THEN
                    ADIST(K1,K2)=1.
                    ADIST(K2,K1)=ADIST(K1,K2)
                ENDIF
                IF(DCOEF.EQ.1) GO TO 520

                 IF (ADIST(K1,K2).LE.0.) THEN
                    ADIST(K1,K2)=99.E9
                    ADIST(K2,K1)=ADIST(K1,K2)
                 ELSE
                    ADIST(K1,K2)=-ALOG(ADIST(K1,K2))
                    ADIST(K2,K1)=ADIST(K1,K2)
                 ENDIF
               GO TO 520
              ENDIF

               IF(DCOEF.LE.4) THEN
                   ADIST(K1,K2)=JXY1/SQRT(JX1*JY1)
                   ADIST(K2,K1)=ADIST(K1,K2)

                   IF(DCOEF.EQ.3) GO TO 520

                IF (ADIST(K1,K2).LE.0.) THEN
                   ADIST(K1,K2)=99.E9
                   ADIST(K2,K1)=ADIST(K1,K2)
                   GO TO 520
                ELSE
                   ADIST(K1,K2)=-ALOG(ADIST(K1,K2))
                   ADIST(K2,K1)=ADIST(K1,K2)
                   GO TO 520
                ENDIF
               ENDIF

               IF(DCOEF.EQ.5) THEN
                   ADIST(K1,K2)=(GX1+GY1)/2. - JXY1
                   ADIST(K2,K1)=ADIST(K1,K2)
                IF (SDIST(K1,K2).LT.0.) THEN
                   ADIST(K1,K2)=0.
                   ADIST(K2,K1)=ADIST(K1,K2)
                   GO TO 520
                ENDIF
                GO TO 520
               ENDIF

               IF(DCOEF.EQ.6) THEN
                ADIST(K1,K2)=(JX1+JY1)/2. - JXY1
                ADIST(K2,K1)=ADIST(K1,K2)
                GO TO 520
               ENDIF

520         CONTINUE
510      CONTINUE

*      WRITE(6,611)NOTU
      WRITE(8,611)NOTU
      DO 580 POP1=1,NOTU
*      WRITE(6,610)POPNUM(POP1),(ADIST(POP1,POP2),POP2=1,NOTU)
      WRITE(8,610)POPNUM(POP1),(ADIST(POP1,POP2),POP2=1,NOTU)
 580  CONTINUE
      ENDIF

      IF(NBOOT.LE.1)THEN
      WRITE (6,602) COEF(DCOEF),ONAME
      RETURN
      ENDIF

  290 CONTINUE

      WRITE (6,601) COEF(DCOEF),NBOOT,ONAME
      RETURN

600   FORMAT (//////T3,76('*')/T3,'*',T78,'*'/T3,'*',2X,A72,
     + '*'/T3,'*',T78,'*'/T3,
     +'*  Bootstrapping of Genetic similarity and distance', 
     +' coefficients',
     +T78,'*'/T3,'*',T78,'*'/T3,'*  BIOSYS-1   Release 1.7',
     +3X,A12,3X,A12,T78,'*'/T3,'*',T78,'*'/T3,76('*')///)
601   FORMAT (/' Matrices calculated and stored for further ',
     +'analysis by PHYLIP3.5C:',//,A64,//,I10,
     +' pseudo-replicated matrices were written to file: ', A64,
     +////////)
602   FORMAT (/' Matrix calculated and stored for further ',
     +'analysis by PHYLIP3.5C:',//,A64,//,
     +' Matrix written to file: ', A64,////////)
610   FORMAT(A4,6X,1000F8.4)
611   FORMAT(I5)  
      END

C     Pseudorandom number generator 
      FUNCTION RANDS(SEED)
      REAL*8 SEED,GENA,GENB
      GENA=(5.0**13.0)*SEED
      GENB=2.0**35.0
      RND=MOD(GENA,GENB)
      RND=RND/(2.0**35)
      RAND=AINT(RND*100000)
      SEED=RAND
      RETURN
      END
