*=======================================================================*
*                                                                       *
*                      S T E P   C L U S T E R                          *
*                                                                       *
*             Cluster analysis using average, complete,                 *
*                   and single linkage algorithms                       *
*                                                                       *
*  Options:                                                             *
*     NCOEF  ("COEF")   = similarity or distance coefficient to be used (
*                            (see routine simdis)                       *
*     NALGOR ("ALGOR")  = clustering algorithm                          *
*                            = 1  UPGMA (default)                       *
*                            = 2  WPGMA                                 *
*                            = 3  single linkage                        *
*                            = 4  complete linkage                      *
*     COPHEN            : print cophenetic matrix                       *
*     NPLASY ("PLASOUT"): print homoplasy matrix                        *
*     PLOT              : plot phenogram on calcomp or zeta plotter     *
*     YLIM              = minimum value (high-clustering) or maximum    *
*                         value (low clustering) of y-axis for plotted  *
*                         phenogram (default is automatic scaling,      *
*                         explicit setting of ylim overrides automatic  *
*                         scaling)                                      *
*                                                                       *
*                                                                       *
*  Subprograms required by step CLUSTER: CLUSTR  READER  COPHMT  TRPRIN *
*                                        TRPLOT  MATPRT  FIT            *
*                                                                       *
*=======================================================================*

      SUBROUTINE CLUSTR
     +   (POPNAM,S,COPH,HPLASY,WT,DELETD,NN,ITAX,
     +    INAME,CD,CUSE,TERMB,IVECT,ICLINE,ICBEG,ICEND,
     +    NPOS,NCENT,NLINE,NORDER,LINKED,MHLFMT,MT)

*  Performs average linkage (weighted or unweighted), single linkage,
*  or complete linkage cluster analysis

      CHARACTER LINE*80,DATE*12,TIME*12,TITLE*72,FMT*80,
     +   INTST*64,LOGST*8,REALST*8,COEF*48,POPNAM*16
      REAL S,COPH,HPLASY,CD,CUSE,TERMB,WT,
     +   REALVL,YLIM
      INTEGER DELETD,NN,ITAX,IVECT,ICLINE,ICBEG,ICEND,NPOS,NCENT,
     +   NLINE,NORDER,LINKED,INAME,INTVL
      LOGICAL COPHEN,NPLASY,PLOT,LOGVL,CRT
      COMMON/GBPARM/NOTU,NOTU2,NLOC,NALL,NLET,CRT,NDATYP
      COMMON/MAT/MATFIL(13),NOCOEF,NMAT,NSING
      COMMON/COEFNM/COEF(13)
      COMMON/CHARV/LINE,DATE,TIME,TITLE,FMT
      DIMENSION POPNAM(MT),INTST(2),LOGST(4),S(MT,MT),COPH(MHLFMT),
     +   HPLASY(MT,MT),CD(MT),CUSE(MT),TERMB(MT),WT(MT),
     +   DELETD(MT),NN(MT),ITAX(MT,2),IVECT(101),ICLINE(MT,2),
     +   ICBEG(MT,2),ICEND(MT,2),NPOS(MT),NCENT(MT),NLINE(MT),
     +   NORDER(MT),LINKED(MT),INAME(MT),
     +   REALST(1),INTVL(2),LOGVL(4),REALVL(1)
      LOGICAL FULSKP
      COMMON/PAGES/FULSKP

      DATA INTST/'COEF','ALGOR'/,REALST/'YLIM'/,
     +     LOGST/'COPHEN','PLASOUT','PLOT','END'/

      CALL PAGSKP
      WRITE (6,200) TITLE,DATE,TIME

      IF (NOCOEF .EQ. 1) THEN
         WRITE (6,300)
         RETURN
      ENDIF
  1   CONTINUE

*  Defaults:

      INTVL(1)=0
      INTVL(2)=1
      REALVL(1)=0.
      LOGVL(1)=.FALSE.
      LOGVL(2)=.FALSE.
      LOGVL(3)=.FALSE.
      LOGVL(4)=.FALSE.

      CALL READER (2,1,4,INTST,REALST,LOGST,INTVL,REALVL,LOGVL,IERROR)
      IF (IERROR.EQ.1) GO TO 1
      IF (LOGVL(4)) RETURN
      NCOEF=INTVL(1)
      NALGOR=INTVL(2)
      YLIM=REALVL(1)
      COPHEN=LOGVL(1)
      NPLASY=LOGVL(2)
      PLOT=LOGVL(3)

      IF (NCOEF.LE.0 .OR. NCOEF.GE.14) THEN
         WRITE (6,301) INTST(1),NCOEF
         IERROR=1
      ENDIF
      IF (NALGOR.LE.0 .OR. NALGOR.GE.5) THEN
         WRITE (6,301) INTST(2),NALGOR
         IERROR=1
      ENDIF
      IF (YLIM.LT.0.) THEN
         WRITE (6,302) YLIM
         IERROR=1
      ENDIF
      IF (IERROR.EQ.1) GO TO 1

      CALL PAGSKP
      IF (NALGOR.EQ.1) THEN
         WRITE (6,201)
      ELSE IF (NALGOR.EQ.2) THEN
         WRITE (6,202)
      ELSE IF (NALGOR.EQ.3) THEN
         WRITE (6,203)
      ELSE IF (NALGOR.EQ.4) THEN
         WRITE (6,204)
      ENDIF

*  Get input matrix:

      REWIND 15
      DO 500 J=1,NMAT
         IF (MATFIL(J).EQ.NCOEF) GO TO 10
            READ (15)
500   CONTINUE
      WRITE (6,303) NCOEF,COEF(NCOEF)
      GO TO 1
 10   READ (15) ((S(I,J),I=J,NOTU),J=1,NOTU)

      DO 501 J=1,NOTU
         DO 502 I=J,NOTU
            S(J,I)=S(I,J)
502      CONTINUE
501   CONTINUE
*
      IF (NCOEF.NE.1 .AND. NCOEF.NE.3 .AND. NCOEF.NE.7) THEN
         NH= -1
      ELSE
         NH=1
      ENDIF
      WRITE (6,205) COEF(NCOEF)

      DO 503 I=1,NOTU
         DELETD(I)=0
         NN(I)=I
         WT(I)=1.
503   CONTINUE
      NPHOLD=1
      ITER=0
      NPAIR=0

*  Find mutually most similar (least dissimilar) clusters:

 13   ITER=ITER+1
      DO 504 I=1,NOTU
         IF (DELETD(I).NE.1) THEN
            SMAX=-99.E9
            DO 505 J=1,NOTU
               IF (DELETD(J).NE.1 .AND. I.NE.J) THEN
                  SIJ=S(I,J)*NH
                  IF (SIJ.GT.SMAX) THEN
                     NN(I)=J
                     SMAX=SIJ
                  ENDIF
               ENDIF
505         CONTINUE
         ENDIF
504   CONTINUE
      DO 506 I=1,NOTU
         IF (DELETD(I).LE.0) THEN
            IF (NN(NN(I)) .EQ. I) THEN
               J=NN(I)
               NPAIR=NPAIR+1
               ITAX(NPAIR,1)=I
               ITAX(NPAIR,2)=J
               CD(NPAIR)=S(I,J)
               DELETD(J)=2
               WRITE (6,206) (ITAX(NPAIR,K),K=1,2),CD(NPAIR),ITER
            ENDIF
         ENDIF
506   CONTINUE

*  Recompute distances between clusters:

      DO 507 K=NPHOLD,NPAIR
         K1=ITAX(K,1)
         K2=ITAX(K,2)
         WTNEW=WT(K1)+WT(K2)
         DO 508 I=1,NOTU
            IF (DELETD(I).NE.1 .AND. I.NE.K1 .AND. I.NE.K2) THEN
               IF (NALGOR.EQ.1) THEN
*                 UPGMA:
                  S(I,K1)=(WT(K1)*S(I,K1)+WT(K2)*S(I,K2))/WTNEW
               ELSE IF (NALGOR.EQ.2) THEN
*                 WPGMA:
                  S(I,K1)=(S(I,K1)+S(I,K2))/2.
               ELSE IF ((NALGOR.EQ.3) .OR. (NALGOR.EQ.4)) THEN
                  NTEST=NALGOR+NH
                  IF (NTEST.NE.3 .AND. NTEST.NE.4) THEN
*                    Single linkage (or complete linkage-high):
                     S(I,K1)=MIN(S(I,K1),S(I,K2))
                  ELSE
*                    Complete linkage (or single linkage-high):
                     S(I,K1)=AMAX1(S(I,K1),S(I,K2))
                  ENDIF
               ENDIF
               S(K1,I)=S(I,K1)
               DELETD(K2)=1
            ENDIF
508      CONTINUE
         WT(K1)=WTNEW
507   CONTINUE

      NPHOLD=NPAIR+1
      IF (NPHOLD.LT.NOTU) GO TO 13
      WRITE (6,207)
      CALL COPHMT (POPNAM,S,COPH,HPLASY,CD,ITAX,INAME,
     +   COPHEN,NPLASY,NCOEF,MHLFMT,MT)
      CALL TRPRIN (NH,0,PATHLF,PDSUM,LINKED,POPNAM,TERMB,ITAX,
     +   CD,CUSE,IVECT,ICLINE,ICBEG,ICEND,NPOS,NCENT,
     +   NLINE,NORDER,MT)
      IF (PLOT) WRITE (6,*)
     +   'NOTE: This version of BIOSYS does not support plotting'
      GO TO 1
*
200   FORMAT (//////T3,76('*')/T3,'*',T78,'*'/T3,'*',2X,A72,
     +'*'/T3,'*',T78,'*'/T3,
     +'*  Cluster Analysis',
     +T78,'*'/T3,'*',T78,'*'/T3,'*  BIOSYS-1   Release 1.7',3X,A12,3X,
     +A12,T78,'*'/T3,'*',T78,'*'/T3,76('*')//////)
201   FORMAT (////' Cluster analysis using unweighted pair group ',
     +'method'/1X,51('*')//)
202   FORMAT (////' Cluster analysis using weighted pair group method'/
     +1X,49('*')//)
203   FORMAT (////' Cluster analysis using single linkage method'/
     +1X,44('*')//)
204   FORMAT (////' Cluster analysis using complete linkage method'/
     +1X,46('*')//)
205   FORMAT (5X,'Coefficient used: ',A48//T6,54('-')/T6,
     +'Population or cluster',T35,'Clustering'/T10,'numbers joined',
     +T38,'level',T55,'Cycle'/T6,54('-'))
206   FORMAT (9X,I3,T20,I3,T33,F10.5,T55,I3)
207   FORMAT (T6,54('-')////)
300   FORMAT (' ERROR: Step SIMDIS has not been performed')
301   FORMAT (' ERROR: Illegal value for ',A5,':',I5/)
302   FORMAT (' ERROR: Illegal value for YLIM:',F10.3/)
303   FORMAT (////' ERROR: Matrix no.',I3,' = ',A48/8X,
     +'not on file.')
      END


      SUBROUTINE COPHMT
     +   (POPNAM,S,COPH,HPLASY,CD,ITAX,INAME,
     +    COPHEN,NPLASY,NCOEF,MHLFMT,MT)

*  Prepares cophenetic matrix from clustering data and calls printing
*  routine

      CHARACTER COEF*48,POPNAM*16
      REAL CD,S,COPH,HPLASY
      INTEGER ITAX,INAME
      LOGICAL COPHEN,NPLASY,CRT
      COMMON/GBPARM/NOTU,NOTU2,NLOC,NALL,NLET,CRT,NDATYP
      COMMON/MAT/MATFIL(13),NOCOEF,NMAT,NSING
      COMMON/COEFNM/COEF(13)
      DIMENSION POPNAM(MT),CD(MT),S(MT,MT),COPH(MHLFMT),
     +   HPLASY(MT,MT),ITAX(MT,2),INAME(MT)

      NTM1=NOTU-1
      DO 500 I=1,NOTU
         INAME(I)=I
500   CONTINUE

      DO 501 I=1,NTM1
         K1=ITAX(I,1)
         K2=ITAX(I,2)
  1      COPH(MDEX(K1,K2))=CD(I)
         IF (INAME(K1).NE.K1) THEN
            K1=INAME(K1)
            GO TO 1
         ELSE
            IF (INAME(K2).NE.K2) THEN
               K2=INAME(K2)
               K1=ITAX(I,1)
               GO TO 1
            ENDIF
         ENDIF
         INAME(K1)=ITAX(I,2)
501   CONTINUE
      IF (COPHEN) THEN
         DIAG=0.
         IF (NCOEF.EQ.1 .OR. NCOEF.EQ.3 .OR. NCOEF.EQ.7) DIAG=1.
*        Copy COPH to HPLASY (for printing of cophenetic matrix only)
         DO 502 I=1,NOTU
            DO 503 J=1,I
               HPLASY(J,I)=COPH(MDEX(I,J))
503         CONTINUE
            HPLASY(I,I)=DIAG
502      CONTINUE
         CALL MATPRT (3,0,0,'DUMMY   ',HPLASY,POPNAM,2,MT)
      ENDIF

*  Recover original coefficients for goodness of fit statistics:

      REWIND 15
      DO 504 J=1,NMAT
         IF (MATFIL(J).EQ.NCOEF) GO TO 5
            READ (15)
504   CONTINUE
  5   READ (15) ((S(I,J),I=J,NOTU),J=1,NOTU)

      CALL FIT (COPH,S,HPLASY,POPNAM,NPLASY,MHLFMT,MT)
      RETURN
      END
