*=======================================================================*
*                                                                       *
*                       S T E P   D A T A                               *
*                                                                       *
*                                                                       *
*    Data input and calculation of allele and genotype frequencies      *
*                                                                       *
*  Options and parameters:                                              *
*     DATYP         = 1  single individual genotype input               *
*                   = 2  genotype frequency input                       *
*                   = 3  allele frequency input                         *
*     NCOL          = maximum number of genotypic combinations          *
*                        permitted at a locus if DATYP = 2              *
*                   = maximum number of alleles observed in any single  *
*                        population if DATYP = 3                        *
*     ALPHA         : alphabetic allelic designations                   *
*     NUMER         : numeric allelic designations (default)            *
*     CHKSUM        : checks allele frequencies for additivity to 1     *
*                        (valid only if DATYP=3)                        *
*     RNCOND        : abort run if checksum detects non-additivity of   *
*                        allele frequencies (valid only if CHKSUM is    *
*                        in effect)                                     *
*     NOPRIN        : suppress printing of allele frequency matrix      *
*     NOCORR        : unless NOCORR is specified, input frequencies     *
*                        will be corrected for rounding error by        *
*                        multiplying by 2n, rounding to nearest         *
*                        integer, and dividing by 2n.  (E.g., 0.67 is   *
*                        converted to 0.66666... in memory.)  NOCORR    *
*                        suppresses this correction.                    *
*                        (parameter ignored unless DATYP=3)             *
*                                                                       *
*  Subprograms required by step DATA:  DATIN READER DATERR SWITCH       *
*                                      LETNUM                           *
*=======================================================================*

      SUBROUTINE DATIN
     +   (POPNAM,LOCNAM,NOBS,FREQ,TFREQ,MINA,MAXA,NSMPL,MINI,MAXI,N,
     +    FREQ2,MINI2,MAXI2,N2,FRQHOM,MAXIT,NG1,NG2,POPNUM,NA,Q,MG1,
     +    MG2,NIND,KHIER,IGRP,MA,MA2,ML,MT,MCOM,CG1,CG2,CMG1,CMG2,
     +    CNA,DARECL)

*  Reads input data in form of single-individual genotypes, genotype
*  frequencies, or allele frequencies, and sets up arrays used in
*  all subsequent steps.

      CHARACTER LINE*80,DATE*12,TIME*12,TITLE*72,FMT*80,CG1,CG2,
     +   LETTER,POPNAM*16,POPNUM*4,CMG1,CMG2,CNA,INTST*64,
     +   REALST*8,LOGST*8,LOCNAM*5,NUM*4
      REAL FREQ,FREQ2,FRQHOM,Q,TFREQ,ZERO
      INTEGER NOBS,MINI,MAXI,N,MAXIT,MINA,MAXA,NSMPL,MINI2,MAXI2,N2,
     +   NG1,NG2,NA,MG1,MG2,NIND,KHIER,IGRP,DATYP,DARECL
      LOGICAL ALPHA,NUMER,CHKSUM,RNCOND,NOPRIN,NORUN,NOCORR,CRT,LOGVL
      PARAMETER (ZERO=1.E-6)
      COMMON/GBPARM/NOTU,NOTU2,NLOC,NALL,NLET,CRT,DATYP
      COMMON/SYMB/LETTER(26)
      COMMON/CHARV/LINE,DATE,TIME,TITLE,FMT
      DIMENSION CG1(ML),CG2(ML),POPNAM(MT),POPNUM(MT),INTST(2),LOGST(6),
     +   REALST(1),LOCNAM(ML),FREQ(MA,ML),FREQ2(MA,MT),FRQHOM(MA,MT),
     +   TFREQ(MA2),Q(MA),NOBS(MA,MA,ML),MINI(ML),MAXI(ML),N(ML),
     +   MINI2(MT),MAXI2(MT),N2(MT),NG1(ML),NG2(ML),NA(MA),MG1(MCOM),
     +   MINA(ML,MT),MAXA(ML,MT),NSMPL(ML,MT),MG2(MCOM),NIND(MCOM),
     +   KHIER(MT,3),IGRP(MT),CMG1(MCOM),CMG2(MCOM),CNA(MA),MAXIT(ML),
     +   INTVL(2),LOGVL(6),REALVL(1)
      LOGICAL FULSKP
      COMMON/PAGES/FULSKP
      SAVE INTST,REALST,LOGST

      DATA INTST/'DATYP','NCOL'/,REALST/'ZZZ'/,
     +   LOGST/'CHECKSUM','RUNCOND','PRINT','ALPHA','NUMER','CORR'/

      CALL PAGSKP
      WRITE (6,200) TITLE,DATE,TIME
      IREC=0
      NORUN=.FALSE.
      K=0

*  Defaults:

      INTVL(1)=0
      INTVL(2)=0
      LOGVL(1)=.FALSE.
      LOGVL(2)=.FALSE.
      LOGVL(3)=.TRUE.
      LOGVL(4)=.FALSE.
      LOGVL(5)=.FALSE.
      LOGVL(6)=.TRUE.

      CALL READER (2,0,6,INTST,REALST,LOGST,INTVL,REALVL,LOGVL,IERROR)
      IF (IERROR.EQ.1) STOP
      DATYP=INTVL(1)
      NCOL=INTVL(2)
      CHKSUM=LOGVL(1)
      RNCOND=LOGVL(2)
      NOPRIN=.NOT.LOGVL(3)
      ALPHA=LOGVL(4)
      NUMER=.NOT.ALPHA
      NOCORR=.NOT.LOGVL(6)
      NLET=0
      IF (ALPHA) NLET=1

*  Error check:

      IERROR=0
      IF (DATYP.LE.0 .OR. DATYP.GE.4) THEN
         WRITE (6,300) INTST(1),DATYP
         IERROR=1
      ENDIF
      IF (NCOL.GT.MCOM) THEN
         WRITE (6,301)
         IERROR=1
      ENDIF
      IF (DATYP.NE.1 .AND. NCOL.LE.0.) THEN
         WRITE (6,302)
         IERROR=1
      ENDIF
      IF (IERROR.EQ.1) STOP

      OPEN (11,ACCESS='DIRECT',STATUS='SCRATCH',RECL=DARECL,
     +   FORM='UNFORMATTED')

      REWIND 17
      IF (DATYP .NE. 3) REWIND 18

      IF (DATYP.EQ.1) THEN
         WRITE (6,201)
      ELSE IF (DATYP.EQ.2) THEN
         WRITE (6,202)
      ELSE IF (DATYP.EQ.3) THEN
         WRITE (6,203)
         IF (NOCORR) WRITE (6,204)
      ENDIF
      IF (ALPHA) THEN
         WRITE (6,206)
      ELSE
         WRITE (6,205)
      ENDIF
      IF (CHKSUM) THEN
         WRITE (6,207)
         IF (RNCOND) WRITE (6,208)
      ENDIF
      IF (NOPRIN) WRITE (6,209)
      WRITE (6,210)

      READ (5,100) FMT

*  Array initialization:

      DO 500 J=1,NLOC
         MINI(J)=1
         MAXI(J)=NALL
         MAXIT(J)=0
500   CONTINUE

      DO 501 I=1,NALL
         TFREQ(I)=0.
         TFREQ(I+NALL)=0.
501   CONTINUE

 11   CONTINUE

      DO 502 J=1,NLOC
         MINIJ=MINI(J)
         MAXIJ=MAXI(J)
         MINI(J)=999
         MAXI(J)=0
         N(J)=0
         DO 503 I1=MINIJ,MAXIJ
            FREQ(I1,J)=0.
            DO  504 I2=MINIJ,I1
               NOBS(I1,I2,J)=0
               NOBS(I2,I1,J)=0
504         CONTINUE
503      CONTINUE
502   CONTINUE

*  New population data:

      K=K+1
      READ (5,101) POPNUM(K),POPNAM(K),(KHIER(K,NH),NH=1,3),IGRP(K)
      IF (DATYP.LE.2) THEN
         IF (DATYP.EQ.1) THEN

*        Single-individual genotype input:

 12         CONTINUE
            IF (NUMER) THEN
               READ (5,FMT,ERR=30) NUM,(NG1(J),NG2(J),J=1,NLOC)
            ELSE
               READ (5,FMT,ERR=30) NUM,(CG1(J),CG2(J),J=1,NLOC)
            ENDIF
            IF (NUM.NE.'NEXT') THEN
               DO 505 J=1,NLOC
                  IF (NUMER) THEN
                     I1=NG1(J)
                     IF (I1.GT.NALL) CALL DATERR (2,I1,J,K,POPNAM,
     +                                            LOCNAM,ML,MT)
                     I2=NG2(J)
                     IF (I2.GT.NALL) CALL DATERR (2,I2,J,K,POPNAM,
     +                                            LOCNAM,ML,MT)
                  ELSE
                     I1=LETNUM(CG1(J),IERR)
                     IF (IERR.GT.0) CALL DATERR (IERR,I1,J,K,POPNAM,
     +                                           LOCNAM,ML,MT)
                     I2=LETNUM(CG2(J),IERR)
                     IF (IERR.GT.0) CALL DATERR (IERR,I2,J,K,POPNAM,
     +                                           LOCNAM,ML,MT)
                  ENDIF
                  CALL SWITCH (I1,I2,I2,I1)
                  IF (I1.NE.0) THEN
                     N(J)=N(J)+1
                     NOBS(I1,I2,J)=NOBS(I1,I2,J)+1
                     NOBS(I2,I1,J)=NOBS(I1,I2,J)
                     IF (I1.LT.MINI(J)) MINI(J)=I1
                     IF (I2.GT.MAXI(J)) THEN
                        MAXI(J)=I2
                        IF (I2.GT.MAXIT(J)) MAXIT(J)=I2
                     ENDIF
                  ENDIF
505            CONTINUE
               GO TO 12
            ENDIF
         ELSE

*     Genotype frequency input:

            DO 506 J=1,NLOC
               IF (NUMER) THEN
                  READ (5,FMT,ERR=30) (MG1(NC),MG2(NC),NIND(NC),
     +               NC=1,NCOL)
               ELSE
                  READ (5,FMT,ERR=30) (CMG1(NC),CMG2(NC),NIND(NC),
     +               NC=1,NCOL)
               ENDIF
               DO 507 NC=1,NCOL
                  IF (NUMER) THEN
                     I1=MG1(NC)
                     IF (I1.GT.NALL) CALL DATERR (2,I1,J,K,POPNAM,
     +                                            LOCNAM,ML,MT)
                     I2=MG2(NC)
                     IF (I2.GT.NALL) CALL DATERR (2,I2,J,K,POPNAM,
     +                                            LOCNAM,ML,MT)
                  ELSE
                     I1=LETNUM(CMG1(NC),IERR)
                     IF (IERR.GT.0) CALL DATERR (IERR,I1,J,K,POPNAM,
     +                                           LOCNAM,ML,MT)
                     I2=LETNUM(CMG2(NC),IERR)
                     IF (IERR.GT.0) CALL DATERR (IERR,I2,J,K,POPNAM,
     +                                           LOCNAM,ML,MT)
                  ENDIF
                  CALL SWITCH (I1,I2,I2,I1)
                  IF (I1.NE.0) THEN
                     N(J)=N(J)+NIND(NC)
                     NOBS(I1,I2,J)=NIND(NC)
                     NOBS(I2,I1,J)=NIND(NC)
                     IF (I1.LT.MINI(J)) MINI(J)=I1
                     IF (I2.GT.MAXI(J)) THEN
                        MAXI(J)=I2
                        IF (I2.GT.MAXIT(J)) MAXIT(J)=I2
                     ENDIF
                  ENDIF
507            CONTINUE
506         CONTINUE
         ENDIF

*     Calculate allele frequencies:

         DO 508 J=1,NLOC
            MINIJ=MINI(J)
            MAXIJ=MAXI(J)
            NJ=N(J)
            TWONJ=2*NJ

            IF (NJ.LE.0) THEN
               WRITE (6,303) J,LOCNAM(J),POPNAM(K),POPNUM(K)
               STOP
            ENDIF
            DO 509 I1=MINIJ,MAXIJ
               NSUM=0
               DO 510 I2=MINIJ,MAXIJ
                  NSUM=NSUM+NOBS(I1,I2,J)
                  IF (I1.EQ.I2) NSUM=NSUM+NOBS(I1,I2,J)
510            CONTINUE
               QFREQ=FLOAT(NSUM)/TWONJ
               FREQ(I1,J)=QFREQ
               TFREQ(I1)=QFREQ
               TFREQ(I1+NALL)=FLOAT(NOBS(I1,I1,J))/NJ
509         CONTINUE
            MINA(J,K)=MINIJ
            MAXA(J,K)=MAXIJ
            NSMPL(J,K)=NJ
            IREC=IREC+1
            WRITE (11,REC=IREC) TFREQ
            DO 511 I=MINIJ,MAXIJ
               TFREQ(I)=0.
               TFREQ(I+NALL)=0.
511         CONTINUE
508      CONTINUE
      ELSE

*        Allele frequency input:

         DO 512 J=1,NLOC
            IF (NUMER) THEN
               READ (5,FMT,ERR=30) N(J),(NA(NC),Q(NC),NC=1,NCOL)
            ELSE
               READ (5,FMT,ERR=30) N(J),(CNA(NC),Q(NC),NC=1,NCOL)
            ENDIF
            MINIJ=999
            MAXIJ=0
            DO 513 NC=1,NCOL
               IF (ALPHA) THEN
                  I=LETNUM(CNA(NC),IERR)
                  IF (IERR.GT.0) CALL DATERR (IERR,I,J,K,POPNAM,
     +                                        LOCNAM,ML,MT)
               ELSE
                  I=NA(NC)
                  IF (I.GT.NALL) CALL DATERR (2,I,J,K,POPNAM,
     +                                        LOCNAM,ML,MT)
               ENDIF
               IF (I.NE.0) THEN
                  IF (NOCORR) THEN
                     QFREQ=Q(NC)
                  ELSE
*                    ( correct for rounding error )
                     TWONJ=2*N(J)
                     QFREQ=NINT(Q(NC)*TWONJ)/TWONJ
                  ENDIF
                  TFREQ(I)=QFREQ
                  FREQ(I,J)=QFREQ
                  IF (I.LT.MINIJ) MINIJ=I
                  IF (I.GT.MAXIJ) THEN
                     MAXIJ=I
                     IF (I.GT.MAXIT(J)) MAXIT(J)=I
                  ENDIF
               ENDIF
513         CONTINUE

*           Check to make sure allele frequencies sum to 1:

            IF (CHKSUM) THEN
               SUM=0.
               DO 514 I=1,NALL
                  SUM=SUM+TFREQ(I)
514            CONTINUE
               IF (ABS(SUM-1.).GT.ZERO) THEN
                  WRITE (6,304) LOCNAM(J),POPNAM(K),POPNUM(K)
                  NORUN=.TRUE.
               ENDIF
            ENDIF
            MINI(J)=MINIJ
            MAXI(J)=MAXIJ
            MINA(J,K)=MINIJ
            MAXA(J,K)=MAXIJ
            NSMPL(J,K)=N(J)
            IREC=IREC+1
            WRITE (11,REC=IREC) TFREQ
            DO 515 I=MINIJ,MAXIJ
               TFREQ(I)=0.
515         CONTINUE
512      CONTINUE
      ENDIF

*  Save arrays before beginning next set of population data:

      WRITE (17) FREQ,MINI,MAXI,N
      IF (DATYP .NE. 3) WRITE (18) NOBS
      IF (K.LT.NOTU) GO TO 11
      REWIND 14
      WRITE (14) KHIER

*  Write FREQA, MINA, MAXA, and NSMPL arrays to file locus-wise:

      REWIND 16
      DO 516 J=1,NLOC
         DO 517 K=1,NOTU
            KREC=(K-1)*NLOC+J
            READ (11,REC=KREC) TFREQ
            DO 518 I=1,NALL
               FREQ2(I,K)=TFREQ(I)
               IF (DATYP.LE.2) FRQHOM(I,K)=TFREQ(I+NALL)
518         CONTINUE
517      CONTINUE
         IF (DATYP.NE.3) THEN
            WRITE (16) FREQ2,(MINA(J,K),K=1,NOTU),
     +         (MAXA(J,K),K=1,NOTU),(NSMPL(J,K),K=1,NOTU),FRQHOM
         ELSE
            WRITE (16) FREQ2,(MINA(J,K),K=1,NOTU),
     +         (MAXA(J,K),K=1,NOTU),(NSMPL(J,K),K=1,NOTU)
         ENDIF
516   CONTINUE
      IF (NOPRIN) RETURN

*  Print allele frequency matrix:

      K1=1
      NCOL=15
      IF (CRT) NCOL=9
 28   K2=MIN(K1+NCOL-1,NOTU)
      NDIFF=NOTU-K2
      IF (NDIFF .EQ. 1) THEN
         K2=K2-1
         NCOL=NCOL-1
      ENDIF
      CALL PAGSKP
      WRITE (6,211) K1,K2
      WRITE (6,212) ('--------',K=K1,K2)
      NCOLS=K2-K1+1
      FMT='(7X,  (4X),''Population'')'
      WRITE (FMT(5:6),'(I2)') NCOLS-1
      WRITE (6,FMT)
      WRITE (6,213) ('--------',K=K1+1,K2)
      WRITE (6,214) (K,K=K1,K2)
      WRITE (6,212) ('--------',K=K1,K2)
      REWIND 16
      DO 519 J=1,NLOC
         READ (16) FREQ2,MINI2,MAXI2,N2
         WRITE (6,215) LOCNAM (J),(N2(K),K=K1,K2)
         MAXITJ=MAXIT(J)
         DO 520 I=1,MAXITJ
            IF (NUMER) THEN
               WRITE (6,217) I,(FREQ2(I,K),K=K1,K2)
            ELSE
               WRITE (6,216) LETTER(I),(FREQ2(I,K),K=K1,K2)
            ENDIF
520      CONTINUE
519   CONTINUE
      WRITE (6,212) ('--------',K=K1,K2)
      WRITE (6,218)
      IF (K2.NE.NOTU) THEN
         K1=K1+NCOL
         GO TO 28
      ENDIF
      WRITE (6,219)
      DO 521 K=1,NOTU
         WRITE (6,220) POPNUM(K),K,POPNAM(K)
521   CONTINUE
      WRITE (6,221)
      IF (RNCOND .AND. NORUN) STOP
      RETURN

 30   WRITE (6,305) K,J
      STOP

100   FORMAT (A80)
101   FORMAT (A4,1X,A16,4X,4I5)
200   FORMAT (//////T3,76('*')/T3,'*',T78,'*'/T3,'*',2X,A72,
     +'*'/T3,'*',T78,'*'/T3,
     +'*  Initial Data Step',
     +T78,'*'/T3,'*',T78,'*'/T3,'*  BIOSYS-1   Release 1.7',3X,A12,3X,
     +A12,T78,'*'/T3,'*',T78,'*'/T3,76('*')//////)
201   FORMAT (' Input data: Single-individual genotypes')
202   FORMAT (' Input data: Genotype frequencies')
203   FORMAT (' Input data: Allele frequencies')
204   FORMAT (/' Correction for rounding error in allele frequencies ',
     +'suppressed')
207   FORMAT (/' Allele frequencies will be checked for additivity to'
     +,' one')
208   FORMAT (' Run will be aborted if checksum finds error(s)')
209   FORMAT (/' Printing of allele frequency matrix suppressed')
210   FORMAT (////)
205   FORMAT (/' Allelic designations: Numeric')
206   FORMAT (/' Allelic designations: Alphabetic')
211   FORMAT (' Allele frequencies in populations',I3,' thru',
     +I3/1X,44('*')///)
212   FORMAT (' ------',15A8)
213   FORMAT (9X,'------',14A8)
214   FORMAT (' Locus',I7,14I8)
215   FORMAT (/1X,A5/2X,'(N)',15I8)
216   FORMAT (3X,A1,2X,15F8.3)
217   FORMAT (1X,I3,2X,15F8.3)
218   FORMAT (//////)
219   FORMAT (' Key to populations'/1X,18('*')//1X,50('-')/
     +' Original    Pop. no. on'/' pop. no.',5X,'printout',10X,
     +'Population name'/1X,50('-'))
220   FORMAT (/2X,A4,I13,T33,A16)
221   FORMAT (1X,50('-'))
300   FORMAT (' ERROR: Illegal value for ',A8,':',I5/)
301   FORMAT (' ERROR: NCOL exceeds program dimensions'/)
302   FORMAT (' ERROR: NCOL not specified'/)
303   FORMAT (//' ERROR: No data for locus',I3,' (',A5,')',
     +' in population ',A16,'(',A4,').'//
     +8X,'execution terminated.')
304   FORMAT (' WARNING: Allele frequencies at locus ',A5,
     +' in population ',A16,'(',A4,')'/10X,'do not sum to one')
305   FORMAT (' ERROR: Invalid input (population',I3,
     +   ', locus',I3,')')
      END


      INTEGER FUNCTION LETNUM (ALLDES,IERR)

*  Returns the ordinal position of an alphabetic character in 'A'..'Z'

      CHARACTER LETTER,ALLDES
      LOGICAL CRT
      COMMON/GBPARM/NOTU,NOTU2,NLOC,NALL,NLET,CRT,DATYP
      COMMON/SYMB/LETTER(26)

      IERR=0
      IF (ALLDES.EQ.' ' .OR. ALLDES.EQ.'0') THEN
         LETNUM=0
         RETURN
      ENDIF

*--------------------------------------------------------------*
*  Comment out code below that does not apply to your machine  *
*--------------------------------------------------------------*

*---------------- Begin EBCDIC (IBM mainframes) ---------------*
C     LETNUM=ICHAR(ALLDES)
C     IF (LETNUM.LT.193 .OR. LETNUM.GT.233) THEN
C        IERR=1
C        WRITE (6,300) ALLDES
C     ELSE IF (LETNUM.LE.201) THEN
C        LETNUM=LETNUM-192
C     ELSE IF (LETNUM.LE.217) THEN
C        LETNUM=LETNUM-199
C     ELSE
C        LETNUM=LETNUM-207
C     ENDIF
*------------------------- End EBCDIC -------------------------*

*------------ Begin CDC default collating sequence ------------*
C     LETNUM=ICHAR(ALLDES)-32
C     IF (LETNUM.LT.1 .OR. LETNUM.GT.26) THEN
C        IERR=1
C        WRITE (6,300) ALLDES
C     ENDIF
*------------- End CDC default collating sequence -------------*

*------------- Begin ASCII (IBM-PC, DEC VAX, etc.) ------------*
      LETNUM=ICHAR(ALLDES)-64
      IF (LETNUM.LT.1 .OR. LETNUM.GT.26) THEN
         IERR=1
         WRITE (6,300) ALLDES
      ENDIF
*----------------------- End ASCII ----------------------------*

*---------------------- Begin PRIME ---------------------------*
C     LETNUM=ICHAR(ALLDES)-192
C     IF (LETNUM.LT.1 .OR. LETNUM.GT.26) THEN
C        IERR=1
C        WRITE (6,300) ALLDES
C     ENDIF
*----------------------- End PRIME ----------------------------*

      IF (LETNUM.GT.NALL) IERR=2
      RETURN
300   FORMAT (' ERROR: Invalid allelic designation encountered: "',
     +A1,'"')
      END


      SUBROUTINE DATERR (IERR,IVAL,J,K,POPNAM,LOCNAM,ML,MT)

*  Prints diagnostic information when invalid allelic designation
*  encountered

      CHARACTER LETTER,POPNAM*16,LOCNAM*5
      COMMON/GBPARM/NOTU,NOTU2,NLOC,NALL,NLET,CRT,DATYP
      COMMON/SYMB/LETTER(26)
      DIMENSION POPNAM(MT),LOCNAM(ML)

      IF (IERR.NE.1) THEN
         IF (NLET.EQ.1) THEN
            WRITE (6,300) LETTER(IVAL)
         ELSE
            WRITE (6,301) IVAL
         ENDIF
      ENDIF
      WRITE (6,302) J,LOCNAM(J),POPNAM(K)
      STOP

300   FORMAT (' ERROR: Allele "',A1,'" greater than nall.')
301   FORMAT (' ERROR: Allele',I5,' greater than nall.')
302   FORMAT (/8X,'Error occurred while attempting to read data for ',
     +'locus',I3,' (',A5,')'/8X,'in population ',A16////)
      END
