*=======================================================================*
*                                                                       *
*                      S T E P   D I S T R I B                          *
*                                                                       *
*      Frequency distributions of single-locus similarity and           *
*      distance coefficients at different levels of divergence          *
*                                                                       *
*  Calculates frequency distribution of single-locus similarity or      *
*  distance coefficients at different levels of evolutionary divergence *
*  and prints resulting histograms, in format similar to that commonly  *
*  used by F.J. Ayala and J.C. Avise.  Requires prior performance of    *
*  step HIERARCHY.                                                      *
*                                                                       *
*  Parameter:                                                           *
*     NCOEF ("COEF") = coefficient to be used (see routine SIMDIS)      *
*                                                                       *
*  Subprograms required by step DISTRIB: AYAGRM READER SIMDIS           *
*                                                                       *
*=======================================================================*

      SUBROUTINE AYAGRM
     +   (DSING,LD,JXY,DROG1,DROG2,DARC,DCHORD,DPREV,DEDW,SROG,IUNB,
     +    DUNB,IDEN,DIST,DMINUN,DMIN,FREQ2,MINI2,MAXI2,N2,JX,GX,SMJX2,
     +    SMGX2,EDJX2,IHIER,HNUM,NDIST,POPNAM,MA,MT)

      CHARACTER LINE*80,DATE*12,TIME*12,TITLE*72,FMT*80,LETTER,
     +   JLINE*61,INTST*64,REALST*8,LOGST*8,COEF*48,POPNAM*16,HNAME*10
      REAL JXY,DSING,DROG1,DROG2,DARC,DCHORD,DPREV,DEDW,SROG,IUNB,
     +   DUNB,IDEN,DIST,DMINUN,DMIN,JX,GX,SMJX2,SMGX2,EDJX2,FREQ2,
     +   YLAB,INTRVL,REALVL
      INTEGER LD,MINI2,MAXI2,N2,IHIER,HNUM,NDIST,LSUM,INTVL
      LOGICAL LOGVL
      COMMON/GBPARM/NOTU,NOTU2,NLOC,NALL,NLET,CRT,NDATYP
      COMMON/MAT/MATFIL(13),NOCOEF,NMAT,NSING
      COMMON/COEFNM/COEF(13)
      COMMON/HIER1/NOHIER,NLEVEL,NROW,NHIER(4)
      COMMON/HIER/HNAME(4)
      COMMON/SYMB/LETTER(26)
      COMMON/CHARV/LINE,DATE,TIME,TITLE,FMT
      DIMENSION POPNAM(MT),INTST(1),REALST(1),LOGST(1),JXY(MT,MT),
     +   DSING(MT,MT),DROG1(MT,MT),DROG2(MT,MT),DARC(MT,MT),
     +   DPREV(MT,MT),DEDW(MT,MT),SROG(MT,MT),IUNB(MT,MT),DUNB(MT,MT),
     +   IDEN(MT,MT),DIST(MT,MT),DMINUN(MT,MT),DMIN(MT,MT),JX(MT),
     +   GX(MT),SMJX2(MT),SMGX2(MT),EDJX2(MT),FREQ2(MA,MT),YLAB(10),
     +   LD(MT,MT),MINI2(MT),MAXI2(MT),N2(MT),IHIER(MT,3),HNUM(MT),
     +   NDIST(20,4),LSUM(4),DCHORD(MT,MT),INTVL(1),REALVL(1),LOGVL(1)
      LOGICAL FULSKP
      COMMON/PAGES/FULSKP
      DATA INTST/'COEF'/,LOGST/'END'/,REALST/'ZZZ'/

      CALL PAGSKP
      WRITE (6,200) TITLE,DATE,TIME
      IF (NOHIER.NE.0) THEN
         WRITE (6,300)
         RETURN
      ENDIF

*  Get hierarchy data:
      REWIND 14
      READ (14) IHIER,HNUM
      NLEVP1=NLEVEL+1

  2   INTVL(1)=0
      LOGVL(1)=.FALSE.
      CALL READER (1,0,1,INTST,REALST,LOGST,INTVL,REALVL,LOGVL,IERROR)
      IF (IERROR.EQ.1) GO TO 2
      IF (LOGVL(1)) RETURN
      NCOEF=INTVL(1)
      IF (NCOEF.LE.0 .OR. NCOEF.GE.14) THEN
         WRITE (6,301) NCOEF
         GO TO 2
      ENDIF

      IF (NCOEF .EQ. 2 .OR. NCOEF .EQ. 4) THEN
         WRITE (6,302)
         GO TO 2
      ENDIF

*  Calculate coefficients if not already on file:

      IF (NCOEF .NE. NSING) CALL SIMDIS (JXY,DSING,DROG1,DROG2,DARC,
     +   DPREV,DCHORD,DEDW,SROG,IUNB,DUNB,IDEN,DIST,DMINUN,DMIN,FREQ2,
     +   MINI2,MAXI2,N2,JX,GX,SMJX2,SMGX2,EDJX2,POPNAM,MA,MT,NCOEF)
      SCALE=1.
      IF (NCOEF.GE.11 .AND. NCOEF.NE.12) THEN
         SCALE=2.*SQRT(2.)/3.141592654
         IF (NCOEF.EQ.13) SCALE=SCALE*2.
      ENDIF
      INTRVL=SCALE/20.
      DO 500 L=1,NLEVP1
         LSUM(L)=0
500   CONTINUE

*  Find "level of divergence" for each pairwise comparison:

      DO 501 K1=2,NOTU
         NH1=HNUM(K1)
         DO 502 K2=1,K1-1
            NH2=HNUM(K2)
            IF (NH1.EQ.NH2) THEN
               L=NLEVP1
            ELSE
               DO 503 L=1,NLEVEL
                  IF (IHIER(NH1,L) .NE. IHIER(NH2,L)) GO TO 7
503            CONTINUE
            ENDIF
  7         CONTINUE
            LD(K1,K2)=L
            LSUM(L)=LSUM(L)+1
502      CONTINUE
501   CONTINUE

*  Count number of comparisons falling into each range:

      DO 504 I=1,20
         DO 505 J=1,NLEVP1
            NDIST(I,J)=0
505      CONTINUE
504   CONTINUE
      REWIND 19
      DO 506 J=1,NLOC
         READ (19) ((DSING(K1,K2),K1=K2,NOTU),K2=1,NOTU)
         DO 507 K1=2,NOTU
            DO 508 K2=1,K1-1
               ICLASS=INT(DSING(K1,K2)/INTRVL)+1
               IF (ICLASS.GT.20) ICLASS=20
               LEVEL=LD(K1,K2)
               NDIST(ICLASS,LEVEL)=NDIST(ICLASS,LEVEL)+1
508         CONTINUE
507      CONTINUE
506   CONTINUE

*  Histogram output:

      CALL PAGSKP
      WRITE (6,201) COEF(NCOEF)
      PERC=0.
      DO 509 I=1,10
         PERC=PERC+10.
         YLAB(I)=PERC
509   CONTINUE
      DO 510 L=1,NLEVP1
         LSUML=LSUM(L)*NLOC
         WRITE (6,202) HNAME(L),LSUML
*        (Find maximum value for scaling)
         NMAX=-1
         DO 511 I=1,20
            IF (NDIST(I,L).GT.NMAX) NMAX=NDIST(I,L)
511      CONTINUE
         FRQMAX=NMAX*100./LSUML
         IF (FRQMAX.LE.50.) THEN
            YRANGE=50.
            NTIC=6
            NLAB=5
         ELSE
            IF (FRQMAX.LE.60.) THEN
               YRANGE=60.
               NTIC=5
               NLAB=6
            ELSE
               YRANGE=100.
               NTIC=3
               NLAB=10
            ENDIF
         ENDIF
         DO 512 I=1,60
            JLINE(I:I)='-'
512      CONTINUE
         DO 513 I=1,61,NTIC
            JLINE(I:I)='+'
513      CONTINUE
         PERC=0.
         WRITE (6,203)
         NSPACE=NTIC*2-5
         FMT='(T7,''0.0'',10(    X,F5.1))'
         WRITE (FMT(14:17),'(I4.4)') NSPACE
         WRITE (6,FMT) (YLAB(I),I=1,NLAB)
         WRITE (6,204) JLINE
         CU=0.
         DO 514 I=1,20
            CU=CU+INTRVL
            PCTG=100.*NDIST(I,L)/LSUML
            NSTARS=60.*PCTG/YRANGE + 0.5
            IF (NSTARS.NE.0) THEN
               FMT='(T8,''!'',  (''*''),T73,:,''('',F6.2,'')'')'
               WRITE (FMT(9:10),'(I2.2)') NSTARS
               WRITE (6,FMT) PCTG
               WRITE (6,FMT)
            ELSE
               WRITE (6,205) PCTG
               WRITE (6,205)
            ENDIF
            IF (I .NE. 20) WRITE (6,206) CU
514      CONTINUE
         WRITE (6,207) SCALE,JLINE
         FMT='(T7,''0.0'',10(    X,F5.1))'
         WRITE (FMT(14:17),'(I4.4)') NSPACE
         WRITE (6,FMT) (YLAB(I),I=1,NLAB)
         WRITE (6,203)
510   CONTINUE
      GO TO 2

200   FORMAT (//////T3,76('*')/T3,'*',T78,'*'/T3,'*',2X,A72,
     +'*'/T3,'*',T78,'*'/T3,
     +'*  Frequency Distribution of Single-Locus Coefficients',
     +T78,'*'/T3,'*',T78,'*'/T3,'*  BIOSYS-1   Release 1.7',3X,A12,3X,
     +A12,T78,'*'/T3,'*',T78,'*'/T3,76('*')//////)
201   FORMAT (////' Frequency distribution of single-locus similarity/'
     +,'distance coefficients'/1X,71('*')///5X,'Coefficient: ',A48)
202   FORMAT (//////5X,'Level of divergence: ',A10/5X,
     +'Number of comparisons:',I4///)
203   FORMAT (/T19,'Percentage of single-locus coefficients'/)
204   FORMAT (2X,'0.00 ',A61)
205   FORMAT (T8,'!',T73,:,'(',F6.2,')')
206   FORMAT (2X,F4.2,' +')
207   FORMAT (2X,F4.2,1X,A61)
300   FORMAT (' ERROR: Step HIERARCHY has not been performed or ',
     +'previously failed'/)
301   FORMAT (' ERROR: Illegal value for COEF:',I5/)
302   FORMAT (' ERROR: Nei standard distances not permitted')
      END
