*=======================================================================*
*                                                                       *
*                       S T E P   H D Y W B G                           *
*                                                                       *
*         Test for conformance to Hardy-Weinberg equilibrium            *
*                                                                       *
*  Significance probabilities for deviation of genotype frequencies     *
*  from those expected under Hardy-Weinberg equilibrium are calculated  *
*  by the usual chi-square test.  If desired, Levene's correction for   *
*  small sample size may be employed.  Exact probabilities may be       *
*  calculated using Haldane's exact test (see Elston and Forthofer,     *
*  1977; Vithayasai, 1973).                                             *
*                                                                       *
*  The exact test is performed for diallelic loci or after pooling      *
*  of alleles into 2 classes (the most common allele and all other      *
*  alleles).                                                            *
*                                                                       *
*  Options:                                                             *
*                                                                       *
*     LEVENE: use Levene (1949) correction for small sample size        *
*     YATES : apply Yates correction for continuity                     *
*     NOPOOL: suppress pooling of genotypes into three classes          *
*             (nopool over-ridden if EXACTP is specified)               *
*     EXACTP: calculate exact binomial probabilities (in addition       *
*             to normal chi-square analysis)                            *
*                                                                       *
*  Subprograms required by step HDYWBG: HDYWBG  READER  PROB  LOGFAC    *
*                                       FCHSQ   FNRML   G               *
*                                                                       *
*=======================================================================*

      SUBROUTINE HDYWBG
     +   (POPNAM,LOCNAM,POPNUM,FREQ,MINI,MAXI,N,NOBS,OBSHET,EXPHET,
     +    UNCEXP,MAXCOM,JPOOL,MA,ML,MT)

*  Performs chi-square or exact test for deviation of genotype
*  frequencies from those expected under Hardy-Weinberg equilibrium and
*  calculates coefficients for deficiency or excess of heterozygotes

      CHARACTER LINE*80,DATE*12,TIME*12,TITLE*72,FMT*80,LETTER,
     +   POPNAM*16,POPNUM*4,INTST*64,REALST*8,LOGST*8,LOCNAM*5
      DOUBLE PRECISION LGFACT,LOGFCT,LOGFAC,DENOM
      REAL FREQ,EXPHET,UNCEXP,REALVL
      INTEGER MINI,MAXI,N,NOBS,OBSHET,MAXCOM,JPOOL,
     +   OBS,OBS1,OBS2,OBS3,HOBS,R1,R2,R3,X,INTVL
      LOGICAL LEVENE,YATES,NOPOOL,EXACTP,R2ODD,LOGVL
      PARAMETER (NMAX=50)
      COMMON/GBPARM/NOTU,NOTU2,NLOC,NALL,NLET,CRT,NDATYP
      COMMON/SYMB/LETTER(26)
      COMMON/BIPROB/LGFACT(NMAX),DENOM,R1,R2,R3,NJ
      COMMON/CHARV/LINE,DATE,TIME,TITLE,FMT
      DIMENSION POPNAM(MT),POPNUM(MT),INTST(1),LOGST(5),LOCNAM(ML),
     +   FREQ(MA,ML),EXPHET(ML),UNCEXP(ML),MINI(ML),MAXI(ML),N(ML),
     +   NOBS(MA,MA,ML),OBSHET(ML),MAXCOM(ML),JPOOL(ML),INTVL(1),
     +   LOGVL(5),REALVL(1),REALST(1)
      LOGICAL FULSKP
      COMMON/PAGES/FULSKP
      DATA LOGST/'LEVENE','YATES','NOPOOL','EXACTP','END'/,
     +   INTST/'ZZZ'/,REALST/'ZZZ'/

      CALL PAGSKP
      WRITE (6,200) TITLE,DATE,TIME
      IF (NDATYP .EQ. 3) THEN
         WRITE (6,300)
         RETURN
      ENDIF

*  Defaults:

      DO 500 I=1,5
         LOGVL(I)=.FALSE.
500   CONTINUE

      CALL READER (1,1,5,INTST,REALST,LOGST,INTVL,REALVL,LOGVL,IERROR)
      IF (IERROR.EQ.1) RETURN
      LEVENE=LOGVL(1)
      YATES=LOGVL(2)
      NOPOOL=LOGVL(3)
      EXACTP=LOGVL(4)

      IF (LEVENE) WRITE (6,201)
      IF (YATES) WRITE (6,202)
      IF (NOPOOL) WRITE (6,203)
      IF (EXACTP) NOPOOL=.FALSE.
      REWIND 17
      REWIND 18
      DO 501 K=1,NOTU
         CALL PAGSKP
         WRITE (6,204) POPNAM(K),POPNUM(K)
         READ (17) FREQ,MINI,MAXI,N
         READ (18) NOBS
         KPOLY=0
         KPOOL=0

*        Without pooling:

         DO 502 J=1,NLOC
            JPOOL(J)=0
            IF (MINI(J).NE.MAXI(J)) THEN
               KPOLY=KPOLY+1
               IF (KPOLY.EQ.1) WRITE (6,205)
               WRITE (6,206) LOCNAM(J)
               NALLS=0
               MAXFRQ=0
               CHISQ=0.
               EXPHET(J)=0.
               UNCEXP(J)=0.
               OBSHET(J)=0
               MINIJ=MINI(J)
               MAXIJ=MAXI(J)
               NJ=N(J)
               TWONJ=2*NJ
               FOURNJ=4*NJ
               DO 503 I1=MINIJ,MAXIJ
                  IF (FREQ(I1,J).NE.0.) THEN
                     NALLS=NALLS+1
                     IG1=NINT(FREQ(I1,J)*TWONJ)
                     G1=IG1
                     IF (IG1.GT.MAXFRQ) THEN
                        MAXCOM(J)=I1
                        MAXFRQ=IG1
                     ENDIF
                     DO 504 I2=I1,MAXIJ
                        OBS=NOBS(I1,I2,J)
                        IF (I2.LE.I1) THEN
                           EXP=G1*G1/FOURNJ
                           EXPUNC=EXP
                           IF (LEVENE) EXP=(G1*G1-G1)/(FOURNJ-2)
                        ELSE
                           IF (FREQ(I2,J).EQ.0.) GO TO 504
                           IG2=NINT(FREQ(I2,J)*TWONJ)
                           G2=IG2
                           EXP=G1*G2/TWONJ
                           EXPUNC=EXP
                           IF (LEVENE) EXP=G1*G2/(TWONJ-1)
                           OBSHET(J)=OBSHET(J)+OBS
                           EXPHET(J)=EXPHET(J)+EXP
                           UNCEXP(J)=UNCEXP(J)+EXPUNC
                        ENDIF
                        IF (EXP.LE.0.) THEN
                           CALC=0.
                        ELSE
                           D=FLOAT(OBS)-EXP
                           IF (YATES) THEN
                              D=ABS(D)-.5
                              IF (D.LT.0.) D=0.
                           ENDIF
                           CALC=D*D/EXP
                        ENDIF
                        CHISQ=CHISQ+CALC
                        IF (NLET.EQ.1) THEN
                           WRITE (6,208) LETTER(I1),LETTER(I2),OBS,EXP
                        ELSE
                           WRITE (6,207) I1,I2,OBS,EXP
                        ENDIF
504                  CONTINUE
                  ENDIF
503            CONTINUE
               IDF=NALLS*(NALLS-1)/2
               P=1.-FCHSQ(CHISQ,IDF)
               WRITE (6,209) CHISQ,IDF,P
               IF (NALLS.GE.3) THEN
                  KPOOL=1
                  JPOOL(J)=1
               ENDIF
            ENDIF
502      CONTINUE
         IF (KPOLY.LE.0) THEN
            WRITE (6,210)
            GO TO 501
         ENDIF
         WRITE (6,211)
         IF (KPOOL.NE.0 .AND. .NOT. NOPOOL) THEN

*           With pooling:

            CALL PAGSKP
            WRITE (6,212) POPNAM(K),POPNUM(K)
            DO 505 J=1,NLOC
               IF (JPOOL(J).EQ.1) THEN
                  MINIJ=MINI(J)
                  MAXIJ=MAXI(J)
                  NJ=N(J)
                  TWONJ=2*NJ
                  FOURNJ=4*NJ
                  I1=MAXCOM(J)
                  OBS1=NOBS(I1,I1,J)
                  OBS2=0
                  DO 506 I2=MINIJ,MAXIJ
                     IF (I2.NE.I1) OBS2=OBS2+NOBS(I1,I2,J)
506               CONTINUE
                  OBS3=NJ-OBS1-OBS2
                  G1=FREQ(I1,J)*TWONJ
                  G2=TWONJ-G1
                  IF (LEVENE) THEN
                     EXP1=(G1*G1-G1)/(FOURNJ-2)
                     EXP2=(G1*G2)/(TWONJ-1)
                     EXP3=(G2*G2-G2)/(FOURNJ-2)
                  ELSE
                     EXP1=G1*G1/FOURNJ
                     EXP2=G1*G2/TWONJ
                     EXP3=G2*G2/FOURNJ
                  ENDIF
                  D1=OBS1-EXP1
                  D2=OBS2-EXP2
                  D3=OBS3-EXP3
                  IF (YATES) THEN
                     D1=ABS(D1)-.5
                     IF (D1.LT.0.) D1=0.
                     D2=ABS(D2)-.5
                     IF (D2.LT.0.) D2=0.
                     D3=ABS(D3)-.5
                     IF (D3.LT.0.) D3=0.
                  ENDIF
                  IDF=1
                  IF (EXP3.LE.1.E-6) THEN
                     CHISQ=0.
                     P=1.
                  ELSE
                     CHISQ= D1*D1/EXP1 + D2*D2/EXP2 + D3*D3/EXP3
                     P=1.-FCHSQ(CHISQ,IDF)
                  ENDIF
                  WRITE (6,213) LOCNAM(J),OBS1,EXP1
                  WRITE (6,214) OBS2,EXP2
                  WRITE (6,215) OBS3,EXP3,CHISQ,IDF,P
               ENDIF
505         CONTINUE
            WRITE (6,216)
         ENDIF
         IF (EXACTP) THEN

*           Calculation of exact P-values using binomial expectations:

            CALL PAGSKP
            WRITE (6,217) POPNAM(K),POPNUM(K)

*           Log factorial table:

            LGFACT(1)=0.
            LOGFCT=0.
            DO 507 I=2,NMAX
               LOGFCT=LOGFCT + DLOG10(DBLE(I))
               LGFACT(I)=LOGFCT
507         CONTINUE
            DO 508 J=1,NLOC
               MINIJ=MINI(J)
               MAXIJ=MAXI(J)
               IF (MINIJ.NE.MAXIJ) THEN
                  NJ=N(J)
                  NTOT=NJ+NJ
                  I1=MAXCOM(J)
                  R1=NOBS(I1,I1,J)
                  R2=0
                  DO 509 I2=MINIJ,MAXIJ
                     IF (I2.NE.I1) R2=R2+NOBS(I1,I2,J)
509               CONTINUE
                  R2ODD = R2/2 .NE. (R2+1)/2
                  R3=NJ-R1-R2
                  IG1=R1+R1+R2
                  IG2=NTOT-IG1
                  DENOM=LOGFAC(NTOT)-LOGFAC(IG1)-LOGFAC(IG2)
                  POUTCM=PROB(R2)
                  PSUM=POUTCM

                  IF (4*R1*R3-R2*R2 .LT. 0) THEN

*                    Heterozygote excess:

                     IF (R3.NE.0) THEN
                        DO 510 I=1,R3
                           PSUM=PSUM+PROB(R2+2*I)
510                     CONTINUE
                     ENDIF
                     I=0
                     IF (R2ODD) I=1
 18                  IF (I.LT.R2) THEN
                        P=PROB(I)
                        IF (P.LE.POUTCM) THEN
                           PSUM=PSUM+P
                           I=I+2
                           GO TO 18
                        ENDIF
                     ENDIF
                     WRITE (6,218) LOCNAM(J),R1,R2,R3,PSUM

                  ELSE IF (4*R1*R3-R2*R2 .EQ. 0) THEN

*                    Perfect fit to H-W expectations:

                     PSUM=1.0
                     WRITE (6,218) LOCNAM(J),R1,R2,R3,PSUM

                  ELSE IF (4*R1*R3-R2*R2 .GT. 0) THEN

*                    Heterozygote deficiency:

                     ISTART=2
                     IF (R2ODD) ISTART=3
                     IF (ISTART.LT.R2) THEN
                        DO 511 I=ISTART,R2,2
                           PSUM=PSUM+PROB(I-2)
511                     CONTINUE
                     ENDIF
                     I=0
 23                  X=R2+2*(R3-I)
                     IF (X.NE.R2) THEN
                        P=PROB(X)
                        IF (P.LE.POUTCM) THEN
                           PSUM=PSUM+P
                           I=I+1
                           GO TO 23
                        ENDIF
                     ENDIF
                     WRITE (6,218) LOCNAM(J),R1,R2,R3,PSUM
                  ENDIF
               ENDIF
508         CONTINUE
            WRITE (6,219)
         ENDIF

*     Selander's "D", Wright's fixation index:
                                                   

         CALL PAGSKP
         WRITE (6,220) POPNAM(K),POPNUM(K)
         DO 512 J=1,NLOC
            IF (MINI(J).NE.MAXI(J)) THEN
              IF(N(J).EQ.0)GO TO 512                                     
              PSQR=0.0                                                      
              PCUBE=0.0                                                     
                   DO 1460 IWC=1,NALL                                               
                      PSQR=PSQR+(FREQ(IWC,J)**2.0)                          
                      PCUBE=PCUBE+(FREQ(IWC,J)**3.0)
 1460              CONTINUE                                                          
               HOBS=OBSHET(J)
               HEXP=EXPHET(J)
               HEXPUN=UNCEXP(J)
*             POOL HETEROZYGOTE CLASSES AND CALCULATE CHI-SQUARE GOODNESS       
*             OF FIT.
                 ROBS=HOBS
                 DR=N(J)
                 HETEXP=HEXP/DR                                           
                 HETSUM=ROBS/DR                            
                 IF(HETEXP.EQ.0) GO TO 512                                    
                 IF(HEXP.EQ.0) GO TO 512                                    
                 IF(HEXPUN.EQ.0) GO TO 512                                    
                 D=(HOBS-HEXP)/HEXP
                 F=(HEXPUN-HOBS)/HEXPUN
                 DIFF=HETEXP-HETSUM                                        
                 ANUMER=DR*((ABS(DIFF))**2.0)                              
                 DENOM=PSQR+(PSQR**2.0)-(2*PCUBE)                      
                 CHISC=ANUMER/DENOM                                                
                     IDF=1                                                             
                     P=1.-FCHSQ(CHISC,IDF)
               WRITE (6,221) LOCNAM(J),HOBS,HEXP,F,D,CHISC,P
            ENDIF
512      CONTINUE
         WRITE (6,222)
501   CONTINUE
      RETURN

200   FORMAT (//////T3,76('*')/T3,'*',T78,'*'/T3,'*',2X,A72,
     +'*'/T3,'*',T78,'*'/T3,
     +'*  Test for conformance to Hardy-Weinberg equilibrium',
     +T78,'*'/T3,'*',T78,'*'/T3,'*  BIOSYS-1   Release 1.7',3X,A12,3X,
     +A12,T78,'*'/T3,'*',T78,'*'/T3,76('*')////)
201   FORMAT (' Levene (1949) correction for small sample size ',
     +'employed in chi-square analyses')
202   format (' Yates correction for continuity applied in chi-square',
     +'analyses')
203   format (' Genotype pooling (with more than two alleles) ',
     +'suppressed')
204   format (/////' Chi-square test for deviation from Hardy-Weinberg',
     +' equilibrium'/1X,61('*')//6X,'Population: ',A16,' (',A4,')'///)
205   FORMAT (1X,76('-')/T24,'Observed',T37,'Expected',T54,'Chi-'/
     +' Locus',T14,'Class',T24,'frequency',T37,'frequency',T53,
     +'square',T65,'DF',T75,'P'/1X,76('-'))
206   FORMAT (/1X,A5)
207   FORMAT (T14,I2,'-',I2,T27,I3,T38,F7.3)
208   FORMAT (T15,A1,'-',A1,T27,I3,T38,F7.3)
209   FORMAT (T49,F10.3,T64,I3,T73,F5.3)
210   FORMAT (6X,'*** No polymorphic loci ***')
211   FORMAT (1X,76('-'))
212   FORMAT (/////' Chi-square test with pooling'/1X,28('*')//6X,
     +'Population: ',A16,' (',A4,')'////1X,78('-')/T32,'Observed',
     +T44,'Expected',T58,'Chi-'/' Locus',T14,'Class',T32,'frequency',
     +T44,'frequency',T57,'square',T68,'DF',T77,'P'/1X,78('-'))
213   FORMAT (/1X,A5,2X,'Homozygotes for'/T12,'most common allele',
     +T35,I3,T45,F7.3)
214   FORMAT (T9,'Common/rare'/T12,'heterozygotes',T35,I3,T45,F7.3)
215   FORMAT (T9,'Rare homozygotes and'/T12,'other heterozygotes',
     +T35,I3,T45,F7.3,T56,F7.3,T68,I2,T75,F5.3)
216   FORMAT (1X,78('-'))
217   FORMAT (/////' Significance test using exact probabilities'/
     +1X,43('*')//6X,'Population: ',A16,' (',A4,')'////1X,45('-')/
     +' Locus',8X,'R1',8X,'R2',8X,'R3',7X,'P'/1X,45('-'))
218   FORMAT (1X,A5,3I10,F10.3)
219   FORMAT (1X,45('-'))
220   FORMAT (/////' Coefficients for heterozygote deficiency or ',
     +'excess'/1X,50('*')//6X,'Population: ',A16,' (',A4,')'////1X,
     +70('-')/T14,'Heterozygotes',T33,'Fixation',T55,'Chi-',/,
     +' Locus',T11,'Observed',T22,'Expected',T33,'index (F)',
     +T47,'D',T55,'square',T67,'P',/1X,70('-'))
221   FORMAT (/1X,A5,T13,I3,T22,F7.3,T33,F6.3,T44,F6.3,T53,F7.2,
     +T64,F6.4)
222   FORMAT (1X,70('-'))
300   FORMAT (///' ERROR: Hardy-Weinberg test cannot be performed with',
     +' allele frequency input'/8X,'(DATYP = 3)')
      END


      FUNCTION PROB (X)

*  Calculates binomial probability of a given genotypic frequency
*  distribution (2 alleles; 3 genotypes) given 2r1+r2 = g1 genes
*  of one allelic type are present

      DOUBLE PRECISION LOGFAC,LOGPRB,LGFACT,DENOM
      INTEGER A,B,C,R1,R2,R3,X
      PARAMETER (NMAX=50)
      COMMON/BIPROB/LGFACT(NMAX),DENOM,R1,R2,R3,N

      A=R1+(R2-X)/2
      B=X
      C=R3+(R2-X)/2
      LOGPRB=LOGFAC(N)-LOGFAC(A)-LOGFAC(B)-LOGFAC(C)+DLOG10(2.D0**X)
     +       - DENOM
      PROB=10.D0**LOGPRB
      RETURN
      END


      DOUBLE PRECISION FUNCTION LOGFAC (K)

*  Calculates log of K!

*  NOTE:  To save execution-time, a table of log factorials is set up
*         in routine HDYWBG.  The number of elements in this table
*         is equal to "NMAX".  If NMAX is exceeded, log factorial
*         calculation is done by repetitive addition in the
*         usual manner.

      DOUBLE PRECISION LGFACT,DENOM
      PARAMETER (NMAX=50)
      COMMON/BIPROB/LGFACT(NMAX),DENOM,R1,R2,R3,N

      IF (K.LE.0) THEN
         LOGFAC=0.
         RETURN
      ENDIF

      IF (K.LE.NMAX) THEN
         LOGFAC=LGFACT(K)
         RETURN
      ENDIF
      LOGFAC=LGFACT(NMAX)
      NP1=NMAX+1
      DO 500 I=NP1,K
         LOGFAC=LOGFAC+LOG10(DBLE(I))
500   CONTINUE
      RETURN
      END
