*=======================================================================*
*                                                                       *
*                       S T E P   H E T X S Q                           *
*                                                                       *
*            Contingency chi-square test for significant                *
*                heterogeneity in allele frequencies                    *
*                                                                       *
*  Testing is by the usual chi-square test for m x n contingency tables *
*  with (m-1)*(n-1) degrees of freedom (m = number of populations; n =  *
*  number of alleles).                                                  *
*                                                                       *
*  Options:                                                             *
*     SUBDIV = 0   analysis of full set of otu's (default)              *
*            > 0   independent analysis of subsets of otu's; SUBDIV     *
*                  specifies level of hierarchy at which analyses       *
*                  are to be performed                                  *
*     CONTAB : calculate and print contingency tables                   *
*                                                                       *
*  Subprograms required by step HETXSQ: HETXSQ  READER  FCHSQ  FNRML  G *
*                                                                       *
*=======================================================================*

      SUBROUTINE HETXSQ
     +   (FREQ2,MINI2,MAXI2,N2,OBSUM,NUMALL,LOCNAM,EXP,
     +    OBS,CALC,POPNAM,NAME,IHIER,HNUM,MA,ML,MT)

      CHARACTER LINE*80,DUM,DATE*12,TIME*12,TITLE*72,FMT*80,NAME*16,
     +   LETTER,INTST*64,REALST*8,LOGST*8,LOCNAM*5,POPNAM*16,HNAME*10
      REAL FREQ2,OBSUM,EXP,OBS,CALC,REALVL
      INTEGER MINI2,MAXI2,N2,NUMALL,IHIER,HNUM,SUBDIV
      LOGICAL IPRES,SUB,CONTAB,LOGVL,CRT
      COMMON/GBPARM/NOTU,NOTU2,NLOC,NALL,NLET,CRT,NDATYP
      COMMON/HIER1/NOHIER,NLEVEL,NROW,NHIER(4)
      COMMON/HIER/HNAME(4)
      COMMON/SYMB/LETTER(26)
      COMMON/CHARV/LINE,DATE,TIME,TITLE,FMT
      DIMENSION DUM(15),INTST(1),LOGST(2),LOCNAM(ML),POPNAM(MT),
     +   NAME(MT,3),FREQ2(MA,MT),OBSUM(MA),EXP(MA),OBS(MA),CALC(MA),
     +   MINI2(MT),MAXI2(MT),N2(MT),NUMALL(MA),IHIER(MT,3),HNUM(MT),
     +   REALST(1),INTVL(1),REALVL(1),LOGVL(2)
      LOGICAL FULSKP
      COMMON/PAGES/FULSKP
      DATA INTST/'SUBDIV'/,LOGST/'CONTAB','END'/,ZERO/1.E-6/

      CALL PAGSKP
      WRITE (6,200) TITLE,DATE,TIME

*  Defaults:

      INTVL(1)=0
      LOGVL(1)=.FALSE.
      LOGVL(2)=.FALSE.
      CALL READER (1,0,2,INTST,REALST,LOGST,INTVL,REALVL,LOGVL,IERROR)
      IF (IERROR.EQ.1) RETURN
      CONTAB=LOGVL(1)
      SUBDIV=INTVL(1)
      NCYC=1
      NPOPS=NOTU
      SUB=SUBDIV.NE.0
      IF (SUB) THEN
         IF (NOHIER.NE.0) THEN
            WRITE (6,300)
            RETURN
         ENDIF
         REWIND 14
         READ (14) IHIER,HNUM,NAME
         IF (SUBDIV.GT.NLEVEL) THEN
            WRITE (6,301) NLEVEL
            RETURN
         ENDIF
         NCYC=NHIER(SUBDIV)
      ENDIF
      DO 500 ICYC=1,NCYC
         IF (SUB) THEN
            NPOPS=0
            DO 501 K=1,NOTU
               IF (IHIER(HNUM(K),SUBDIV).EQ.ICYC) NPOPS=NPOPS+1
501         CONTINUE
            IF (NPOPS.LE.1) THEN
               WRITE (6,201) HNAME(SUBDIV),NAME(ICYC,SUBDIV)
               GO TO 500
            ENDIF
         ENDIF
         IF (CONTAB) THEN
            REWIND 16
            DO 502 J=1,NLOC
               READ (16) FREQ2,MINI2,MAXI2,N2

*           Get range of alleles and total sample size;
*           Determine if locus is variable:

               MINA=999
               MAXA=0
               NSUM=0
               DO 503 K=1,NOTU
                  IF (SUB) THEN
                     IF (IHIER(HNUM(K),SUBDIV) .NE. ICYC) GO TO 503
                  ENDIF
                  IF (MINI2(K).LT.MINA) MINA=MINI2(K)
                  IF (MAXI2(K).GT.MAXA) MAXA=MAXI2(K)
                  NSUM=NSUM+N2(K)
503            CONTINUE
               IF (MINA.NE.MAXA) THEN

*              Determine which alleles are present, count frequencies:

                  NALLS=0
                  DO 504 I=MINA,MAXA
                     IPRES=.FALSE.
                     DO 505 K=1,NOTU
                        IF (SUB) THEN
                           IF (IHIER(HNUM(K),SUBDIV) .NE. ICYC)
     +                        GO TO 505
                        ENDIF
                        IF (.NOT. FREQ2(I,K).LE.ZERO) THEN
                           IF (.NOT. IPRES) THEN
                              NALLS=NALLS+1
                              IALL=NALLS
                              NUMALL(IALL)=I
                              IPRES=.TRUE.
                              OBSUM(IALL)=0.
                           ENDIF
                           OBSUM(IALL)=OBSUM(IALL)+FREQ2(I,K)*(2*N2(K))
                        ENDIF
505                  CONTINUE
504               CONTINUE
                  CALL PAGSKP
                  WRITE (6,202) LOCNAM(J)
                  IF (SUB) WRITE (6,203) HNAME(SUBDIV),NAME(ICYC,SUBDIV)
                  NCOL=9
                  IF (CRT) NCOL=4
                  CHISQ=0.
                  I1=1
  8               I2=MIN(I1+NCOL-1,NALLS)
                  WRITE (6,204) ('----------',II=I1,I2)
                  NSKIPS=(I2-I1+1)*5
                  FMT='(T35,    X,''Allele'')'
                  WRITE (FMT(6:9),'(I4)') NSKIPS
                  WRITE (6,FMT)
                  WRITE (6,205) ('----------',II=I1,I2)
                  IF (NLET.NE.1) THEN
                     WRITE (6,206) (NUMALL(I),I=I1,I2)
                  ELSE
                     WRITE (6,207) (LETTER(NUMALL(I)),I=I1,I2)
                  ENDIF
                  WRITE (6,208) ('----------',II=I1,I2)
                  DO 506 K=1,NOTU
                     IF (SUB) THEN
                        IF (IHIER(HNUM(K),SUBDIV) .NE. ICYC) GO TO 506
                     ENDIF
                     TWONK=2*N2(K)
                     DO 507 I=I1,I2
                        IALL=NUMALL(I)
                        OBSI=FREQ2(IALL,K)*TWONK
                        EXPI=FLOAT(N2(K))/NSUM*OBSUM(I)
                        CALC(I)=((OBSI-EXPI)**2)/EXPI
                        CHISQ=CHISQ+CALC(I)
                        OBS(I)=OBSI
                        EXP(I)=EXPI
507                  CONTINUE

                     WRITE (6,209) POPNAM(K),(OBS(I),I=I1,I2)
                     WRITE (6,210) (EXP(I),I=I1,I2)
                     WRITE (6,211) (CALC(I),I=I1,I2)
506               CONTINUE
                  WRITE (6,208) ('----------',II=I1,I2)
                  IF (I2.NE.NALLS) THEN
                     I1=I1+NCOL
                     GO TO 8
                  ENDIF
                  IDF=(NPOPS-1)*(NALLS-1)
                  P=1.-FCHSQ(CHISQ,IDF)
                  IF (ABS(P).LT.ZERO) P=0.
                  WRITE (6,212) CHISQ,IDF,P
               ENDIF
502         CONTINUE
         ENDIF

*     Calculations using computational formula:

         CSQSUM=0.
         IDFSUM=0
         CALL PAGSKP
         WRITE (6,213)
         IF (SUB) WRITE (6,203) HNAME(SUBDIV),NAME(ICYC,SUBDIV)
         WRITE (6,214)
         REWIND 16
         DO 508 J=1,NLOC
            READ (16) FREQ2,MINI2,MAXI2,N2
            MINA=999
            MAXA=0
            DO 509 K=1,NOTU
               IF (SUB) THEN
                  IF (IHIER(HNUM(K),SUBDIV) .NE. ICYC) GO TO 509
               ENDIF
               IF (MINI2(K).LT.MINA) MINA=MINI2(K)
               IF (MAXI2(K).GT.MAXA) MAXA=MAXI2(K)
509         CONTINUE
            IF (MINA.NE.MAXA) THEN
               CHISQ=0.
               NALLS=0
               DO 510 I=MINA,MAXA
                  NSUM=0
                  SUMX=0.
                  SUMXSQ=0.
                  IPRES=.FALSE.
                  DO 511 K=1,NOTU
                     IF (SUB) THEN
                        IF (IHIER(HNUM(K),SUBDIV) .NE. ICYC) GO TO 511
                     ENDIF
                     NK=N2(K)*2
                     QFREQ=FREQ2(I,K)
                     SUMX=SUMX+NK*QFREQ
                     SUMXSQ=SUMXSQ+NK*QFREQ*QFREQ
                     NSUM=NSUM+NK
                     IF (QFREQ.GT.ZERO) THEN
                        IF (.NOT. IPRES) THEN
                           NALLS=NALLS+1
                           IPRES=.TRUE.
                        ENDIF
                     ENDIF
511            CONTINUE
               IF (SUMX.GT.ZERO) THEN
                  WTVAR=(SUMXSQ-((SUMX*SUMX)/NSUM))/NSUM
                  WTXBAR=SUMX/NSUM
                  CHISQ=CHISQ+WTVAR/WTXBAR
               ENDIF
510            CONTINUE
               CHISQ=NSUM*CHISQ
               IDF=(NPOPS-1)*(NALLS-1)
               P=1.-FCHSQ(CHISQ,IDF)
               IF (ABS(P).LT.ZERO) P=0.
               WRITE (6,215) LOCNAM(J),NALLS,CHISQ,IDF,P
               CSQSUM=CSQSUM+CHISQ
               IDFSUM=IDFSUM+IDF
            ENDIF
508      CONTINUE
         P=1.-FCHSQ(CSQSUM,IDFSUM)
         IF (ABS(P).LT.ZERO) P=0.
         WRITE (6,216) CSQSUM,IDFSUM,P
500   CONTINUE
      RETURN

200   FORMAT (//////T3,76('*')/T3,'*',T78,'*'/T3,'*',2X,A72,
     +'*'/T3,'*',T78,'*'/T3,
     +'*  Contingency Chi-Square Analysis',
     +T78,'*'/T3,'*',T78,'*'/T3,'*  BIOSYS-1   RELEASE 1.7',3X,A12,3X,
     +A12,T78,'*'/T3,'*',T78,'*'/T3,76('*')//////)

201   FORMAT (' Data for only one population in ',A8,3X,A16/
     +' No analysis performed')
202   FORMAT (' Contingency chi-square table'/1X,28('*')//6X,
     +'Locus: ',A5)
203   FORMAT (/6X,A10,': ',A16)
204   FORMAT (///1X,34('-'),9A10)
205   FORMAT (T36,9A10)
206   FORMAT (' Population',22X,9I10)
207   FORMAT (' Population',30X,9(A1,9X))
208   FORMAT (1X,34('-'),9A10)
209   FORMAT (/1X,A16,T21,'Obs (O)',T36,9F10.3)
210   FORMAT (T21,'Exp (E)',T36,9F10.3)
211   FORMAT (T21,'(O-E)**2 / E',T36,9F10.3)
212   FORMAT (//' Chi-square =',F10.3/' D.F. =',I4/' P =',F8.5////)
213   FORMAT (' Contingency chi-square analysis at all loci'/
     +1X,43('*'))
214   FORMAT (///5X,50('-')/T16,'No. of'/T6,'Locus',T16,'alleles',T26,
     +'Chi-square',T40,'D.F.',T52,'P'/5X,50('-'))
215   FORMAT (T6,A5,I9,F15.3,I8,F13.5)
216   FORMAT (5X,50('-')/T6,'(Totals)',F21.3,I8,F13.5/5X,50('-')//////)
300   FORMAT (' ERROR: Step HIERARCHY has not been performed or ',
     +'previously failed')
301   FORMAT (' ERROR: Previously specified hierarchy has only',i2,
     +' levels')
      END
