*=======================================================================*
*                                                                       *
*                    S T E P   H I E R A R C H Y                        *
*                                                                       *
*     Establishment of hierarchical arrangement of populations          *
*                      for use in other steps                           *
*                                                                       *
*  Hierarchy is initialized in combination with step DATA (array        *
*  KHIER). KHIER is created in step data and specifies, for each        *
*  population, a number representing that population's position at all  *
*  levels of the hierarchy.  Step HIERARCHY is then used to provide     *
*  names for the various "supra-populational" groupings or taxa and to  *
*  arrange the set of populations within this structure.                *
*                                                                       *
*  Options:                                                             *
*     NLEVEL       = number of hierarchical levels, not including       *
*                       populations and total                           *
*     NOPRIN       : suppress printing of hierarchy and associated      *
*                       populations                                     *
*                                                                       *
*  Subprograms required by step HIERARCHY: HRCHY  READER                *
*                                                                       *
*=======================================================================*

      SUBROUTINE HRCHY
     +   (IHIER,KHIER,LHIER,NAME,POPNAM,HNUM,MT)

*  Gets hierarchical population structure and sets up arrays used in
*  calculation of F-statistics, heterogeneity chi-square analysis,
*  and similarity/distance coefficient averaging

      CHARACTER LINE*80,DATE*12,TIME*12,TITLE*72,FMT*80,
     +   NAME*16,POPNAM*16,INAME*16,HNAME*10,
     +   INTST*64,REALST*8,LOGST*8,LETTER
      INTEGER IHIER,KHIER,LHIER,HNUM,JHIER,INTVL
      LOGICAL NOPRIN,LOGVL
      REAL REALVL
      COMMON/GBPARM/NOTU,NOTU2,NLOC,NALL,NLET,CRT,NDATYP
      COMMON/HIER1/NOHIER,NLEVEL,NROW,NHIER(4)
      COMMON/HIER/HNAME(4)
      COMMON/SYMB/LETTER(26)
      COMMON/CHARV/LINE,DATE,TIME,TITLE,FMT
      DIMENSION NAME(MT,3),POPNAM(MT),INTST(1),LOGST(1),
     +   REALST(1),IHIER(MT,3),KHIER(MT,3),LHIER(MT,3),HNUM(MT),
     +   JHIER(3),INTVL(1),LOGVL(1),REALVL(1)
      LOGICAL FULSKP
      COMMON/PAGES/FULSKP
      DATA INTST/'NLEVEL'/,LOGST/'NOPRINT'/,REALST/'ZZZ'/

      CALL PAGSKP
      WRITE (6,200) TITLE,DATE,TIME
      NOHIER=0

*  Defaults:

      INTVL(1)=0
      LOGVL(1)=.FALSE.

      CALL READER (1,0,1,INTST,REALST,LOGST,INTVL,REALVL,LOGVL,IERROR)
      IF (IERROR.NE.0) THEN
         NOHIER=1
         RETURN
      ENDIF
      NLEVEL=INTVL(1)
      NOPRIN=LOGVL(1)
      IF (NLEVEL.LE.0 .OR. NLEVEL.GE.4) THEN
         WRITE (6,300) NLEVEL
         RETURN
      ENDIF
      WRITE (6,201) NLEVEL
      IF (NOPRIN) WRITE (6,202)
      WRITE (6,203)
      DO 500 J=1,NLEVEL
         JHIER(J)=0
         NHIER(J)=0
500   CONTINUE
      NLEVP1=NLEVEL+1
      DO 600 J=1,NLEVP1
         READ (5,'(A10)') HNAME(J)
600   CONTINUE
      NROW=0
      NHOLD=1

*  Get name and hierarchical level for a group:

  3   CONTINUE
      READ (5,101) LINE
      IF (LINE(1:3).NE.'END') THEN
         DO 501 JBEG=1,7,3
            IF (LINE(JBEG:JBEG) .NE. ' ') GO TO 4
501      CONTINUE
*        ( ignore blank lines )
         GO TO 3
  4      JEND=JBEG+15
         INAME=LINE(JBEG:JEND)

*        Update arrays specifying hierarchy:

         N=(JBEG+2)/3
         JHIER(N)=JHIER(N)+1
         NHIER(N)=NHIER(N)+1
         IF (N.LE.NHOLD) THEN
            NROW=NROW+1
            IF (N.NE.NHOLD) THEN
               DO 502 J=N+1,NLEVEL
                  JHIER(J)=0
502            CONTINUE
            ENDIF
            IF (N.NE.1) THEN
               DO 503 J=1,N-1
                  IHIER(NROW,J)=IHIER(NROW-1,J)
                  LHIER(NROW,J)=LHIER(NROW-1,J)
503            CONTINUE
            ENDIF
         ENDIF

         IHIER(NROW,N)=NHIER(N)
         LHIER(NROW,N)=JHIER(N)
         NAME(NHIER(N),N)=INAME
         NHOLD=N
         GO TO 3
      ENDIF

*  Associate OTU's (populations) with rows in hierarchy:

      REWIND 14
      READ (14) KHIER
      DO 504 K=1,NOTU
         DO 505 I=1,NROW
            DO 506 J=1,NLEVEL
               IF (KHIER(K,J) .NE. LHIER(I,J)) GO TO 505
506         CONTINUE
*           (If cycle completely through 506 loop, match found)
            HNUM(K)=I
            GO TO 504
505      CONTINUE
*        (If cycle completely through 505 loop, no match found; error)
         WRITE (6,301) POPNAM(K)
         NOHIER=1
504   CONTINUE
      IF (NOHIER.EQ.1) RETURN

*  Store hierarchy data:

      REWIND 14
      WRITE (14) IHIER,HNUM,NAME
      IF (NOPRIN) RETURN

*  Print hierarchy and associated populations:

      CALL PAGSKP
      WRITE (6,204) (HNAME(I),I=1,NLEVEL)
      DO 507 I=1,NROW
         DO 508 J=1,NLEVEL
            IF (I.NE.1) THEN
               IF (IHIER(I,J).EQ.IHIER(I-1,J)) GO TO 508
            ENDIF
            FMT='(/7X,  (5X),A16)'
            WRITE (FMT(6:7),'(I2)') J
            WRITE (6,FMT) NAME(IHIER(I,J),J)
            IF (J.GE.NLEVEL) THEN
               DO 509 K=1,NOTU
                  IF (HNUM(K) .EQ. I) THEN
                     FMT='(/10X, (5X),A16)'
                     WRITE (FMT(7:7),'(I1)') NLEVEL
                     WRITE (6,FMT) POPNAM(K)
                  ENDIF
509            CONTINUE
            ENDIF
508      CONTINUE
507   CONTINUE
      RETURN

101   FORMAT (A22)
200   FORMAT (//////T3,76('*')/T3,'*',T78,'*'/T3,'*',2X,A72,
     +'*'/T3,'*',T78,'*'/T3,
     +'*  Hierarchy initialization',
     +T78,'*'/T3,'*',T78,'*'/T3,'*  BIOSYS-1   Release 1.7',3X,A12,3X,
     +A12,T78,'*'/T3,'*',T78,'*'/T3,76('*')////)
201   FORMAT (' Number of hierarchical levels:',I2)
202   FORMAT (/' Printing of hierarchy suppressed')
203   FORMAT (//////)
204   FORMAT (' Hierarchy saved for further operations'/1X,38('*')////
     +5X,'Hierarchical levels: ',3(A10,3X))
300   FORMAT (' ERROR: Illegal value for NLEVEL:',I5/)
301   FORMAT (///' ERROR: Hierarchy incompatible with parameters ',
     +'specified in step data.'/8X,'Population ',A16,' cannot be ',
     +'placed in hierarchy.')
      END
