* $LARGE
* $DEBUG
*************************************************************************
*                                                                       *
*                    P R O G R A M   B I O S Y S - 2                    *
*                                                                       *
*                                 F O R                                 *
*                                                                       *
*      I B M   V S - F O R T R A N   A N D   O T H E R   A N S I -      *
*                                                                       *
*          S T A N D A R D   F O R T R A N   C O M P I L E R S          *
*                                                                       *
*-----------------------------------------------------------------------*
*                                                                       *
*       BIOSYS-2 is a program for the comprehensive analysis of         *
*  electrophoretic data in population genetics and systematics.  Its    *
*  capabilities include:                                                *
*                                                                       *
*     (1) Computation of allele and genotype frequencies from raw data  *
*     (2) Computation of measures of genetic variability                *
*     (3) Testing for deviation from Hardy-Weinberg equilibrium         *
*     (4) Calculation of Wright's F-statistics                          *
*     (5) Heterogeneity chi-square analysis                             *
*     (6) Computation of genetic similarity and distance coefficients   *
*     (7) Dendrogram construction using UPGMA, WPGMA, single-linkage,   *
*         or complete linkage cluster analysis; distance Wagner         *
*         procedure; and modified distance Wagner procedure             *
*                                                                       *
*  A more complete presentation of the program's capabilities is        *
*  provided in Swofford and Selander (J. Heredity, 1981) or in the      *
*  accompanying user's guide.                                           *
*                                                                       *
*       The program, user's guide (instructions and documentation),     *
*  and test data are available from:                                    *
*                                                                       *
*             William C. Black IV                                       *
*             Department of Microbiology                                *
*             Colorado State University                                 *
*             Ft. Collins, CO 80523                                     *
*                                                                       *
*       HISTORY OF UPDATES:                                             *
*                                                                       *
*            Release 1.0: November 1981 (CDC version)                   *
*            Release 1.1: November 1981 (IBM version)                   *
*            Release 1.2: March 1982 (bugs fixed in SIMDIS-CDC)         *
*            Release 1.3: March 1982 (bugs fixed in SIMDIS-IBM)         *
*            Release 1.4: November 1982 (bugs fixed in SIMDIS           *
*                         and DISWAG-CDC)                               *
*            Release 1.5: November 1982 (bugs fixed in SIMDIS-IBM)      *
*            Release 1.6: January 1985 (Fortran 77 conversion)          *
*            Release 1.7: August 1988  (IBM PC version)                 *
*            Release 2.0: June 1997 (IBM PC version)                    *
*                         LINKDIS options, FSTAT modified,              *
*                         BOOTDIST option                               *
*  REDIMENSIONING:                                                      *
*                                                                       *
*       The program is originally provided with the following upper     *
*  limits:                                                              *
*                                       Parameter     Original          *
*              Limit                      name          value           *
*       -------------------------------------------------------         *
*       Number of taxa (populations)       MXT           40             *
*       Number of loci                     MXL           40             *
*       Number of alleles per locus        MXA           20             *
*       Maximum value of "NCOL"            MXC           20             *
*                                                                       *
*  To increase or decrease these limits, simply modify the PARAMETER    *
*  statement in the main program to reflect the desired limits.  There  *
*  are a few restrictions that MUST be observed:                        *
*                                                                       *
*      1.  MXT must be at least 12                                      *
*      2.  See the comment block just preceding the COMMON/ALIGN2/      *
*          statement.                                                   *
*      3.  If 10*MXA < MT or MXT**2 < 25*MXA, one or more EQUIVALENCE   *
*          statements will have to be removed (see comments below).     *
*                                                                       *
*                                                                       *
*  MACHINE-DEPENDENT CODE:  See the main program (BIOSYS), subroutine   *
*                           GETDT, function LETNUM, and subroutine      *
*                           PAGSKP                                      *
*  DEVICES USED IN CURRENT CODE: 2,3,5,6,11,14,15,16,17,18,19           *
*************************************************************************


      PROGRAM BIOSYS

*-----------------------------------------------------------------*
*  On CDC Cyber machines, substitute the PROGRAM statement below  *
*                   for the one above                             *
*-----------------------------------------------------------------*
C     PROGRAM BIOSYS (INFIL,OUTFIL,OPTIN,TAPE1=INFIL,TAPE2=OUTFIL,
C    +   TAPE3=OPTIN)

      INTEGER DWRECL,DARECL
      LOGICAL CRT,IBM,PRIME,IBMPC,GNU77
      CHARACTER LINE*80,DATE*12,TIME*12,TITLE*72,FMT*80,
     +   COEF*48,WORD1*4,WORD2*4

*-----------------------------------------------------------------*
*  IMPORTANT:  All but one of the parameter statements below      *
*              should be commented out.  Also, make sure that     *
*              either COMMON/ALIGN1/ or COMMON/ALIGN2/ is         *
*              commented out (see below).                         *
*-----------------------------------------------------------------*
*  Parameter statement for IBM-PC large version and default
*     mainframe dimensions:
*      PARAMETER (MXT=50, MXL=75, MXA=25, MXC=25, MXA2=MXA+MXA,
*     +  MXLT=MXL*MXT, MXT2=MXT+MXT, MXHLFT=MXT2*(MXT2+1)/2)
*  Parameter statement for IBM-PC small version:
*      PARAMETER (MXT=40, MXL=30, MXA=10, MXC=8, MXA2=MXA+MXA,
*     +   MXLT=MXL*MXT, MXT2=MXT+MXT, MXHLFT=MXT2*(MXT2+1)/2)
*  Parameter statement for real computers in the 21st century
      PARAMETER (MXT=200, MXL=200, MXA=200, MXC=200, MXA2=MXA+MXA,
     +  MXLT=MXL*MXT, MXT2=MXT+MXT, MXHLFT=MXT2*(MXT2+1)/2)

      LOGICAL FULSKP
      COMMON/PAGES/FULSKP

*---------------------------------------------*
*  Use statement below in IBM-PC version only *
*---------------------------------------------*
*  Most arrays are dimensioned in the main program and passed to
*  subprograms as arguments to permit execution-time dimensioning
*  of arrays.  This approach greatly simplifies redimensioning of
*  the program (only the main program need be modified and
*  recompiled) and speeds scratch file I/O.

*  Main program:

      CHARACTER INTST(3)*64,REALST(1)*8,LOGST(2)*8
      INTEGER INTVL(3)
      REAL REALVL(1)
      LOGICAL LOGVL(2)

*  Global:

      CHARACTER POPNAM(MXT)*16,POPNUM(MXT)*4,LOCNAM(MXL)*5,LETTER*1
      INTEGER IGRP(MXT)

*  DATIN:

      CHARACTER CG1(MXL),CG2(MXL),CMG1(MXC),CMG2(MXC),CNA(MXA)
      REAL Q(MXC),TFREQ(MXA2)
      INTEGER MAXIT(MXL),NG1(MXL),NG2(MXL),NA(MXA),MG1(MXC),
     +   MG2(MXC),NIND(MXC),MINA(MXL,MXT),MAXA(MXL,MXT),
     +   NSMPL(MXL,MXT),INDX(MXLT)

*  DATIN, VARIAB, and HDYWBG:
*****
      REAL FREQ(MXA,MXL)
      INTEGER MINI(MXL),MAXI(MXL),N(MXL),NOBS(MXA,MXA,MXL)

*  DATIN and HRCHY:

      INTEGER KHIER(MXT,3)

*  DATIN, SIMDIS, BOOTDIST, SINGLE, AYAGRM, HETXSQ, FSTAT, and WRIGHT:
****
      REAL FREQ2(MXA,MXT)
      INTEGER MINI2(MXT),MAXI2(MXT),N2(MXT)

*  DATIN and FSTAT:
*****
      REAL FRQHOM(MXA,MXT)

*  VARIAB:

      REAL HBI(MXL),HUNB(MXL),HDC(MXL),AVEN(MXT),
     +   AVEALL(MXT),HETUNB(MXT),HETBIA(MXT),HETDIR(MXT),POLYNO(MXT),
     +   POLY99(MXT),POLY95(MXT),SEN(MXT),SEALL(MXT),SEHBIA(MXT),
     +   SEHUNB(MXT),SEHDIR(MXT)
      INTEGER NALLEL(MXL)

*  SIMDIS:

      REAL JXY(MXT,MXT),DSING(MXT,MXT),DROG1(MXT,MXT),
     +   DARC(MXT,MXT),DCHORD(MXT,MXT),DPREV(MXT,MXT),DEDW(MXT,MXT),
     +   SROG(MXT,MXT),IUNB(MXT,MXT),DUNB(MXT,MXT),IDEN(MXT,MXT),
     +   DIST(MXT,MXT),DMINUN(MXT,MXT),DMIN(MXT,MXT),JX(MXT),GX(MXT),
     +   SMJX2(MXT),SMGX2(MXT),EDJX2(MXT),DROG2(MXT,MXT)

*  BOOTDIST:

      REAL BDIST(MXL,MXT,MXT),SDIST(MXT,MXT),ADIST(MXT,MXT)
      INTEGER SEED
      CHARACTER PHYF*50

*  MATOUT,SINGLE,CLUSTER, and SIMAVE:

      REAL MATRIX(MXT,MXT)

*  HDYWBG:

      REAL EXPHET(MXL),UNCEXP(MXL)
      INTEGER OBSHET(MXL),JPOOL(MXL),MAXCOM(MXL)

*  HRCHY, SIMAVE:

      CHARACTER NAME(MXT,3)*16

*  HRCHY:

      INTEGER LHIER(MXT,3)

*  HRCHY, SIMAVE, AYAGRM, HETXSQ, and WRIGHT:

      CHARACTER HNAME*10
      INTEGER IHIER(MXT,3),HNUM(MXT)

*  SIMAVE:

      REAL CAVE(MXT,MXT),CMIN(MXT,MXT),CMAX(MXT,MXT)
      INTEGER NPOPS(MXT)

*  AYAGRM:

      INTEGER LD(MXT,MXT),JLINE(61),NDIST(20,4)

*  HETXSQ:

      REAL EXP(MXA),OBS(MXA),OBSUM(MXA),CALC(MXA)
      INTEGER NTOT(MXA),NUMALL(MXA)

*  FSTAT and WRIGHT:
***
      LOGICAL PRES(MXA)

*  FSTAT:

      REAL BIGP(MXA),SMALLP(MXA),SMPSQ(MXA),FISNUM(MXT),FISDEN(MXT),
     +   FISIK(MXT),FISI(MXT),FIS(MXL),FIT(MXL),FST(MXL),WGT(MXT),
     +   POPCNT(MXL),AL(MXL),BL(MXL),CL(MXL),CFJK(MXL),TJK(MXL),
     +   XFJK(MXL),FL(MXL),THETL(MXL),CAPFL(MXL),HETER(MXA,MXL)

      LOGICAL VAR(MXL)

*  WRIGHT78:

      REAL QSUM(MXT,3),VC(5,5,MXA),F(5,5,MXA)
      INTEGER NSUM(MXT,3)

*  DISWAG and CLUSTER:

      REAL HPLASY(MXT,MXT),CD(MXT),CUSE(MXT),TERMB(MXT)
      INTEGER ITAX(MXT,2)

*  CLUSTER:

      REAL S(MXT,MXT),COPH(MXHLFT),WT(MXT)
      INTEGER NN(MXT),INAME(MXT),DELETD(MXT)

*  DISWAG:

      REAL D(MXHLFT),P(MXHLFT),DPHEN(MXT,MXT)
      INTEGER ANC(MXT2),NCONN(MXT2,3),ANCTOP(MXT2),ANCPRV(MXT2),
     +   NWT(MXT2),NTERM(MXT2)

*  Subroutine TRPRIN:

      INTEGER IVECT(101),ICLINE(MXT,2),ICBEG(MXT,2),ICEND(MXT,2),
     +   NPOS(MXT),NCENT(MXT),NLINE(MXT),NORDER(MXT),LINKED(MXT)

      COMMON/GBPARM/NOTU,NOTU2,NLOC,NALL,NLET,CRT,NDATYP
      COMMON/SYMB/LETTER(26)
      COMMON/MAT/MATFIL(13),NOCOEF,NMAT,NSING
      COMMON/COEFNM/COEF(13)
      COMMON/HIER1/NOHIER,NLEVEL,NROW,NHIER(4)
      COMMON/HIER/HNAME(4)
      COMMON/ALIGN1/FREQ2,FRQHOM
      COMMON/CHARV/LINE,DATE,TIME,TITLE,FMT

*--------------------------------------*
*  If MXT >= MXL, use COMMON/ALIGN2/;  *
*  If MXL > MXT, use COMMON/ALIGN3/    *
*--------------------------------------*
      COMMON/ALIGN2/JXY,DPREV,DROG1,DARC
C     COMMON/ALIGN3/MINA,MAXA,INDX,NSMPL

*-------------------------------------------------*
*  If 10*MXA < MXT, comment out statement below:  *
*-------------------------------------------------*
      EQUIVALENCE ( SMGX2(1),FRQHOM(1, 1)), ( EDJX2(1),FRQHOM(1, 6))

*----------------------------------------------------*
*  If MXT**2 < 25*MXA, comment out statement below:  *
*----------------------------------------------------*
      EQUIVALENCE (VC(1,1,1),JXY(1,1)),(F(1,1,1),DPREV(1,1))

      EQUIVALENCE (JXY(1,1),DSING(1,1),SROG(1,1),MINA(1,1),CAVE(1,1))
      EQUIVALENCE (DPREV(1,1),DEDW(1,1),IUNB(1,1),DUNB(1,1),CMIN(1,1),
     +   LD(1,1),MAXA(1,1))
      EQUIVALENCE (DROG1(1,1),DROG2(1,1),IDEN(1,1),DIST(1,1),NSMPL(1,1))
      EQUIVALENCE (DARC(1,1),DCHORD(1,1),DMINUN(1,1),DMIN(1,1),
     +   CMAX(1,1),INDX(1))
      EQUIVALENCE (JXY(1,1),D(1),P(1),COPH(1))
      EQUIVALENCE (FREQ2(1,1),NOBS(1,1,1),MATRIX(1,1),S(1,1),
     +   HPLASY(1,1),DPHEN(1,1))
      EQUIVALENCE (MINI(1),MINI2(1)),(MAXI(1),MAXI2(1))
      EQUIVALENCE (N(1),N2(1)),(NG1(1),NA(1)),(NG2(1),Q(1))
      EQUIVALENCE (  AVEN(1),   JXY(1, 1)), (AVEALL(1),   JXY(1, 2)),
     +            (HETUNB(1),   JXY(1, 3)), (HETBIA(1),   JXY(1, 4)),
     +            (HETDIR(1),   JXY(1, 5)), (POLYNO(1),   JXY(1, 6)),
     +            (POLY99(1),   JXY(1, 7)), (POLY95(1),   JXY(1, 8)),
     +            (   SEN(1),   JXY(1, 9)), ( SEALL(1),   JXY(1,10)),
     +            (SEHBIA(1), DPREV(1, 1)), (SEHUNB(1), DPREV(1, 2)),
     +            (SEHDIR(1), DPREV(1, 3))
      EQUIVALENCE (   HBI(1), NSMPL(1, 1)), (  HUNB(1), NSMPL(1, 2)),
     +            (   HDC(1), NSMPL(1, 3)), (NALLEL(1), NSMPL(1, 4))
      EQUIVALENCE (OBSHET(1),  MINA(1, 1)), (EXPHET(1),  MINA(1, 2)),
     +            (UNCEXP(1),  MINA(1, 3)), (MAXCOM(1),  MINA(1, 4)),
     +            ( JPOOL(1),  MINA(1, 5))
      EQUIVALENCE (    JX(1), KHIER(1, 1)), (    GX(1), KHIER(1, 2)),
     +            ( SMJX2(1), KHIER(1, 3))
      EQUIVALENCE ( LHIER(1,1), JXY(1, 1)), ( IHIER(1,1), DROG1(1, 1)),
     +            (  HNUM(1), DROG1(1, 4)), ( NPOPS(1), KHIER(1, 1))
      EQUIVALENCE ( JLINE(1),   JXY(1, 1)), ( NDIST(1,1), DROG1(1, 1))
      EQUIVALENCE (   EXP(1),  FREQ(1, 1)), (   OBS(1),  FREQ(1, 2)),
     +            ( OBSUM(1),  FREQ(1, 3)), (  CALC(1),  FREQ(1, 4)),
     +            (  NTOT(1),  FREQ(1, 5))
      EQUIVALENCE (NUMALL(1),TFREQ(1))
      EQUIVALENCE (  BIGP(1),  FREQ(1, 1)), (SMALLP(1),  FREQ(1, 2)),
     +            ( SMPSQ(1),  FREQ(1, 3))
      EQUIVALENCE (FISNUM(1),   JXY(1, 1)), (FISDEN(1),   JXY(1, 2)),
     +            ( FISIK(1),   JXY(1, 3)), (  FISI(1),   JXY(1, 4))
      EQUIVALENCE (FIS(1),DPREV(1,1)),(FIT(1),DARC(1,1))
      EQUIVALENCE (VAR(1),NG1(1)),(PRES(1),NG2(1))
      EQUIVALENCE (  QSUM(1,1), DROG1(1, 5)), (  NSUM(1,1), DROG1(1, 8))
      EQUIVALENCE (ITAX(1,1),KHIER(1,1)),(CD(1),MINI(1))
      EQUIVALENCE (CUSE(1),MAXI(1)),(TERMB(1),N(1))
      EQUIVALENCE (    WT(1),   JXY(1, 3)), (DELETD(1),   JXY(1, 4)),
     +            (    NN(1),   JXY(1, 5))
      EQUIVALENCE (   ANC(1), DROG1(1, 2)), ( NCONN(1,1), DROG1(1, 4)),
     +            ( NTERM(1),  DARC(1, 1)), (ANCTOP(1),  DARC(1, 3)),
     +            (ANCPRV(1),  DARC(1, 5))
      EQUIVALENCE (INAME(1),FREQ2(1,1)),(NWT(1),ANCTOP(1))
      EQUIVALENCE (ICLINE(1,1),  DARC(1, 1)),( ICBEG(1,1),  DARC(1, 3)),
     +            ( ICEND(1,1),  DARC(1, 5)),(  NPOS(1),  DARC(1, 7)),
     +            ( NCENT(1),  DARC(1, 8)),  ( NLINE(1),  DARC(1, 9)),
     +            (NORDER(1),  DARC(1,10)),  (LINKED(1),  DARC(1,11))

      DATA INTST/'NOTU','NLOC','NALL'/,LOGST/'CRT','FULLSKIP'/

*--------------------------------------------------------------------*
*  Set IBM to .TRUE. if program is to be compiled under VS-Fortran   *
*  on an IBM mainframe, to .FALSE. otherwise                         *
*--------------------------------------------------------------------*
      DATA IBM/.FALSE./

*--------------------------------------------------------------------*
*  Set PRIME to .TRUE. if program is to be compiled under the PRIME  *
*  F77 compiler, to .FALSE. otherwise.  ALSO, change all I/O         *
*  references to unit 1 to unit 5 and unit 2 to unit 6 (READs,       *
*  WRITEs, and REWINDS) before compiling on the PRIME.               *
*--------------------------------------------------------------------*
      DATA PRIME/.FALSE./

*--------------------------------------------------------------------*
*  Set IBMPC to .TRUE. if program is to be compiled under Microsoft  *
*  Fortran (4.01 or higher) on the IBM PC, to .FALSE. otherwise.     *
*--------------------------------------------------------------------*
      DATA IBMPC/.FALSE./

*--------------------------------------------------------------------*
*  Set GNU77 to .TRUE. if program is to be compiled under GNU        *
*  Fortran, to .FALSE. otherwise.                                    *
*--------------------------------------------------------------------*
      DATA GNU77/.TRUE./

      IF (IBMPC) THEN
         OPEN (1, FILE=' ')
         OPEN (2, FILE=' ')
      ENDIF

      IF (.NOT. IBM) THEN
         DO 500 I=14,19
            OPEN (I,STATUS='SCRATCH',FORM='UNFORMATTED')
500      CONTINUE
      ENDIF
      NOHIER=1
      NOCOEF=1
      NSING=0

*     MS-Fortran chokes on attempt to rewind printer or console,
*     but the rewind isn't needed in any case.  It IS needed on
*     some of the other systems, however.
*      IF (.NOT.IBMPC) THEN
*         REWIND 1
*         REWIND 2
*      ENDIF
      CALL GETDT (DATE,TIME)
      CALL PAGSKP
      WRITE (6,200) DATE,TIME
      READ (5,'(A72)',END=23) TITLE
      WRITE (6,201) TITLE


*  Options and parameters:
*     NOTU   = number of populations (OTU's)
*     NLOC   = number of loci
*     NALL   = maximum number of alleles per locus (default = mxa)
*     CRT    : restrict output to 80 columns
*     New option added with PC version:
*     FULSKP : output form feed for page skips rather than just
*              skipping a few lines
*

*  Defaults:

      INTVL(1)=0
      INTVL(2)=0
      INTVL(3)=MXA
      LOGVL(1)=.FALSE.
      LOGVL(2)=.FALSE.
      
      CALL READER (3,0,2,INTST,REALST,LOGST,INTVL,REALVL,LOGVL,
     +             IERROR)
      IF (IERROR.EQ.1) STOP
      NOTU=INTVL(1)
      NLOC=INTVL(2)
      NALL=INTVL(3)
      CRT=LOGVL(1)
      FULSKP=LOGVL(2)

*  Error check:

      IERROR=0
      IF (NOTU.LE.1) THEN
         WRITE (6,300) NOTU
         IERROR=1
      ELSE IF (NOTU.GT.MXT) THEN
         WRITE (6,301) INTST(1)
         IERROR=1
      ELSE IF (NLOC.LE.0) THEN
         WRITE (6,302)
         IERROR=1
      ELSE IF (NLOC.GT.MXL) THEN
         WRITE (6,301) INTST(2)
         IERROR=1
      ELSE IF (NALL.GT.MXA) THEN
         WRITE (6,301) INTST(3)
         IERROR=1
      ENDIF
      IF (IERROR.EQ.1) STOP

      WRITE (6,202) NOTU,NLOC,NALL
      IF (CRT) WRITE (6,203)

*  Parameters for variable dimensioning:

      MA=NALL
      MA2=MA+MA
      ML=NLOC
      MT=NOTU
      MT2=MT+MT
      MCOM=MXC
      MHLFMT=MT2*(MT2+1)/2
      NOTU2=MT2

*------------------------------------------------------------------*
*     Record lengths for direct access files are calculated        *
*     below in single-precision words.  If your compiler's OPEN    *
*     statement expects something else, multiply by the necessary  *
*     factor.                                                      *
*------------------------------------------------------------------*
      DWRECL=2*NOTU*NOTU+3*NOTU
      DARECL=2*NALL
      IF (IBM .OR. IBMPC) THEN
*        ( IBM VS-Fortran and MS-Fortran want RECL in bytes )
         DWRECL=DWRECL*4
         DARECL=DARECL*4
      ELSE IF (PRIME) THEN
*        ( PRIME wants RECL in 16-bit words )
         DWRECL=DWRECL*2
         DARECL=DARECL*2
      ELSE IF (GNU77) THEN
*        this addition was key. 
         DWRECL=DWRECL*4
         DARECL=DARECL*4
      ENDIF

*  Locus names:

      READ (5,'(A80)') FMT
      READ (5,FMT) (LOCNAM(J),J=1,NLOC)

*  Calls to "step" subroutines:

  6   READ (5,101,IOSTAT=IOS) WORD1,WORD2
      IF (IOS.LT.0) STOP
      IF (WORD1 .NE. 'STEP') GO TO 6

      IF (WORD2.EQ.'DATA') THEN
         CALL DATIN (POPNAM,LOCNAM,NOBS,FREQ,TFREQ,MINA,MAXA,NSMPL,
     +      MINI,MAXI,N,FREQ2,MINI2,MAXI2,N2,FRQHOM,MAXIT,NG1,NG2,
     +      POPNUM,NA,Q,MG1,MG2,NIND,KHIER,IGRP,MA,MA2,ML,MT,MCOM,
     +      CG1,CG2,CMG1,CMG2,CNA,DARECL)
      ELSE IF (WORD2.EQ.'VARI') THEN
         CALL VARIAB (POPNAM,POPNUM,LOCNAM,FREQ,MINI,MAXI,N,NOBS,
     +      HBI,HUNB,HDC,NALLEL,AVEN,AVEALL,HETUNB,HETBIA,HETDIR,
     +      POLYNO,POLY99,POLY95,SEN,SEALL,SEHBIA,SEHUNB,SEHDIR,
     +      MT,ML,MA)
      ELSE IF (WORD2.EQ.'SIMD') THEN
         CALL SIMDIS (JXY,DSING,DROG1,DROG2,DARC,DCHORD,DPREV,
     +      DEDW,SROG,IUNB,DUNB,IDEN,DIST,DMINUN,DMIN,FREQ2,
     +      MINI2,MAXI2,N2,JX,GX,SMJX2,SMGX2,EDJX2,POPNAM,MA,MT,0)
      ELSE IF (WORD2.EQ.'BOOT') THEN
         CALL BOOTDIST (BDIST,SDIST,ADIST,FREQ2,MINI2,MAXI2,N2,JX,
     +   GX,SMJX2,SMGX2,EDJX2,POPNUM,ML,MA,MT)
      ELSE IF (WORD2.EQ.'COEF') THEN
         CALL MATOUT (MATRIX,POPNAM,MT)
      ELSE IF (WORD2.EQ.'SING') THEN
         CALL SINGLE (MATRIX,POPNAM,LOCNAM,JXY,DSING,DROG1,DROG2,
     +      DARC,DCHORD,DPREV,DEDW,SROG,IUNB,DUNB,IDEN,DIST,DMINUN,
     +      DMIN,FREQ2,MINI2,MAXI2,N2,JX,GX,SMJX2,SMGX2,EDJX2,
     +      MA,ML,MT)
      ELSE IF (WORD2.EQ.'HDYW') THEN
         CALL HDYWBG (POPNAM,LOCNAM,POPNUM,FREQ,MINI,MAXI,N,NOBS,
     +      OBSHET,EXPHET,UNCEXP,MAXCOM,JPOOL,MA,ML,MT)
      ELSE IF (WORD2.EQ.'CLUS') THEN
         CALL CLUSTR (POPNAM,S,COPH,HPLASY,WT,DELETD,
     +      NN,ITAX,INAME,CD,CUSE,TERMB,IVECT,ICLINE,
     +      ICBEG,ICEND,NPOS,NCENT,NLINE,NORDER,LINKED,
     +      MHLFMT,MT)
      ELSE IF (WORD2.EQ.'HIER') THEN
         CALL HRCHY (IHIER,KHIER,LHIER,NAME,POPNAM,HNUM,MT)
      ELSE IF (WORD2.EQ.'SIMA') THEN
         CALL SIMAVE (NAME,MATRIX,CAVE,CMIN,CMAX,IHIER,NPOPS,HNUM,MT)
      ELSE IF (WORD2.EQ.'DIST') THEN
         CALL AYAGRM (DSING,LD,JXY,DROG1,DROG2,DARC,DCHORD,DPREV,
     +      DEDW,SROG,IUNB,DUNB,IDEN,DIST,DMINUN,DMIN,FREQ2,MINI2,
     +      MAXI2,N2,JX,GX,SMJX2,SMGX2,EDJX2,IHIER,HNUM,
     +      NDIST,POPNAM,MA,MT)
      ELSE IF (WORD2.EQ.'HETX') THEN
         CALL HETXSQ (FREQ2,MINI2,MAXI2,N2,OBSUM,NUMALL,LOCNAM,
     +      EXP,OBS,CALC,POPNAM,NAME,IHIER,HNUM,MA,ML,MT)
      ELSE IF (WORD2.EQ.'DISW') THEN
         CALL DISWAG (D,P,DPHEN,HPLASY,POPNAM,ANC,NTERM,
     +      TERMB,NCONN,NWT,IGRP,ITAX,CD,CUSE,
     +      IVECT,ICLINE,ICBEG,ICEND,NPOS,NCENT,NLINE,
     +      NORDER,LINKED,MT,MT2,MHLFMT,ANCTOP,ANCPRV,DWRECL)
      ELSE IF (WORD2.EQ.'FSTA') THEN
         CALL FSTAT4 (FREQ2,FREQ,FRQHOM,BIGP,SMALLP,SMPSQ,FISNUM,
     +    FISDEN,FISIK,FISI,FIS,FIT,FST,MINI,MAXI,MINI2,MAXI2,N,
     +    N2,NOBS,LOCNAM,PRES,VAR,MA,ML,MT,WGT,POPCNT,AL,BL,CL,
     +    CFJK,TJK,XFJK,FL,THETL,CAPFL,HETER)
      ELSE IF (WORD2.EQ.'WRIG') THEN
         CALL WRIGHT (FREQ2,QSUM,VC,F,MINI2,MAXI2,N2,IHIER,
     +      HNUM,NSUM,PRES,LOCNAM,MA,ML,MT)
      ELSE
         WRITE (6,303) WORD2
      ENDIF
      GO TO 6
 23   WRITE (6,304)
      STOP

101   FORMAT (A4,1X,A4)
C$$$200   FORMAT (////////T3,8("B"),T14,7("I"),T24,7("O"),T35,8("S"),
C$$$     +T44,3("Y"),T52,3("Y"),T57,8("S"),/T3,9("B"),T14,
C$$$     +7("I"),T23,9("O"),T34,9("S"),T45,3("Y"),T51,3("Y"),T56,9("S"),
C$$$     +/T3,3("B"),T9,3("B"),T16,3("I"),T23,3("O"),T29,
C$$$     +3("O"),T34,3("S"),T46,3("Y"),T50,3("Y"),T56,3("S"),/T3,
C$$$     +3("B"),T9,3("B"),T16,3("I"),T23,3("O"),T29,3("O"),T34,3("S"),T47,
C$$$     +5("Y"),T56,3("S"),/T3,8("B"),T16,3("I"),T23,3("O"),
C$$$     +T29,3("O"),T34,8("S"),T48,3("Y"),T56,8("S"),
C$$$     +/T3,8("B"),T16,3("I"),T23,3("O"),T29,3("O"),T35,8("S"),
C$$$     +T48,3("Y"),T57,8("S"),/T3,3("B"),T9,
C$$$     +3("B"),T16,3("I"),T23,3("O"),T29,3("O"),T40,3("S"),T48,
C$$$     +3("Y"),T62,3("S"),/T3,3("B"),T9,3("B"),T16,3("I"),
C$$$     +T23,3("O"),T29,3("O"),T40,3("S"),T48,3("Y"),T62,3("S"),
C$$$     +/T3,9("B"),T14,7("I"),T23,9("O"),T34,9("S"),T48,3("Y"),
C$$$     +T56,9("S"),/T3,8("B"),T14,7("I"),T24,7("O"),T34,
C$$$     +8("S"),T48,3("Y"),T56,8("S")///T3,
C$$$     +"Release 2.0",7X,"Colorado State University",8X,A12,5X,A12///)
 200  FORMAT (/'BIOSYS'/'Release 2.0',7X,'Colorado State University',8X,
     +        A12,5X,A12/'Modified for compilation in GNU Fortran by',
     +        /'Mark A. Jensen, Fortinbras Research')
201   FORMAT (' Title: ',A72)
202   FORMAT (/' Number of populations (OTU''s) =',I5/
     +         ' Number of loci =',I5/
     +         ' Maximum number of alleles per locus =',I5)
203   FORMAT (/' Output restricted to width of 80 columns')
300   FORMAT (' ERROR: Illegal value for NOTU: ',I5/)
301   FORMAT (' ERROR: ',A4,' Exceeds program dimensions'/)
302   FORMAT (' ERROR: Number of loci (NLOC) not specified'/)
303   FORMAT (////' ERROR: Unrecognized step name.  First 4 characters a
     +re "',A4,'"'//)
304   FORMAT (/' ERROR: Data file empty')
 305  FORMAT (/' Dude, \'sup?',A)
      END


      BLOCK DATA

      CHARACTER LETTER,COEF*48
      COMMON/SYMB/LETTER(26)
      COMMON/MAT/MATFIL(13),NOCOEF,NMAT,NSING
      COMMON/COEFNM/COEF(13)
      DATA LETTER/'A','B','C','D','E','F','G','H','I','J','K','L','M',
     +            'N','O','P','Q','R','S','T','U','V','W','X','Y','Z'/
      DATA COEF/
     +  'Nei (1978) unbiased genetic identity',
     +  'Nei (1978) unbiased genetic distance',
     +  'Nei (1972) genetic identity',
     +  'Nei (1972) genetic distance',
     +  'Nei (1978) unbiased minimum distance',
     +  'Nei (1972) minimum distance',
     +  'Rogers (1972) genetic similarity',
     +  'Rogers (1972) genetic distance',
     +  'Modified Rogers distance (Wright,1978)',
     +  'Prevosti distance (Wright,1978)',
     +  'Cavalli-Sforza & Edwards (1967) chord distance',
     +  'Cavalli-Sforza & Edwards (1967) arc distance',
     +  'Edwards (1971,1974) "E" distance'/,
     +   nsing/0/
      END
