      SUBROUTINE GOBBLE (STRING,NONBLK)

*  If NONBLK = TRUE on input, deletes all characters in "STRING"
*  up to first non-comma, non-blank character (characters in "STRING"
*  are shifted left with blank fill on right).  if NONBLK = FALSE
*  on input, all characters prior to first comma or blank are
*  deleted.

      IMPLICIT INTEGER (A-Z)
      CHARACTER STRING*(*),C1*1
      LOGICAL NONBLK

      IF (NONBLK) THEN
         IF (STRING.EQ.' ') RETURN
         LENSTR=LEN(STRING)
         I=1
  1      IF (I.LE.LENSTR) THEN
            C1= STRING(I:I)
            IF (C1.EQ.',' .OR. C1.EQ.' ') THEN
               I=I+1
               GO TO 1
            ENDIF
            CALL SHFTCH (STRING,I)
         ELSE
            STRING=' '
         ENDIF

      ELSE

*        Find first blank or comma and shift. (assumes that "STRING"
*        does indeed contain a blank or comma -- caller should check
*        if this is not known in advance.)

         IPOS=INDEX(STRING,' ')
         JPOS=INDEX(STRING,',')
         KPOS=MIN(IPOS,JPOS)
         IF (KPOS.EQ.0) KPOS=MAX(IPOS,JPOS)
         CALL SHFTCH (STRING,KPOS)
      ENDIF
      RETURN
      END


      SUBROUTINE SHFTCH (STRING,I)

*  Shifts characters in "STRING" I-1 positions to left, with
*  blank fill on right.  Although "STRING=STRING(I:80)" works on
*  most systems, it is technically nonstandard, hence this routine.

      PARAMETER (MAXLEN=80)
      CHARACTER CTEMP*(MAXLEN),STRING*(*)

      LENSTR=LEN(STRING)
      CTEMP= STRING(I:LENSTR)
      STRING=CTEMP
      RETURN
      END


      SUBROUTINE READER (NINT,NREAL,NLOG,INTST,REALST,
     +   LOGST,INTVL,REALVL,LOGVL,ERRCOD)

*  Reads and interprets free-format parameter specification lines.

      IMPLICIT INTEGER (A-Z)
      LOGICAL LOGVL
      REAL REALVL
      CHARACTER LINE*80,KEYWRD*10,INTST*64,REALST*8,LOGST*8,
     +   FMT*10,VALUE*12
      DIMENSION INTST(*),REALST(*),LOGST(*),
     +   INTVL(*),REALVL(*),LOGVL(*)

C Needed due to MS Fortran bug:
      CHARACTER TEMP*10

      LINE=' '
      ERRCOD=0

  1   IF (LINE.EQ.' ' .OR. LINE(1:1).EQ.'*') THEN
         READ (5,'(A80)',IOSTAT=IOS) LINE
         IF (IOS.LT.0) RETURN
      ENDIF
      CALL GOBBLE (LINE,.TRUE.)
      IF (LINE.EQ.' ') GO TO 1
      IF (LINE(1:1).EQ.';') RETURN
      KEYWRD=' '
      VALUE=' '
      CALL GETSTR (LINE,KEYWRD,LENKEY,ERRCOD)
      CALL SHFTCH (LINE,LENKEY+1)
      IF (ERRCOD.NE.0) THEN
         ERRCOD=2
         GO TO 3
      ENDIF

*  Integer-valued parameter:

      DO 500 I=1,NINT
         IF (KEYWRD(1:LENKEY).EQ.INTST(I)(1:LENKEY)) THEN
            CALL GETVAL (LINE,VALUE,LENVAL,ERRCOD)
            IF (ERRCOD.GT.0) GO TO 3
* kludge: look for 'PHYF' and make sure it's formatted
* as character...(so the API remains unchanged)
            IF (KEYWRD(1:LENKEY).EQ.'PHYF') THEN
               WRITE (FMT,410) LENVAL
            ELSE
               WRITE (FMT,400) LENVAL
            ENDIF
 410        FORMAT ('(A',I2,')')
400         FORMAT ('(I',I2,')')
* kludge: put the .phy filename in the INTST array
            IF (KEYWRD(1:LENKEY).EQ.'PHYF') THEN
               READ (VALUE,FMT,ERR=2) INTST(I)
            ELSE
               READ (VALUE,FMT,ERR=2) INTVL(I)
            ENDIF

            GO TO 1
         ENDIF
500   CONTINUE

*  Real-valued parameter:

      DO 501 I=1,NREAL
         IF (KEYWRD(1:LENKEY).EQ.REALST(I)(1:LENKEY)) THEN
            CALL GETVAL (LINE,VALUE,LENVAL,ERRCOD)
            IF (ERRCOD.GT.0) GO TO 3
            WRITE (FMT,401) LENVAL
401         FORMAT ('(F',I2,'.0)')
            READ (VALUE,FMT,ERR=2) REALVL(I)
            GO TO 1
         ENDIF
501   CONTINUE

*  Logical-valued parameter:

      DO 502 I=1,NLOG
         IF (KEYWRD(1:LENKEY).EQ.LOGST(I)(1:LENKEY)) THEN
            LOGVL(I)=.TRUE.
            GO TO 1
         ELSE IF (LENKEY.GT.2) THEN
C Bug in MS Fortran -- expression below always evaluates to TRUE
C           IF (KEYWRD(1:LENKEY).EQ.'NO'//LOGST(I)(1:LENKEY-2)) THEN
C So do this instead...
            TEMP = 'NO'//LOGST(I)(1:LENKEY-2)
            IF (KEYWRD(1:LENKEY).EQ.TEMP) THEN
C (end bug workaround)
               LOGVL(I)=.FALSE.
               GO TO 1
            ENDIF
         ENDIF
502   CONTINUE


*  No match found:

      ERRCOD=2
      GO TO 3

*  Error messages:

  2   ERRCOD=3

  3   CONTINUE
      IF (ERRCOD.EQ.1) THEN
         WRITE (6,300) KEYWRD
300      FORMAT (' Missing value for keyword: ',A)
      ELSE IF (ERRCOD.EQ.2) THEN
         WRITE (6,301) KEYWRD(1:LENKEY)//LINE(1:15)
301      FORMAT (' Syntax error near: ',A)
      ELSE IF (ERRCOD.EQ.3) THEN
         WRITE (6,302) KEYWRD(1:LENKEY),VALUE(1:LENVAL)
302      FORMAT (' Illegal value: ',A,' = ',A)
      ENDIF
      RETURN
      END


      SUBROUTINE GETSTR (LINE,STRING,STRLEN,ERRCOD)

*  Isolates a character string (parameter name or value) from
*  parameter specification line

      IMPLICIT INTEGER (A-Z)
      CHARACTER LINE*(*),STRING*(*),CHAR*1
      LOGICAL SEPRTR

*  Statement function SEPRTR:

      SEPRTR(CHAR) = CHAR.EQ.',' .OR. CHAR.EQ.' ' .OR.
     +               CHAR.EQ.'=' .OR. CHAR.EQ.';'

      CALL GOBBLE (LINE,.TRUE.)
      IF (LINE.EQ.' ') RETURN
      STRLEN=LEN(STRING)
      DO 500 I=1,STRLEN
         IF (SEPRTR(LINE(I:I))) GO TO 1
500   CONTINUE
      IF (.NOT.SEPRTR(LINE(I:I))) THEN
         STRING=LINE(1:STRLEN)
         ERRCOD=1
         RETURN
      ENDIF
  1   STRLEN=I-1
      IF (STRLEN.LT.1) THEN
         STRING=LINE(1:1)
         STRLEN=1
         ERRCOD=1
      ELSE
         STRING=LINE(1:STRLEN)
      ENDIF
      RETURN
      END


      SUBROUTINE GETVAL (LINE,VALUE,LENVAL,ERRCOD)

*  Gets a character string representing a parameter value from
*  parameter specification line

      IMPLICIT INTEGER (A-Z)
      CHARACTER LINE*(*),VALUE*(*)

      ERRCOD=0
      CALL GOBBLE (LINE,.TRUE.)
      IF (LINE(1:1).NE.'=') THEN
         ERRCOD=1
         RETURN
      ENDIF
      CALL SHFTCH (LINE,2)
      CALL GETSTR (LINE,VALUE,LENVAL,ERRCOD)
      IF (ERRCOD.NE.0) THEN
         ERRCOD=3
         RETURN
      ENDIF
      IF (VALUE.EQ.' ') THEN
         ERRCOD=1
      ELSE
         CALL SHFTCH (LINE,LENVAL+1)
      ENDIF
      RETURN
      END


      SUBROUTINE SWITCH (I,J,K,L)

      IF (I.GE.J) THEN
         M=J
         K=I
         L=M
      ELSE
         M=I
         K=J
         L=M
      ENDIF
      RETURN
      END


      SUBROUTINE GETDT (DAY,TIM)

*  Returns date and time as Character*12 variables.  Requires
*  local tailoring.

      CHARACTER*12 DAY,TIM

*------------------------------------------------------------------*
*  If you don't want to fool with date/time routines, just return  *
*  blank values and comment everything else out.  No date/time     *
*  information will appear in the output.                          *
*------------------------------------------------------------------*
C     DAY = ' '
C     TIM = ' '

*--------------------- Begin VM/CMS ---------------------------*
*  Note: DAYTIM is a special date/time routine provided on     *
*        BIOSYS distribution tape and only works on IBM CMS    *
C     CHARACTER DD*8,TT*8
C     CALL DAYTIM (DD,TT)
C     DAY=DD
C     TIM=TT
*---------------------- End VM/CMS ----------------------------*

*----------------------- Begin CDC ----------------------------*
C     CHARACTER DD*10,TT*10
C     CALL CLOCK (TT)
C     CALL DATE (DD)
C     DAY=DD
C     TIM=TT
*------------------------ End CDC -----------------------------*

*---------------------- Begin VAX -----------------------------*
C     CHARACTER DD*9,TT*8
C     CALL TIME (TT)
C     CALL DATE (DD)
C     TIM=TT
C     DAY=DD
*----------------------- End VAX ------------------------------*

*--------------------- Begin PRIME ----------------------------*
C     INTEGER*2 DATSTR(28),HRS,MIN,SEC
C     CALL TIMDAT (DATSTR,INTS(28))
C     WRITE (DAY,200) (DATSTR(I),I=1,3)
C 200 FORMAT (A2,'/',A2,'/',A2,4X)
C     HRS = DATSTR(4)/60
C     MIN = MOD(DATSTR(4),60)
C     SEC=DATSTR(5)
C     WRITE (TIM,201) HRS,MIN,SEC
C 201 FORMAT (I2.2,':',I2.2,':',I2.2,4X)
*---------------------- End PRIME -----------------------------*

*--------------------- Begin IBMPC ----------------------------*
C      INTEGER*2   IHR,IMIN,ISEC,I100TH,IYR,IMON,IDAY
C      CALL GETTIM(IHR,IMIN,ISEC,I100TH)
C      CALL GETDAT(IYR,IMON,IDAY)
C      WRITE (DAY,200) IDAY,IMON,IYR
C  200 FORMAT (I2.2,':',I2.2,':',I4.4,2X)
C      WRITE (TIM,201) IHR,IMIN,ISEC
C  201 FORMAT (I2.2,':',I2.2,':',I2.2,4X)
*---------------------- End IBMPC -----------------------------*

*--------------------- Begin GNU g77 /maj------------------------*
      CHARACTER DATE*12,TIME*12,ZONE*12
      CALL Date_and_Time(DATE,TIME,ZONE)
      WRITE (DAY,200) DATE(1:4),DATE(5:6),DATE(7:8)
  200 FORMAT (A4,'-',A2,'-',A2)
      WRITE (TIM,201) TIME(1:2),TIME(3:4),TIME(5:6)
  201 FORMAT (A2,':',A2,':',A2)
*---------------------- End GNU g77 -----------------------------*

*--------------------------------------------------------------*
*  If you want to provide your own date and time routines, do  *
*  it here and comment out VM/CMS, CDC, VAX, and IBMPC calls   *
*  above.                                                      *
*--------------------------------------------------------------*

      RETURN
      END


      INTEGER FUNCTION LENSTR (STR)

*   Returns the length of the character string STR after removing
*   trailing blanks.

      CHARACTER STR*(*)
      INTRINSIC LEN

      ILEN = LEN(STR)
      DO 500 I=ILEN,1,-1
         IF (STR(I:I) .NE. ' ') GO TO 1
500   CONTINUE
      LENSTR = 0
      RETURN

  1   LENSTR = I
      RETURN
      END


      SUBROUTINE PAGSKP
      LOGICAL FULSKP
      COMMON/PAGES/FULSKP
      IF (FULSKP) THEN
*-----------------------------------------------------------------*
*     Comment out one of the two statements below, depending on   *
*     whether or not output devices are expected to support       *
*     Fortran-style carriage control.  Ordinarily, mainframes     *
*     will use the '1', micros will use the ASCII formfeed.       *
*-----------------------------------------------------------------*
C        WRITE (6,'(''1'')')
         WRITE (6,'(A1)') CHAR(12)
      ELSE
         WRITE (6,'(///)')
      ENDIF
      RETURN
      END


      SUBROUTINE MATPRT
     +   (MTYPE,NBELOW,NABOVE,NAMLOC,MATRIX,POPNAM,NTRIAG,MT)

*  Prints triangular (symmetric) matrices

      CHARACTER COEF*48,NAMLOC*5,POPNAM*16,LETTER,LINE*7
      REAL MATRIX
      LOGICAL CRT
      COMMON/GBPARM/NOTU,NOTU2,NLOC,NALL,NLET,CRT,NDATYP
      COMMON/MAT/MATFIL(13),NOCOEF,NMAT,NSING
      COMMON/COEFNM/COEF(13)
      COMMON/SYMB/LETTER(26)
      DIMENSION POPNAM(MT),MATRIX(MT,MT),LINE(15)

      NCOL=15
      IF (CRT) NCOL=8

*  Headings:

      CALL PAGSKP
      IF (MTYPE.EQ.1) THEN
*        Multi-locus similarity and distance coefficients:
         WRITE (6,200)
         WRITE (6,201) COEF(NBELOW)
         IF (NABOVE.NE.0) WRITE (6,202) COEF(NABOVE)
      ELSE IF (MTYPE.EQ.2) THEN
*        Single locus coefficients:
         WRITE (6,203) COEF(NSING),NAMLOC
      ELSE IF (MTYPE.EQ.3) THEN
*        Cophenetic values:
         WRITE (6,204)
      ELSE IF (MTYPE.EQ.4) THEN
*        Homoplasy matrix;
         WRITE (6,205)
      ELSE IF (MTYPE.EQ.5) THEN
*        Phenetic distance matrix (DISWAG):
         WRITE (6,206)
      ELSE IF (MTYPE.EQ.6) THEN
*        Patristic distance matrix (DISWAG):
         WRITE (6,207)
      ENDIF

      KMIN=1
  8   KMAX=MIN(KMIN+NCOL-1,NOTU)
      IF (KMIN .NE. 1) WRITE (6,208)
      WRITE (6,209)
      WRITE (6,210) ('-------',K=KMIN,KMAX)
      WRITE (6,211) (K,K=KMIN,KMAX)
      WRITE (6,210) ('-------',K=KMIN,KMAX)
      KBEG=1
      IF (NABOVE.EQ.0) KBEG=KMIN
      DO 500 K1=KBEG,NOTU
         KEND=KMAX
         IF (NABOVE.EQ.0) KEND=MIN(K1,KMAX)
         K=0
         DO 501 K2=KMIN,KEND
            K=K+1
            IF (NTRIAG.EQ.1) THEN
               I=K1
               J=K2
            ELSE
               I=K2
               J=K1
            ENDIF
            IF (MATRIX(I,J).EQ.99.E9) THEN
               LINE(K)='  *****'
            ELSE
               WRITE (LINE(K),'(F7.3)') MATRIX(I,J)
            ENDIF
501      CONTINUE
         WRITE (6,212) K1,POPNAM(K1),(LINE(I),I=1,K)
500   CONTINUE
      WRITE (6,210) ('-------',K=KMIN,KMAX)
      WRITE (6,214)
      IF (KMAX.EQ.NOTU) RETURN
      KMIN=KMIN+NCOL
      GO TO 8

200   FORMAT (////' Matrix of genetic similarity and/or distance ',
     +'coefficients'/1X,57('*')///)
201   FORMAT (5X,'Below diagonal: ',A48)
202   FORMAT (/5X,'Above diagonal: ',A48)
203   FORMAT (////' Matrix of single-locus genetic similarity or ',
     +'distance coefficients'/1X,66('*')///5X,'Coefficient: ',A48//5X,
     +'Locus: ',A5)
204   FORMAT (////' Cophenetic matrix'/1X,17('*'))
205   FORMAT (////' Homoplasy matrix'/1X,16('*'))
206   FORMAT (////' Phenetic distance matrix'/1X,24('*'))
207   FORMAT (////' Patristic distance matrix'/1X,25('*'))
208   FORMAT (////' (Matrix continued)')
209   FORMAT (//)
210   FORMAT (1X,25('-'),15A7)
211   FORMAT (5X,'Population',7X,15I7)
212   FORMAT (/1X,I3,1X,A16,2X,15A7)
214   FORMAT (//////)
      END


      FUNCTION FCHSQ (X,N)

*  Distribution function, evaluated at X, of the chi-square distribution
*  with N degrees of freedom.  Modified from I.D. Hill and M.C. Pike,
*  Communications of the ACM, 10:243-244(1967).

      DOUBLE PRECISION A,XA,Y,S,Z,E,C,G
      LOGICAL ODD,BIGX
      PARAMETER (UNDLIM=200.,ZERO=1.E-6)

      IF (X.LE.ZERO) THEN
         FCHSQ=0.
         RETURN
      ENDIF
      BIGX=.FALSE.
      IF (X.GT.UNDLIM) BIGX=.TRUE.
      ODD=2*(N/2) .NE. N
      A=X/2.D0
      IF (N.LE.2) THEN
         IF (.NOT. BIGX) THEN
            IF (ODD) THEN
               FCHSQ=1.D0-2.D0*FNRML(-SQRT(X))
            ELSE
               FCHSQ=1.D0-DEXP(-A)
            ENDIF
            RETURN
         ENDIF
         FCHSQ=1.0
         RETURN
      ENDIF
      XA=DBLE(N-1)/2.D0
      IF (N.LE.30) THEN
         IF (.NOT. BIGX) THEN
            Y=DEXP(-A)
            IF (ODD) THEN
               S=2.D0*FNRML(-SQRT(X))
               Z=-.5D0
               E=.564189583548/DSQRT(A)
*              ( Note: 0.564189583548 = 1/sqrt(pi) )
            ELSE
               S=Y
               Z=0.D0
               E=1.D0
            ENDIF
            C=0.D0

  8         Z=Z+1.D0
            IF (Z.LT.XA) THEN
               E=E*A/Z
               C=C+E
               GO TO 8
            ENDIF
            FCHSQ=1.D0-(C*Y+S)
         ELSE
            IF (ODD) THEN
               S=2.D0*FNRML(-SQRT(X))
               Z=-.5D0
               E=0.572364942925D0
*              ( Note: 0.572364942925 = ln(sqrt(pi) )
            ELSE
               S=0.D0
               Z=0.D0
               E=0.D0
            ENDIF
            C=DLOG(A)

 13         Z=Z+1.D0
            IF (Z.LT.XA) THEN
               E=DLOG(Z)+E
               S=DEXP(C*Z-A-E)+S
               GO TO 13
            ENDIF
            FCHSQ=1.D0-S
         ENDIF
         RETURN
      ENDIF

*  Peizer and Pratt (1968: J. Amer. Stat. Assoc. 63:1416-1456) normal
*  approximation for chi-square distribution, as implemented by Lozy
*  (1976: ACM Trans. Math. Doftware 2:393-395).
*
      S=N-1
      D=X-N+.666666666667-0.08D0/N
      ZARG=D*DSQRT((1.D0+G(S/X))/(2.D0*X))
      FCHSQ=FNRML(ZARG)
      RETURN
      END


      FUNCTION FNRML (Z)

*  Distribution function, evaluated at z, of the standard normal
*  distribution.  Modified from I.D. Hill and S.A. Joyce, Communications
*  of the ACM, 10:374-375 (1967).

      DOUBLE PRECISION X,X2,AN,Y,P1,P2,Q1,Q2,S,AM,T
      LOGICAL UPPER
      PARAMETER (ZERO=1.E-6)

      UPPER=Z.GT.0.D0
      IF (ABS(Z).LE.ZERO) THEN
         FNRML=.5D0
         RETURN
      ENDIF
      X=DBLE(ABS(Z))
      X2=X*X
      IF (X2.GE.200.) THEN
         FNRML=0.D0
         IF (UPPER) FNRML=1.D0
         RETURN
      ENDIF
      Y=.3989422804014*DEXP(-.5D0*X2)
*     ( Note: .3989422804014 = 1/sqrt(2*pi) )
      AN=Y/X
      IF (DABS(AN).LE.ZERO) THEN
         FNRML=0.D0
         IF (UPPER) FNRML=1.D0
         RETURN
      ENDIF
      TEST=3.5
      IF (UPPER) TEST=2.32
      IF (X.GT.TEST) THEN
         Q1=X
         P2=Y*X
         AN=1.D0
         P1=Y
         Q2=X2+1.D0
         S=P1/Q1
         AM=S
         T=P2/Q2

  4      IF (DABS(AM-T).GT.ZERO .AND. DABS(S-T).GT.ZERO) THEN
            AN=AN+1.D0
            S=X*P2+AN*P1
            P1=P2
            P2=S
            S=X*Q2+AN*Q1
            Q1=Q2
            Q2=S
            S=AM
            AM=T
            T=P2/Q2
            GO TO 4
         ENDIF
         FNRML=T
         IF (UPPER) FNRML=1.D0-T
         RETURN
      ENDIF
      S=Y*X
      X=S
      AN=1.D0
      T=0.D0

  7   IF (DABS(T-S).GT.ZERO) THEN
         AN=AN+2.D0
         T=S
         X=X*X2/AN
         S=S+X
         GO TO 7
      ENDIF
      REM=.5D0-S
      FNRML=REM
      IF (UPPER) FNRML=1.D0-REM
      RETURN
      END


      DOUBLE PRECISION FUNCTION G (T)

*  Function used in Peizer and Pratt (1968) normal approximation
*  to chi-square distribution function

      DOUBLE PRECISION T

      IF (DABS(1.D0 - T).GE.0.1D0) THEN
         G=(1.D0 - T*T + 2.D0 * T * DLOG(T)) / ((1.D0 - T)**2)
         RETURN
      ENDIF
      IF (T.NE.1.) THEN
         G=0.D0
         DO 500 J=1,5
            G=G+(2.D0 * (1.D0-T)**J) / ((J+1) * (J+2))
500      CONTINUE
         RETURN
      ENDIF
      G=0.D0
      RETURN
      END
