      FUNCTION MDEX (I,J)

*  Calculates address of M(i,j) where M is an NOTU*2 x NOTU*2 symmetric
*  matrix (only lower triangle is stored)

      COMMON/GBPARM/NOTU,NOTU2,NLOC,NALL,NLET,CRT,NDATYP

      IF (I.GE.J) THEN
         MDEX=(NOTU2*2-J)*(J-1)/2+I
      ELSE
         MDEX=(NOTU2*2-I)*(I-1)/2+J
      ENDIF
      RETURN
      END


      SUBROUTINE FIT
     +   (P,DPHEN,HPLASY,POPNAM,NPLASY,MHLFMT,MT)

*  Calculates statistics for goodness of fit of phenograms
*  and Wagner networks to corresponding phenetic distance matrices

      CHARACTER POPNAM*16
      REAL P,DPHEN,HPLASY
      LOGICAL NPLASY
      COMMON/GBPARM/NOTU,NOTU2,NLOC,NALL,NLET,CRT,NDATYP
      DIMENSION POPNAM(MT),P(MHLFMT),DPHEN(MT,MT),HPLASY(MT,MT)
      LOGICAL FULSKP
      COMMON/PAGES/FULSKP
      DATA ZERO/1.E-6/

      SSX1=0.
      SSX2=0.
      SX1X2=0.
      SUMX=0.
      SUMY=0.
      FARRF=0.
      FMSS=0.
      HPLASY(1,1)=0.
      DO 500 I=2,NOTU
         HPLASY(I,I)=0.
         DO 501 J=1,I-1
            DIJ=DPHEN(I,J)
            PIJ=P(MDEX(I,J))
            SSX1=SSX1+DIJ*DIJ
            SSX2=SSX2+PIJ*PIJ
            SX1X2=SX1X2+DIJ*PIJ
            SUMX=SUMX+DIJ
            SUMY=SUMY+PIJ
            HPLSY=PIJ-DIJ
            IF (.NOT. ABS(HPLSY).GT.ZERO) HPLSY=0.
            FARRF=FARRF+ABS(HPLSY)
            IF (DIJ .NE. 0) FMSS=FMSS+(HPLSY*100./DIJ)**2
            HPLASY(J,I)=HPLSY
501      CONTINUE
500   CONTINUE
      IF (NPLASY) CALL MATPRT (4,0,0,'DUMMY',HPLASY,POPNAM,2,MT)
      CALL PAGSKP
      WRITE (6,200)
      N=NOTU*(NOTU-1)/2
      SUMXSQ=SSX1-SUMX*SUMX/N
      SUMYSQ=SSX2-SUMY*SUMY/N
      SUMXY=SX1X2-SUMX*SUMY/N
      CPCC=SUMXY/SQRT(SUMXSQ*SUMYSQ)
      PRAGF=100.*FARRF/SUMX
      FMSD=SQRT(FMSS/(N-1))
      WRITE (6,201) FARRF,PRAGF,FMSD,CPCC
      RETURN
200   FORMAT (////' Goodness of fit statistics'/1X,26('*'))
201   FORMAT (///5X,'Farris (1972) "f" =',F10.3//
     +5X,'Prager and Wilson (1976) "F" =',F7.3//
     +5X,'Percent standard deviation (Fitch and Margoliash,1967) =',
     +F7.3//5X,'Cophenetic correlation =',F5.3)
      END


      SUBROUTINE TRPRIN
     +   (NH,IWAG,PATHLF,PDSUM,LINKED,POPNAM,TERMB,ITAX,CD,CUSE,
     +    IVECT,ICLINE,ICBEG,ICEND,NPOS,NCENT,NLINE,NORDER,MT)

*   Plots a phenogram or Wagner tree on lineprinter

*   If clustering is high, maximum value of coefficients is assumed to
*   be 1 unless a wagner tree is to be plotted, in which case maximum
*   value is half maximum length of tree and minimum value is assumed
*   to be zero.  If clustering is low, minimum value of coefficients
*   is assumed to be zero.

      CHARACTER LETTER,POPNAM*16,LINE*101,DUM*80,DATE*12,TIME*12,
     +   TITLE*72,FMT*80
      REAL TERMB,CD,CUSE,XNUM
      INTEGER IVECT,ICLINE,ICBEG,ICEND,NPOS,NCENT,NLINE,NORDER,
     +   LINKED,ITAX
      LOGICAL WAGNER,ADD1,ADD2,ADD3,ADD4,CRT
      COMMON/GBPARM/NOTU,NOTU2,NLOC,NALL,NLET,CRT,NDATYP
      COMMON/SYMB/LETTER(26)
      COMMON/CHARV/DUM,DATE,TIME,TITLE,FMT
      DIMENSION POPNAM(MT),TERMB(MT),CD(MT),CUSE(MT),XNUM(11),
     +   IVECT(101),ICLINE(MT,2),ICBEG(MT,2),ICEND(MT,2),
     +   NPOS(MT),NCENT(MT),NLINE(MT),NORDER(MT),LINKED(MT),ITAX(MT,2)

*   Following five parameters fix the size of the graph:
*   BASE=100 for lineprinter; BASE=60 for CRT.

      BASE=100.
      IF (CRT) BASE=60.
      IBASE=BASE
      INTER=IBASE/10
      ISPACE=IBASE+1
      INCR=(IBASE/10)+1

      NM1=NOTU-1
      WAGNER=.FALSE.
      IF (IWAG.EQ.1) WAGNER=.TRUE.
      IF (NH.NE.1) THEN
         DO 500 I=1,NM1
            CUSE(I)=CD(I)
500      CONTINUE
      ELSE
*        Take complement of cd(i) if clustering high:
         CON=1.
         IF (WAGNER) CON=PATHLF
         DO 501 I=1,NM1
            CUSE(I)=CON-CD(I)
501      CONTINUE
      ENDIF

      IF (WAGNER) THEN
         SCRAN=PATHLF
         BEG=0.
      ELSE
         RANGE=-99.E9
         DO 502 I=1,NM1
            RANGE=AMAX1(RANGE,CUSE(I))
502      CONTINUE
         IF (RANGE.LE.0.10) THEN
            SCRAN=.10
         ELSE
            SCRAN=1.
  5         IF (RANGE.GT.SCRAN) THEN
               SCRAN=SCRAN+1.
               GO TO 5
            ENDIF
            XINC=.20
            SCRAN=SCRAN-1.
  7         SCRAN=SCRAN+XINC
               IF (RANGE.GE.SCRAN) GO TO 7
         ENDIF
      ENDIF
      TX=BASE/SCRAN
      ZINC=SCRAN/INTER
      IF (.NOT. WAGNER) THEN
         IF (NH.NE.1) THEN
            DO 503 I=1,INCR
               XNUM(I)=SCRAN-(I-1)*ZINC
503         CONTINUE
            IMOVE=(ISPACE-8)/2+2
            FMT='(///1X,               X,''Distance''/)'
            WRITE (FMT(8:22),'(I15)') IMOVE
            WRITE (6,FMT)
            GO TO 12
         ENDIF
         BEG=1.-SCRAN
      ENDIF
      DO 504 I=1,INCR
         XNUM(I)=BEG+(I-1)*ZINC
504   CONTINUE
      IF (WAGNER) THEN
         IMOVE=(ISPACE-18)/2+2
         FMT='(///1X,    X,''Distance from root''/)'
         WRITE (FMT(8:11),'(I4)') IMOVE
         WRITE (6,FMT)
      ELSE
         IMOVE=(ISPACE-11)/2+2
         FMT='(///1X,            X,''Similarity''/)'
         WRITE (FMT(8:19),'(I12)') IMOVE
         WRITE (6,FMT)
      ENDIF
 12   DO 505 I=1,NOTU
         LINKED(I)=0
505   CONTINUE
      DO 506 I=1,NM1
         IF(ITAX(I,1) .EQ. 1) THEN
            NPOS(1)=1
            LINKED(1)=1
            NPOS(2)=ITAX(I,2)
            LINKED(NPOS(2))=1
            GO TO 13
         ENDIF
506   CONTINUE
 13   DO 507 I=3,NOTU
         L=I-1
 14      NPOSL=NPOS(L)
         DO 508 K=1,2
            KDEX=3-K
            DO 509 J=1,NM1
               IF (ITAX(J,K) .EQ. NPOSL) THEN
                  L1=ITAX(J,KDEX)
                  IF (LINKED(L1).NE.1) THEN
                     NPOS(I)=L1
                     LINKED(L1)=1
                     GO TO 507
                  ENDIF
               ENDIF
509         CONTINUE
508      CONTINUE
         L=L-1
         GO TO 14
507   CONTINUE

*   Copy NPOS to NORDER for use in printing OTU names:

      DO 510 I=1,NOTU
         NORDER(I)=NPOS(I)
510   CONTINUE

      CALL TAXIS (LINE,IBASE,ISPACE)
      WRITE (6,200) (XNUM(J),J=1,INCR)
      WRITE (6,201) LINE(1:ISPACE)
      WRITE (6,202)
      DO 511 I=1,NOTU
         NLINE(NPOS(I))=2*I-1
511   CONTINUE
      DO 512 J=1,NM1
         NTOT=0
         DO 513 K=1,2
            IDEX=ITAX(J,K)
            ICBEG(J,K)=CUSE(J)*TX+.5
            IF (NPOS(IDEX).NE.0) THEN

*              OTU not used before:
                  IF (WAGNER) THEN
                     ICEND(J,K)=ICBEG(J,K)-NINT(TERMB(IDEX)*TX)
                  ELSE
                     ICEND(J,K)=0
                  ENDIF
                  ICLINE(J,K)=NLINE(IDEX)
                  NPOS(IDEX)=0

            ELSE

*              Check backward for preceding appearance of OTU:
                  J3=J
                  J1=J-1
                  DO 514 J2=1,J1
                     J3=J3-1
                     IF (IDEX.EQ.ITAX(J3,1) .OR. IDEX .EQ.
     +                  ITAX(J3,2)) GO TO 18
514               CONTINUE
                  WRITE (6,300)
                  STOP
 18               ICEND(J,K)=CUSE(J3)*TX+.5
                  ICLINE(J,K)=NCENT(J3)
            ENDIF
            NTOT=NTOT+ICLINE(J,K)
513      CONTINUE
         NCENT(J)=NTOT/2
         NLINE(ITAX(J,1))=NCENT(J)
512   CONTINUE

*   Prepare and print lines of phenogram by stepping through clustering
*   data with ICLINE, ICBEG, and ICEND data available:

      DO 515 I=1,ISPACE
         LINE(I:I)=' '
         IVECT(I)=0
515   CONTINUE
      NTLINE=2*NOTU-1
      JTAX=0
      DO 516 I=1,NTLINE
         ITERM=0
         DO 517 J=1,NM1
            DO 518 K=1,2
               IF (ICLINE(J,K) .EQ. I) THEN
                  ISTART=ISPACE-ICBEG(J,K)
                  IFIN=ISPACE-ICEND(J,K)
                  ITERM=MAX(ITERM,IFIN)
                  DO 519 L=ISTART,IFIN
                     LINE(L:L)='*'
519               CONTINUE
                  ADD1=K.NE.2
                  ADD2=ICLINE(J,1).LT.ICLINE(J,2)
                  ADD3=.NOT.ADD1
                  ADD4=.NOT.ADD2
*                 ( .EQV. not used due to bug in VAX Fortran compiler )
                  IF ((ADD1 .AND. ADD2) .OR. (ADD3 .AND. ADD4)) THEN
                     IVECT(ISTART)=IVECT(ISTART)+1
                  ELSE
                     IVECT(ISTART)=IVECT(ISTART)-1
                  ENDIF
               ENDIF
518         CONTINUE
517      CONTINUE
         DO 520 L=1,ISPACE
            IF (IVECT(L) .NE. 0) LINE(L:L)='*'
520      CONTINUE
         IF (I/2 .EQ. (I-1)/2) THEN
            JTAX=JTAX+1
            FMT='(2X,A    ,1X,A16)'
            WRITE (FMT(6:8),'(I3)') ITERM
            WRITE (6,FMT) LINE(1:ITERM),POPNAM(NORDER(JTAX))
         ELSE
            FMT='(2X,A    )'
            WRITE (FMT(6:8),'(I3)') ISPACE
            WRITE (6,FMT) LINE(1:ISPACE)
         ENDIF
         DO 521 L=1,ISPACE
            LINE(L:L)=' '
521      CONTINUE
516   CONTINUE
      CALL TAXIS (LINE,IBASE,ISPACE)
      WRITE (6,202)
      WRITE (6,201) LINE(1:ISPACE)
      WRITE (6,200) (XNUM(J),J=1,INCR)
      IF (WAGNER) WRITE (6,203) PDSUM
      RETURN

200   FORMAT (1X,F4.2,4X,10(F6.2,4X))
201   FORMAT (2X,A)
202   FORMAT (/)
203   FORMAT (///' Total length of tree =',F10.3)
300   FORMAT (' **ERROR IN CLUSTERING DATA')
      END


      SUBROUTINE TAXIS (LINE,IBASE,ISPACE)

      CHARACTER LINE*101

      DO 500 I=1,IBASE
         LINE(I:I)='-'
500   CONTINUE

      DO 501 I=1,ISPACE,5
         LINE(I:I)='+'
501   CONTINUE
      RETURN
      END


      SUBROUTINE CHANGE (I,J)
      ITEMP=I
      I=J
      J=ITEMP
      RETURN
      END
