*=======================================================================*
*                                                                       *
*                       S T E P   S I M D I S                           *
*                                                                       *
*           Genetic similarity and distance coefficients                *
*                                                                       *
*  Similarity and distance coefficients are referred to throughout the  *
*  program by the following indices:                                    *
*                                                                       *
*        1 = Nei (1978) I (unbiased)                                    *
*        2 = Nei (1978) D (unbiased)                                    *
*        3 = Nei (1972) I                                               *
*        4 = Nei (1972) D                                               *
*        5 = Nei (1978) minimum D (unbiased)                            *
*        6 = Nei (1972) minimum D                                       *
*        7 = Rogers (1972) S                                            *
*        8 = Rogers (1972) D                                            *
*        9 = modified Rogers distance (Wright, 1978)                    *
*       10 = Prevosti distance (Wright, 1978)                           *
*       11 = Cavalli-Sforza and Edwards (1967) chord distance           *
*       12 = Cavalli-Sforza and Edwards (1967) arc distance             *
*       13 = Edwards (1971,1974) "E" distance                           *
*                                                                       *
*  options:                                                             *
*     ALLCO  ("ALLCOEF") : store all 13 coefficients                    *
*     NEI                : store coefficients 1 thru 6                  *
*     ROGERS             : store coefficients 7 thru 9                  *
*     PREVOS ("PREVOSTI"): store coefficient 10                         *
*     CAVALL ("CAVALLI") : store coefficients 11 and 12                 *
*     EDWRDS ("EDWARDS") : store coefficient 13                         *
*     COFILE             : write matrices to a file (formatted) in form *
*                          used for input to NETOPT and other programs  *
*     NSING  ("SINGLE")  = 0  do not calculate and store single-locus   *
*                                coefficients (default)                 *
*                        > 0  calculate and store single-locus          *
*                                coefficients indexed by nsing          *
*                                                                       *
*  Subprograms required by step SIMDIS: SIMDIS  READER  ALLCNT          *
*                                                                       *
*=======================================================================*

      SUBROUTINE SIMDIS
     +   (JXY,DSING,DROG1,DROG2,DARC,DCHORD,DPREV,DEDW,SROG,IUNB,
     +    DUNB,IDEN,DIST,DMINUN,DMIN,FREQ2,MINI2,MAXI2,N2,JX,GX,
     +    SMJX2,SMGX2,EDJX2,POPNAM,MA,MT,NSONLY)

*  Calculates similarity and distance coefficients for pairwise
*  comparisons between populations

      CHARACTER LINE*80,DATE*12,TIME*12,TITLE*72,FMT*80,
     +   INTST*64,REALST*8,LOGST*8,COEF*48,POPNAM*16
      REAL JXY,DSING,DROG1,DROG2,DARC,DCHORD,DPREV,DEDW,SROG,IUNB,
     +   DUNB,IDEN,DIST,DMINUN,DMIN,JX,GX,SMJX2,SMGX2,EDJX2,FREQ2,
     +   JXY1,JX1,JY1,IUN,ID,REALVL
      INTEGER MINI2,MAXI2,N2,INTVL
      LOGICAL ALLCO,NEI,ROGERS,PREVOS,CAVALL,EDWRDS,COFILE,WARNED,
     +   LOGVL
      PARAMETER (ZERO=1.E-6, PI=3.14159, CFACT=2.*1.41421/PI,
     +    EFACT=CFACT*2.)
      COMMON/GBPARM/NOTU,NOTU2,NLOC,NALL,NLET,CRT,NDATYP
      COMMON/MAT/MATFIL(13),NOCOEF,NMAT,NSING
      COMMON/COEFNM/COEF(13)
      COMMON/CHARV/LINE,DATE,TIME,TITLE,FMT
      DIMENSION POPNAM(MT),INTST(1),LOGST(7),JXY(MT,MT),DSING(MT,MT),
     +   DROG1(MT,MT),DROG2(MT,MT),DARC(MT,MT),DCHORD(MT,MT),
     +   DPREV(MT,MT),DEDW(MT,MT),SROG(MT,MT),IUNB(MT,MT),DUNB(MT,MT),
     +   IDEN(MT,MT),DIST(MT,MT),DMINUN(MT,MT),DMIN(MT,MT),JX(MT),
     +   GX(MT),SMJX2(MT),SMGX2(MT),EDJX2(MT),FREQ2(MA,MT),
     +   MINI2(MT),MAXI2(MT),N2(MT),INTVL(1),REALVL(1),LOGVL(7),
     +   REALST(1)
      LOGICAL FULSKP
      COMMON/PAGES/FULSKP
      SAVE INTST,REALST,LOGST

      DATA LOGST/'ALLCOEF','NEI','ROGERS','PREVOSTI','CAVALLI',
     +   'EDWARDS','COFILE'/,INTST/'SINGLE'/,REALST/'ZZZ'/

      NOCOEF=0
      NSING=NSONLY
      IF (NSONLY.LE.0) THEN
         CALL PAGSKP
         WRITE (6,200) TITLE,DATE,TIME

*     Defaults:

         INTVL(1)=0
         DO 500 I=1,7
            LOGVL(I)=.FALSE.
500      CONTINUE

         CALL READER (1,0,7,INTST,REALST,LOGST,INTVL,REALVL,LOGVL,
     +      IERROR)
         IF (IERROR.EQ.1) RETURN
         NSING=INTVL(1)
         ALLCO=LOGVL(1)
         NEI=LOGVL(2)
         ROGERS=LOGVL(3)
         PREVOS=LOGVL(4)
         CAVALL=LOGVL(5)
         EDWRDS=LOGVL(6)
C        COFILE option no longer supported.
C        COFILE=LOGVL(7)
         COFILE=.FALSE.
         IF (NSING.LT.0 .OR. NSING.GE.14) THEN
            WRITE (6,300) NSING
            NSING=0
         ENDIF
         DO 501 I=1,6
            IF (LOGVL(I)) GO TO 2
501      CONTINUE
         WRITE (6,301)
         NOCOEF=1
         RETURN

  2      IF (ALLCO) THEN
            NEI=.TRUE.
            ROGERS=.TRUE.
            PREVOS=.TRUE.
            CAVALL=.TRUE.
            EDWRDS=.TRUE.
         ENDIF
      ENDIF
      IF (NSING.EQ.13) EDWRDS=.TRUE.

      DO 502 K=1,NOTU
         JX(K)=0.
         GX(K)=0.
         DSING(K,K)=99.E9
         DROG1(K,K)=99.E9
         DARC(K,K)=99.E9
         DPREV(K,K)=99.E9
502   CONTINUE
      DO 503 K1=2,NOTU
         DO 504 K2=1,K1-1
            JXY(K1,K2)=0.
            DROG1(K1,K2)=0.
            DROG2(K2,K1)=0.
            DPREV(K1,K2)=0.
            DEDW(K2,K1)=0.
            DARC(K1,K2)=0.
            DCHORD(K2,K1)=0.
504      CONTINUE
503   CONTINUE

*  Calculate distances:

      REWIND 16
      IF (NSING.GT.0) REWIND 19
      WARNED=.FALSE.
      DO 505 J=1,NLOC
         READ (16) FREQ2,MINI2,MAXI2,N2
         IF (EDWRDS) CALL ALLCNT (FREQ2,MINI2,MAXI2,NALLS,NOTU,MA,MT)
         DO 506 K1=1,NOTU
            MINIX=MINI2(K1)
            MAXIX=MAXI2(K1)
            NX=N2(K1)
            SMJX=0.
            EDJX=0.
            DO 507 I=MINIX,MAXIX
               QFREQ=FREQ2(I,K1)
               SMJX=SMJX+QFREQ*QFREQ
               IF (EDWRDS) EDJX=EDJX+SQRT(QFREQ/NALLS)
507         CONTINUE
            SMJX2(K1)=SMJX
            SMGX2(K1)=(2.*NX*SMJX-1.)/(2*NX-1)
            IF (SMGX2(K1).LE.ZERO) THEN
               IF (.NOT.WARNED .AND. NEI) THEN
                  WRITE (6,302)
                  WARNED=.TRUE.
               ENDIF
               SMGX2(K1)=0.5
            ENDIF
            EDJX2(K1)=EDJX
            JX(K1)=JX(K1)+SMJX
            GX(K1)=GX(K1)+SMGX2(K1)
            IF (K1.NE.1) THEN
               DO 508 K2=1,K1-1
                  MINIY=MINI2(K2)
                  MAXIY=MAXI2(K2)
                  IMIN=MIN(MINIX,MINIY)
                  IMAX=MAX0(MAXIX,MAXIY)
                  IF (MINIX .EQ. MAXIX .AND. MINIY .EQ. MAXIY) THEN
                     IF (IMIN .EQ. IMAX) THEN
*                       ( Populations K1 and K2 fixed identically )
                        SUMXY=1.
                        COSTHE=1.
                        SUM1=0.
                        SUM2=0.
                     ELSE
*                       ( Populations K1 and K2 fixed differently )
                        SUMXY=0.
                        COSTHE=0.
                        SUM1=2.
                        SUM2=2.
                     ENDIF
                  ELSE
*                    ( Populations K1 and/or K2 polymorphic )
                     SUMXY=0.
                     COSTHE=0.
                     SUM1=0.
                     SUM2=0.
                     DO 509 I=IMIN,IMAX
                        XI=FREQ2(I,K1)
                        YI=FREQ2(I,K2)
                        DIFFXY=ABS(XI-YI)
                        SUMXY=SUMXY+XI*YI
                        COSTHE=COSTHE+SQRT(XI*YI)
                        SUM1=SUM1+DIFFXY*DIFFXY
                        SUM2=SUM2+DIFFXY
509                  CONTINUE
                  ENDIF

                  DJROG=SQRT(SUM1/2.)
                  DJPREV=SUM2/2.
                  IF (COSTHE.GT.1.) COSTHE=1.
                  THETA=ACOS(COSTHE)
                  DJARC=2.*THETA/PI
                  DJCHRD=CFACT*SQRT(1.-COSTHE)
                  DJEDW=EFACT*SQRT((1.-COSTHE)/((1.+EDJX2(K1))*
     +                  (1.+EDJX2(K2))))
                  IUN=SUMXY/SQRT(SMGX2(K1)*SMGX2(K2))
                  IF (IUN.GT.1) IUN=1.
                  ID=SUMXY/SQRT(SMJX2(K1)*SMJX2(K2))

*                 Totals:

                  IF (NSONLY.LE.0) THEN
                     JXY(K1,K2)=JXY(K1,K2)+SUMXY
                     DROG1(K1,K2)=DROG1(K1,K2)+DJROG
                     DROG2(K2,K1)=DROG2(K2,K1)+DJROG*DJROG
                     DPREV(K1,K2)=DPREV(K1,K2)+DJPREV
                     DEDW(K2,K1)=DEDW(K2,K1)+DJEDW*DJEDW
                     DARC(K1,K2)=DARC(K1,K2)+DJARC*DJARC
                     DCHORD(K2,K1)=DCHORD(K2,K1)+DJCHRD*DJCHRD
                  ENDIF

*              Single-locus coefficients:

                  NSP1=NSING+1
                  IF (NSP1.EQ.2) THEN
*                    Nei I (unbiased):
                     DSING(K2,K1)=IUN
                  ELSE IF (NSP1.EQ.3) THEN
*                    Nei D (unbiased):
                     IF (IUN.LE.0.) THEN
                        DSING(K2,K1)=99.E9
                     ELSE
                        DSING(K2,K1)=-ALOG(IUN)
                     ENDIF
                  ELSE IF (NSP1.EQ.4) THEN
*                    Nei I:
                        DSING(K2,K1)=ID
                  ELSE IF (NSP1.EQ.5) THEN
*                    Nei D:
                     IF (ID.LE.0.) THEN
                        DSING(K2,K1)=99.E9
                     ELSE
                        DSING(K2,K1)=-ALOG(ID)
                     ENDIF
                  ELSE IF (NSP1.EQ.6) THEN
*                    Nei D-min (unbiased):
                     DSING(K2,K1)=(SMGX2(K1)+SMGX2(K2))/2. - SUMXY
                  ELSE IF (NSP1.EQ.7) THEN
*                    Nei D-min:
                     DSING(K2,K1)=(SMJX2(K1)+SMJX2(K2))/2. - SUMXY
                  ELSE IF (NSP1.EQ.8) THEN
*                    Rogers S:
                     DSING(K2,K1)=1.-DJROG
                  ELSE IF (NSP1.EQ.9 .OR. NSP1.EQ.10) THEN
*                    Rogers D:
                     DSING(K2,K1)=DJROG
                  ELSE IF (NSP1.EQ.11) THEN
*                    Prevosti D:
                     DSING(K2,K1)=DJPREV
                  ELSE IF (NSP1.EQ.12) THEN
*                    Cavalli-Sforza and Edwards chord:
                     DSING(K2,K1)=DJCHRD
                  ELSE IF (NSP1.EQ.13) THEN
*                    Cavalli-Sforza and Edwards arc:
                     DSING(K2,K1)=DJARC
                  ELSE IF (NSP1.EQ.14) THEN
*                    Edwards "E":
                     DSING(K2,K1)=DJEDW
                  ENDIF
508            CONTINUE
            ENDIF
506      CONTINUE

*     Write single-locus coefficients to file locus-wise:
         IF (NSING.GT.0) WRITE (19) ((DSING(K2,K1),K1=K2,NOTU),
     +      K2=1,NOTU)
505   CONTINUE

      IF (NSONLY.GT.0) RETURN

*  Distances averaged across loci:

      DO 510 K1=2,NOTU
         DO 511 K2=1,K1-1
            JXY(K1,K2)=JXY(K1,K2)/NLOC
            DROG1(K1,K2)=DROG1(K1,K2)/NLOC
            DROG2(K2,K1)=SQRT(DROG2(K2,K1)/NLOC)
            DPREV(K1,K2)=DPREV(K1,K2)/NLOC
            DEDW(K2,K1)=SQRT(DEDW(K2,K1)/NLOC)
            DARC(K1,K2)=SQRT(DARC(K1,K2)/NLOC)
            DCHORD(K2,K1)=SQRT(DCHORD(K2,K1)/NLOC)
511      CONTINUE
510   CONTINUE

*  Final computation of distances and writing to file:
*     (All matrices written column-wise as if were in lower triangle)

      NMAT=0
      REWIND 15
      IF (ROGERS) THEN
         DO 512 K1=2,NOTU
            DO 513 K2=1,K1-1
               SROG(K2,K1)=1.-DROG1(K1,K2)
513         CONTINUE
512      CONTINUE
         WRITE (15) ((SROG(K2,K1),K1=K2,NOTU),K2=1,NOTU)
         WRITE (15) ((DROG1(K1,K2),K1=K2,NOTU),K2=1,NOTU)
         WRITE (15) ((DROG2(K2,K1),K1=K2,NOTU),K2=1,NOTU)
         MATFIL(1)=7
         MATFIL(2)=8
         MATFIL(3)=9
         NMAT=3
      ENDIF
      IF (CAVALL) THEN
         NMAT=NMAT+1
         WRITE (15) ((DCHORD(K2,K1),K1=K2,NOTU),K2=1,NOTU)
         MATFIL(NMAT)=11
         NMAT=NMAT+1
         WRITE (15) ((DARC(K1,K2),K1=K2,NOTU),K2=1,NOTU)
         MATFIL(NMAT)=12
      ENDIF
      IF (PREVOS) THEN
         NMAT=NMAT+1
         WRITE (15) ((DPREV(K1,K2),K1=K2,NOTU),K2=1,NOTU)
         MATFIL(NMAT)=10
      ENDIF
      IF (EDWRDS) THEN
         NMAT=NMAT+1
         WRITE (15) ((DEDW(K2,K1),K1=K2,NOTU),K2=1,NOTU)
         MATFIL(NMAT)=13
      ENDIF
      IF (NEI) THEN
         DO 514 K1=2,NOTU
            GX1=GX(K1)/NLOC
            JX1=JX(K1)/NLOC
            DO 515 K2=1,K1-1
               JXY1=JXY(K1,K2)
               GY1=GX(K2)/NLOC
               JY1=JX(K2)/NLOC
               IUNB(K1,K2)=JXY1/SQRT(GX1*GY1)
               IF (IUNB(K1,K2).GT.1.) IUNB(K1,K2)=1.
               IF (IUNB(K1,K2).LE.0.) THEN
                    DUNB(K2,K1)=99.E9
                 ELSE
                    DUNB(K2,K1)=-ALOG(IUNB(K1,K2))
                 ENDIF
               IDEN(K1,K2)=JXY1/SQRT(JX1*JY1)
               IF (IDEN(K1,K2).LE.0.) THEN
                  DIST(K2,K1)=99.E9
               ELSE
                  DIST(K2,K1)=-ALOG(IDEN(K1,K2))
               ENDIF
               DMINUN(K1,K2)=(GX1+GY1)/2. - JXY1
               IF (DMINUN(K1,K2).LT.0.) DMINUN(K1,K2)=0.
               DMIN(K2,K1)=(JX1+JY1)/2. - JXY1
515         CONTINUE
514      CONTINUE
         WRITE (15) ((IUNB(K1,K2),K1=K2,NOTU),K2=1,NOTU)
         WRITE (15) ((DUNB(K2,K1),K1=K2,NOTU),K2=1,NOTU)
         WRITE (15) ((IDEN(K1,K2),K1=K2,NOTU),K2=1,NOTU)
         WRITE (15) ((DIST(K2,K1),K1=K2,NOTU),K2=1,NOTU)
         WRITE (15) ((DMINUN(K1,K2),K1=K2,NOTU),K2=1,NOTU)
         WRITE (15) ((DMIN(K2,K1),K1=K2,NOTU),K2=1,NOTU)
         DO 516 I=1,6
            NMAT=NMAT+1
            MATFIL(NMAT)=I
516      CONTINUE
      ENDIF

      WRITE (6,201)
      IF (COFILE) THEN
         REWIND 3
         WRITE (3,202) TITLE
         DO 517 K=1,NOTU
            WRITE (3,203) POPNAM(K)
517      CONTINUE
         REWIND 15
      ENDIF
      DO 518 I=1,NMAT
         IMAT=MATFIL(I)
         WRITE (6,204) COEF(IMAT)
         IF (COFILE) THEN
            WRITE (3,205) COEF(IMAT)
*           (JXY is used simply for temporary storage)
            READ (15) ((JXY(K1,K2),K1=K2,NOTU),K2=1,NOTU)
            DO 519 K1=1,NOTU
               IF (IMAT.NE.1 .AND. IMAT.NE.3 .AND. IMAT.NE.7) THEN
                  JXY(K1,K1)=0.
               ELSE
                  JXY(K1,K1)=1.
               ENDIF
               WRITE (3,206) (JXY(K1,K2),K2=1,K1)
519         CONTINUE
         ENDIF
518   CONTINUE
      IF (NSING.GT.0) WRITE (6,207) COEF(NSING)
      IF (COFILE) WRITE (6,208)
      RETURN
*
200   FORMAT (//////T3,76('*')/T3,'*',T78,'*'/T3,'*',2X,A72,
     +'*'/T3,'*',T78,'*'/T3,
     +'*  Genetic similarity and distance coefficient calculation',
     +T78,'*'/T3,'*',T78,'*'/T3,'*  BIOSYS-1   Release 1.7',3X,A12,3X,
     +A12,T78,'*'/T3,'*',T78,'*'/T3,76('*')///)
201   format (/' Coefficients calculated and stored for further ',
     +'operations:'/)
202   FORMAT (A72/)
203   FORMAT (A16)
204   FORMAT (/6X,A48)
205   FORMAT (/A48/'(8F10.7)')
206   FORMAT (8F10.7)
207   FORMAT (//' Single-locus coefficients kept for further ',
     +'operations: '//6X,A48)
208   FORMAT (//' Above similarity measures also written to file ',
     +'"COEFS"')
300   FORMAT (' ERROR: Illegal value for single:',I5,
     +   '. No single-locus coefficients stored'/)
301   FORMAT (' ERROR: No similarity/distance coefficients requested'/)
302   FORMAT (/' WARNING: Nei (1978) single-locus distances cannot be '
     +,'calculated for comparisons'/10X,'involving loci that are hetero'''
     +,'zgous in samples of size one.'/10X,
     +'Biased estimate (Nei, 1972) used in these cases.')
      END


      SUBROUTINE ALLCNT (FREQ2,MINI2,MAXI2,NALLS,NOTU,MA,MT)

*  Counts total number of alleles observed at a locus in all populations

      REAL FREQ2
      INTEGER MINI2,MAXI2
      DIMENSION FREQ2(MA,MT),MINI2(MT),MAXI2(MT)
      DATA ZERO/1.E-6/

      MINA=999
      MAXA=0
      DO 500 K=1,NOTU
         IF (MINI2(K).LT.MINA) MINA=MINI2(K)
         IF (MAXI2(K).GT.MAXA) MAXA=MAXI2(K)
500   CONTINUE
      NALLS=0
      DO 501 I=MINA,MAXA
         DO 502 K=1,NOTU
            IF (FREQ2(I,K).GT.ZERO) GO TO 1
502      CONTINUE
  1      NALLS=NALLS+1
501   CONTINUE
      RETURN
      END


*=======================================================================*
*                                                                       *
*                      S T E P   C O E F O U T                          *
*                                                                       *
*               Similarity and distance matrix output                   *
*                                                                       *
*                                                                       *
*  Options:                                                             *
*     BELOW = matrix to be printed below diagonal (see routine SIMDIS)  *
*     ABOVE = matrix to be printed above diagonal (see routine SIMDIS)  *
*               (above not required; lower triangle only                *
*                may be printed)                                        *
*                                                                       *
*  Subprograms required by step COEFOUT: MATOUT  READER  MATPRT
*                                                                       *
*=======================================================================*

      SUBROUTINE MATOUT (MATRIX,POPNAM,MT)

*  Loads previously calculated similarity and distance matrices into
*  "MATRIX" and calls printing routine

      CHARACTER LINE*80,DATE*12,TIME*12,TITLE*72,FMT*80,INTST*64,
     +   REALST*8,LOGST*8,COEF*48,POPNAM*16
      REAL MATRIX,REALVL
      INTEGER ABOVE,BELOW
      LOGICAL LOGVL
      COMMON/GBPARM/NOTU,NOTU2,NLOC,NALL,NLET,CRT,NDATYP
      COMMON/MAT/MATFIL(13),NOCOEF,NMAT,NSING
      COMMON/COEFNM/COEF(13)
      COMMON/CHARV/LINE,DATE,TIME,TITLE,FMT
      DIMENSION POPNAM(MT),INTST(2),REALST(1),LOGST(1),MATRIX(MT,MT),
     +   INTVL(2),REALVL(1),LOGVL(1)
      LOGICAL FULSKP
      COMMON/PAGES/FULSKP
      DATA INTST/'BELOW','ABOVE'/,LOGST/'END'/,REALST/'ZZZ'/

      CALL PAGSKP
      WRITE (6,200) TITLE,DATE,TIME

      IF (NOCOEF .EQ. 1) THEN
         WRITE (6,300)
         RETURN
      ENDIF

*  Defaults:

  2   INTVL(1)=0
      INTVL(2)=0
      LOGVL(1)=.FALSE.

      CALL READER (2,0,1,INTST,REALST,LOGST,INTVL,REALVL,LOGVL,IERROR)
      IF (IERROR.EQ.1) GO TO 2
      IF (LOGVL(1)) RETURN
      BELOW=INTVL(1)
      ABOVE=INTVL(2)
      IF (BELOW.LE.0 .OR. BELOW.GE.14) THEN
         WRITE (6,301) INTST(1),BELOW
         GO TO 2
      ENDIF
      IF (ABOVE.LT.0 .AND. ABOVE.GE.14) THEN
         WRITE (6,301) INTST(2),ABOVE
         GO TO 2
      ENDIF

*  Read coefficients:

      REWIND 15
      DO 500 J=1,NMAT
         IF (MATFIL(J).EQ.BELOW) GO TO 5
            READ (15)
500   CONTINUE
      WRITE (6,302) BELOW,COEF(BELOW)
      GO TO 2
  5   READ (15) ((MATRIX(K1,K2),K1=K2,NOTU),K2=1,NOTU)
      IF (ABOVE.NE.0) THEN
         REWIND 15
         DO 501 J=1,NMAT
            IF (MATFIL(J).EQ.ABOVE) GO TO 6
               READ (15)
501      CONTINUE
         WRITE (6,302) ABOVE,COEF(ABOVE)
         GO TO 2
  6      READ (15) ((MATRIX(K2,K1),K1=K2,NOTU),K2=1,NOTU)
      ENDIF
      CALL MATPRT (1,BELOW,ABOVE,'DUMMY',MATRIX,POPNAM,1,MT)
      GO TO 2

200   FORMAT (//////T3,76('*')/T3,'*',T78,'*'/T3,'*',2X,A72,
     +'*'/T3,'*',T78,'*'/T3,
     +'*  Genetic similarity and distance coefficient output',
     +T78,'*'/T3,'*',T78,'*'/T3,'*  BIOSYS-1   Release 1.7',3X,A12,3X,
     +A12,T78,'*'/T3,'*',T78,'*'/T3,76('*')//////)
300   FORMAT (' ERROR: Step SIMDIS has not been performed')
301   FORMAT (' ERROR: Illegal value for ',A5,':',I5/)
302   FORMAT (////' ERROR: Matrix no.',I3,' = ',A48/8X,
     +'not on file.')
      END


*=======================================================================*
*                                                                       *
*                       S T E P   S I M A V E                           *
*                                                                       *
*           Similarity and distance coefficient averaging               *
*                                                                       *
*  Averages similarity and distance coefficients between taxa of equal  *
*  rank (i.e., averages for all pairwise comparisons of populations of  *
*  one species with those of another species, etc.).  Requires a prior  *
*  performance of step HIERARCHY.                                       *
*                                                                       *
*  Options:                                                             *
*     LEVEL          = hierarchical level at which averaging is         *
*                         performed                                     *
*     NCOEF ("COEF") = similarity or distance coefficient to be used    *
*                         (see routine SIMDIS)                          *
*                                                                       *
*  Subprograms required for step SIMAVE: SIMAVE READER SWITCH           *
*                                                                       *
*=======================================================================*
*
      SUBROUTINE SIMAVE
     +   (NAME,MATRIX,CAVE,CMIN,CMAX,IHIER,NPOPS,HNUM,MT)

*  Averages similarity and distance coefficients at a given hierarchical
*  level

      CHARACTER LINE*80,DATE*12,TIME*12,TITLE*72,FMT*80,LETTER,
     +   INTST*64,REALST*8,LOGST*8,COEF*48,NAME*16,HNAME*10,
     +   LINAVE*5,LINMIN*5,LINMAX*5
      REAL MATRIX,CAVE,CMIN,CMAX,REALVL
      INTEGER IHIER,NPOPS,HNUM
      LOGICAL CRT,LOGVL,NOCOMP
      COMMON/GBPARM/NOTU,NOTU2,NLOC,NALL,NLET,CRT,NDATYP
      COMMON/MAT/MATFIL(13),NOCOEF,NMAT,NSING
      COMMON/COEFNM/COEF(13)
      COMMON/HIER1/NOHIER,NLEVEL,NROW,NHIER(4)
      COMMON/HIER/HNAME(4)
      COMMON/SYMB/LETTER(26)
      COMMON/CHARV/LINE,DATE,TIME,TITLE,FMT
      DIMENSION NAME(MT,3),INTST(2),LOGST(1),MATRIX(MT,MT),CAVE(MT,MT),
     +   CMIN(MT,MT),CMAX(MT,MT),IHIER(MT,3),NPOPS(MT),HNUM(MT),
     +   LINAVE(6),LINMIN(6),LINMAX(6),REALST(1),INTVL(2),REALVL(1),
     +   LOGVL(1)
      LOGICAL FULSKP
      COMMON/PAGES/FULSKP
      DATA INTST/'LEVEL','COEF'/,LOGST/'END'/

      CALL PAGSKP
      WRITE (6,200) TITLE,DATE,TIME

      IF (NOHIER.NE.0) THEN
         WRITE (6,300)
         RETURN
      ENDIF
      IF (NOCOEF .EQ. 1 ) THEN
         WRITE (6,301)
         RETURN
      ENDIF
      REWIND 14
      READ (14) IHIER,HNUM,NAME

*  Defaults:

  3   INTVL(1)=0
      INTVL(2)=0
      LOGVL(1)=.FALSE.

      CALL READER (2,0,1,INTST,REALST,LOGST,INTVL,REALVL,LOGVL,IERROR)
      IF (IERROR.EQ.1) GO TO 3
      IF (LOGVL(1)) RETURN
      LEVEL=INTVL(1)
      NCOEF=INTVL(2)

      IF (LEVEL.LE.0 .OR. LEVEL.GE.4) THEN
         WRITE (6,302) INTST(1),LEVEL
         GO TO 3
      ENDIF
      IF (LEVEL.GT.NLEVEL) THEN
         WRITE (6,303) NLEVEL
         GO TO 3
      ENDIF

      CALL PAGSKP
      WRITE (6,201) HNAME(LEVEL)
      IEND=LENSTR(HNAME(LEVEL))
      WRITE (6,202) ('*',I=1,IEND)
      WRITE (6,203) COEF(NCOEF)

*  Read coefficients:

      REWIND 15
      DO 501 J=1,NMAT
         IF (MATFIL(J).EQ.NCOEF) GO TO 7
            READ (15)
501   CONTINUE
      WRITE (6,304) NCOEF,(COEF((NC-1)*13 + NCOEF),NC=1,6)
      GO TO 3
  7   READ (15) ((MATRIX(I,J),I=J,NOTU),J=1,NOTU)

*  Sums and minima/maxima:

      NT=NHIER(LEVEL)
      DO 502 K1=1,NT
         NPOPS(K1)=0
         DO 503 K2=1,K1
            CMIN(K1,K2)=99.E9
            CMAX(K1,K2)=-99.E9
            CAVE(K1,K2)=0.
503      CONTINUE
502   CONTINUE
      I=IHIER(HNUM(1),LEVEL)
      NPOPS(I)=1
      DO 504 K1=2,NOTU
         I=IHIER(HNUM(K1),LEVEL)
         NPOPS(I)=NPOPS(I)+1
         DO 505 K2=1,K1-1
            J=IHIER(HNUM(K2),LEVEL)
            QCOEF=MATRIX(K1,K2)
            CALL SWITCH (I,J,IS,JS)
            CAVE(IS,JS)=CAVE(IS,JS)+QCOEF
            IF (QCOEF.LT.CMIN(IS,JS)) CMIN(IS,JS)=QCOEF
            IF (QCOEF.GT.CMAX(IS,JS)) CMAX(IS,JS)=QCOEF
505      CONTINUE
504   CONTINUE

*  Averages:

      NOCOMP=.FALSE.
      DO 506 K1=1,NT
         DO 507 K2=1,K1
            IF (K1 .EQ. K2) THEN
               NCOMPS=NPOPS(K1)*(NPOPS(K1)-1)/2
            ELSE
               NCOMPS=NPOPS(K1)*NPOPS(K2)
            ENDIF
            IF (NCOMPS.EQ.0) THEN
*              ( to print asterisks )
               CAVE(K1,K2)=99.E9
               CMIN(K1,K2)=99.E9
               CMAX(K1,K2)=99.E9
               NOCOMP=.TRUE.
            ELSE
               CAVE(K1,K2)=CAVE(K1,K2)/NCOMPS
            ENDIF
507      CONTINUE
506   CONTINUE

*  Output:

      NCOL=6
      IF (CRT) NCOL=3
      KMIN=1
 11   KMAX=MIN(KMIN+NCOL-1,NT)
      IF (KMIN .NE. 1) THEN
         CALL PAGSKP
         WRITE (6,204)
      ENDIF
      WRITE (6,205) ('---------------',K=KMIN,KMAX)
      WRITE (6,206) HNAME(LEVEL),(K,K=KMIN,KMAX)
      WRITE (6,205) ('---------------',K=KMIN,KMAX)
      DO 508 K1=KMIN,NT
         KEND=MIN(K1,KMAX)
         K=0
         DO 550 K2=KMIN,KEND
            K=K+1
            IF (CAVE(K1,K2).EQ.99.E9) THEN
               LINAVE(K)='*****'
               LINMIN(K)='*****'
               LINMAX(K)='*****'
            ELSE
               WRITE (LINAVE(K),'(F5.3)') CAVE(K1,K2)
               WRITE (LINMIN(K),'(F5.3)') CMIN(K1,K2)
               WRITE (LINMAX(K),'(F5.3)') CMAX(K1,K2)
            ENDIF
550      CONTINUE
         WRITE (6,207) K1,NAME(K1,LEVEL),NPOPS(K1),(LINAVE(L),L=1,K)
         WRITE (6,208) (LINMIN(L),LINMAX(L),L=1,K)
508   CONTINUE
      WRITE (6,205) ('----------------',K=KMIN,KMAX)
      IF (NOCOMP) WRITE (6,209)

      IF (KMAX.EQ.NT) GO TO 3
      KMIN=KMIN+NCOL
      GO TO 11

200   FORMAT (//////T3,76('*')/T3,'*',T78,'*'/T3,'*',2X,A72,
     +'*'/T3,'*',T78,'*'/T3,
     +'*  Similarity/distance coefficient averaging',
     +T78,'*'/T3,'*',T78,'*'/T3,'*  BIOSYS-1   Release 1.7',3X,A12,3X,
     +A12,T78,'*'/T3,'*',T78,'*'/T3,76('*')//////)
201   FORMAT (////' Matrix of similarity/distance coefficients ',
     +'averaged by ',a10)
202   FORMAT (1X,55('*'),10A1)
203   FORMAT (/5X,'Coefficient: ',A48///)
204   FORMAT (////' (Matrix continued)'///)
205   FORMAT (1X,29('-'),6A15)
206   FORMAT (T25,'No.of'/T5,A8,'  ',T25,'pops.',I9,5I15)
207   FORMAT (/1X,I2,1X,A16,I6,8X,A5,5(10X,A5))
208   FORMAT (T30,6(2X,:,'(',A5,'-',A5,')'))
C209   FORMAT (////' ***** Only one population included')
209   FORMAT (////' ***** No comparisons')
300   FORMAT (' ERROR: Step HIERARCHY has not been performed or ',
     +'previously failed.')
301   FORMAT (' ERROR: Step SIMDIS has not been performed')
302   FORMAT (' ERROR: Illegal value for ',A5,':',I5/)
303   FORMAT (' ERROR: Previously specified hierarchy has only',i2,
     +' level(s)'/)
304   FORMAT (////' ERROR: Matrix no.',I3,' = ',A48/8X,
     +'not on file.')
      END


*=======================================================================*
*                                                                       *
*                       S T E P   S I N G L E                           *
*                                                                       *
*         Single-locus similarity and distance coefficients             *
*                                                                       *
*  Parameter:                                                           *
*     NCOEF ("COEF") = coefficient to be calculated (see routine        *
*                         SIMDIS)                                       *
*                                                                       *
*  Subprograms required by step SINGLE:  SINGLE READER SIMDIS ALLCNT    *
*                                        MATPRT                         *
*                                                                       *
*=======================================================================*

      SUBROUTINE SINGLE
     +   (MATRIX,POPNAM,LOCNAM,JXY,DSING,DROG1,DROG2,DARC,DCHORD,
     +    DPREV,DEDW,SROG,IUNB,DUNB,IDEN,DIST,DMINUN,DMIN,FREQ2,
     +    MINI2,MAXI2,N2,JX,GX,SMJX2,SMGX2,EDJX2,MA,ML,MT)

*  Gets single-locus similarity coefficients and calls printing routine

      CHARACTER LINE*80,DATE*12,TIME*12,TITLE*72,FMT*80,INTST*64,
     +   REALST*8,LOGST*8,LOCNAM*5,COEF*48,POPNAM*16
      REAL JXY,DSING,DROG1,DROG2,DARC,DCHORD,DPREV,DEDW,SROG,IUNB,DUNB,
     +   IDEN,DIST,DMINUN,DMIN,JX,GX,SMJX2,SMGX2,EDJX2,FREQ2,MATRIX,
     +   REALVL
      INTEGER MINI2,MAXI2,N2,INTVL
      LOGICAL LOGVL
      COMMON/GBPARM/NOTU,NOTU2,NLOC,NALL,NLET,CRT,NDATYP
      COMMON/MAT/MATFIL(13),NOCOEF,NMAT,NSING
      COMMON/COEFNM/COEF(13)
      COMMON/CHARV/LINE,DATE,TIME,TITLE,FMT
      DIMENSION POPNAM(MT),INTST(1),REALST(1),LOGST(1),LOCNAM(ML),
     +   JXY(MT,MT),DSING(MT,MT),DROG1(MT,MT),DROG2(MT,MT),DARC(MT,MT),
     +   DCHORD(MT,MT),DPREV(MT,MT),DEDW(MT,MT),SROG(MT,MT),IUNB(MT,MT),
     +   DUNB(MT,MT),IDEN(MT,MT),DIST(MT,MT),DMINUN(MT,MT),DMIN(MT,MT),
     +   JX(MT),GX(MT),SMJX2(MT),SMGX2(MT),EDJX2(MT),FREQ2(MA,MT),
     +   MATRIX(MT,MT),MINI2(MT),MAXI2(MT),N2(MT),INTVL(1),REALVL(1),
     +   LOGVL(1)
      LOGICAL FULSKP
      COMMON/PAGES/FULSKP
      DATA INTST/'COEF'/,LOGST/'END'/,REALST/'ZZZ'/

      CALL PAGSKP
      WRITE (6,200) TITLE,DATE,TIME

  1   INTVL(1)=0
      LOGVL(1)=.FALSE.
      CALL READER (1,0,1,INTST,REALST,LOGST,INTVL,REALVL,LOGVL,IERROR)
      IF (IERROR.EQ.1) GO TO 1
      IF (LOGVL(1)) RETURN
      NCOEF=INTVL(1)
      IF (NCOEF.LE.0 .OR. NCOEF.GE.14) THEN
         WRITE (6,300) NCOEF
         GO TO 1
      ENDIF

*  Calculate coefficients if not already on file:

      IF (NCOEF .NE. NSING) CALL SIMDIS (JXY,DSING,DROG1,DROG2,DARC,
     +   DCHORD,DPREV,DEDW,SROG,IUNB,DUNB,IDEN,DIST,DMINUN,DMIN,FREQ2,
     +   MINI2,MAXI2,N2,JX,GX,SMJX2,SMGX2,EDJX2,POPNAM,MA,MT,NCOEF)

*  Read coefficients and print:

      REWIND 19
      DO 500 J=1,NLOC
         READ (19) ((MATRIX(K1,K2),K1=K2,NOTU),K2=1,NOTU)
         CALL MATPRT (2,0,0,LOCNAM(J),MATRIX,POPNAM,1,MT)
500   CONTINUE
      GO TO 1

200   FORMAT (//////T3,76('*')/T3,'*',T78,'*'/T3,'*',2X,A72,
     +'*'/T3,'*',T78,'*'/T3,
     +'*  Single-locus similarity/distance coefficient output',
     +T78,'*'/T3,'*',T78,'*'/T3,'*  BIOSYS-1   Release 1.7',3X,A12,3X,
     +A12,T78,'*'/T3,'*',T78,'*'/T3,76('*')//////)
300   FORMAT (' ERROR: Illegal value for COEF:',I5/)
      END
