*=======================================================================*
*                                                                       *
*                  S T E P   V A R I A B I L I T Y                      *
*                                                                       *
*      Genetic variability measures and their standard errors           *
*                                                                       *
*  Options:                                                             *
*     FULOUT     : full output (including single-locus heterozygosity   *
*                     estimates); default is summary table only         *
*     HTYPE      = 1  unbiased H estimates in summary table (default)   *
*                = 2  biased H estimate in summary table                *
*     PCRIT      = 0  no criterion for polymorphism in summary table    *
*                        (default)                                      *
*                = 1  99% criterion for polymorphism                    *
*                = 2  95% criterion for polymorphism                    *
*                                                                       *
*  Subprograms required by step VARIABILITY: VARIAB  READER             *
*                                                                       *
*=======================================================================*

      SUBROUTINE VARIAB
     +   (POPNAM,POPNUM,LOCNAM,FREQ,MINI,MAXI,N,NOBS,HBI,HUNB,HDC,
     +    NALLEL,AVEN,AVEALL,HETUNB,HETBIA,HETDIR,POLYNO,POLY99,POLY95,
     +    SEN,SEALL,SEHBIA,SEHUNB,SEHDIR,MT,ML,MA)

*  Calculates and prints measures of genetic variability within
*  populations

      CHARACTER LINE*80,DUM,DATE*12,TIME*12,TITLE*72,FMT*80,LETTER,
     +   POPNAM*16,POPNUM*4,INTST*64,REALST*8,LOGST*8,LOCNAM*5
      REAL FREQ,HBI,HUNB,HDC,AVEN,AVEALL,HETUNB,HETBIA,HETDIR,POLYNO,
     +   POLY99,POLY95,SEN,SEALL,SEHBIA,SEHUNB,SEHDIR,PLEVEL,DIFF
      INTEGER MINI,MAXI,N,NOBS,HTYPE,PCRIT
      LOGICAL FULOUT,CRT,LOGVL
      COMMON/GBPARM/NOTU,NOTU2,NLOC,NALL,NLET,CRT,NDATYP
      COMMON/SYMB/LETTER(26)
      COMMON/CHARV/LINE,DATE,TIME,TITLE,FMT
      DIMENSION DUM(11),POPNAM(MT),POPNUM(ML),FREQ(MA,ML),HBI(ML),
     +   HUNB(ML),HDC(ML),AVEN(MT),AVEALL(MT),HETUNB(MT),HETBIA(MT),
     +   HETDIR(MT),POLYNO(MT),POLY99(MT),POLY95(MT),SEN(MT),
     +   SEALL(MT),SEHBIA(MT),SEHUNB(MT),SEHDIR(MT),PLEVEL(2),
     +   INTST(2),LOGST(2),REALST(1),LOCNAM(ML),MINI(ML),MAXI(ML),N(ML),
     +   NOBS(MA,MA,ML),INTVL(2),LOGVL(2),REALVL(1),NALLEL(ML)
      LOGICAL FULSKP
      COMMON/PAGES/FULSKP
      SAVE INTST,LOGST,REALST,PLEVEL

      DATA PLEVEL/0.99,0.95/
      DATA INTST/'HTYPE','PCRIT'/,LOGST/'FULLOUT','END'/,REALST/'ZZZ'/

      CALL PAGSKP
      WRITE (6,200) TITLE,DATE,TIME

*  Defaults:

      INTVL(1)=1
      INTVL(2)=0
      LOGVL(1)=.FALSE.
      LOGVL(2)=.FALSE.

      CALL READER (2,0,2,INTST,REALST,LOGST,INTVL,REALVL,LOGVL,IERROR)
      IF (IERROR.EQ.1) RETURN
      FULOUT=LOGVL(1)
      HTYPE=INTVL(1)
      PCRIT=INTVL(2)
      IF (HTYPE.NE.1 .AND. HTYPE.NE.2) THEN
         WRITE (6,300) INTST(2),HTYPE
         HTYPE=1
      ENDIF
      IF (PCRIT.LT.0 .OR. PCRIT.GE.3) THEN
         WRITE (6,300) INTST(2),PCRIT
         PCRIT=0
      ENDIF
      REWIND 17
      IF (NDATYP .NE. 3) REWIND 18
      DO 500 K=1,NOTU
         READ (17) FREQ,MINI,MAXI,N
         IF (NDATYP .NE. 3) READ (18) NOBS
         LPSUM=0
         MPSUM=0
         NPSUM=0
         NSUM=0
         NASUM=0
         SUMHBI=0.
         SUMHUN=0.
         SUMHDC=0.
         IMIN=999
         IMAX=0

*     Number of alleles, test for polymorphism, and Hardy-Weinberg
*     expected and direct-count heterozygosity for each locus:

         DO 501 J=1,NLOC
            MINIJ=MINI(J)
            IF (MINIJ.LT.IMIN) IMIN=MINIJ
            MAXIJ=MAXI(J)
            IF (MAXI(J).GT.IMAX) IMAX=MAXIJ
            NJ=N(J)
            NHOM=0
            LPOLY=0
            MPOLY=0
            NPOLY=0
            NALLS=0
            SUMXSQ=0.
*           ( If MINIJ=MAXIJ, locus is monomorphic in Kth population )
            IF (MINIJ.EQ.MAXIJ) THEN
               NALLS=1
               NASUM=NASUM+1
               NHOM=NJ
               SUMXSQ=SUMXSQ+1.
            ELSE
               LPOLY=1
               DO 502 I=MINIJ,MAXIJ
                  IF (NDATYP .NE. 3) NHOM=NHOM+NOBS(I,I,J)
                  QFREQ=FREQ(I,J)
                  SUMXSQ=SUMXSQ+QFREQ*QFREQ
                  IF (QFREQ.NE.0.) THEN
                     NALLS=NALLS+1
                     IF (QFREQ.GT.0.01 .AND. QFREQ.LE.0.99) THEN
                        MPOLY=1
                        IF (.NOT.(QFREQ.LE.0.05  .OR.
     +                            QFREQ.GT.0.95)) NPOLY=1
                     ENDIF
                  ENDIF
502            CONTINUE
               LPSUM=LPSUM+LPOLY
               MPSUM=MPSUM+MPOLY
               NPSUM=NPSUM+NPOLY
               NASUM=NASUM+NALLS
            ENDIF
            NALLEL(J)=NALLS
            HBIA=1.-SUMXSQ
            HBI(J)=HBIA
            SUMHBI=SUMHBI+HBIA
            HUNBIA=2.*NJ*HBIA/(2.*NJ-1)
            HUNB(J)=HUNBIA
            SUMHUN=SUMHUN+HUNBIA
            HDIR=FLOAT(NJ-NHOM)/NJ
            HDC(J)=HDIR
            SUMHDC=SUMHDC+HDIR
            NSUM=NSUM+NJ
501      CONTINUE

*        Averages across loci:

         SUMN=NSUM
         SUMALL=NASUM
         AVEN(K)=SUMN/NLOC
         AVEALL(K)=SUMALL/NLOC
         HETBIA(K)=SUMHBI/NLOC
         HETUNB(K)=SUMHUN/NLOC
         HETDIR(K)=SUMHDC/NLOC
         POLYNO(K)=(100.*LPSUM)/NLOC
         POLY99(K)=(100.*MPSUM)/NLOC
         POLY95(K)=(100.*NPSUM)/NLOC

*        Standard errors:

         SSN = 0.
         SSALL = 0.
         SSHBIA=0.
         SSHUNB=0.
         SSHDC=0.
         DO 503 J=1,NLOC
            DIFF=N(J)-AVEN(K)
            SSN=SSN+DIFF*DIFF
            DIFF=NALLEL(J)-AVEALL(K)
            SSALL=SSALL+DIFF*DIFF
            DIFF=HBI(J)-HETBIA(K)
            SSHBIA=SSHBIA+DIFF*DIFF
            DIFF=HUNB(J)-HETUNB(K)
            SSHUNB=SSHUNB+DIFF*DIFF
            DIFF=HDC(J)-HETDIR(K)
            SSHDC=SSHDC+DIFF*DIFF
503      CONTINUE
         DENOM=NLOC*(NLOC-1)
         SEN(K)=SQRT(SSN/DENOM)
         SEALL(K)=SQRT(SSALL/DENOM)
         SEHBIA(K)=SQRT(SSHBIA/DENOM)
         SEHUNB(K)=SQRT(SSHUNB/DENOM)
         SEHDIR(K)=SQRT(SSHDC/DENOM)

*        Full output:

         IF (FULOUT) THEN
            NCOL=15
            IF (CRT) NCOL=9
            CALL PAGSKP
            WRITE (6,201) POPNAM(K),POPNUM(K)
            J1=1
  5         J2=MIN(J1+NCOL-1,NLOC)
            NDIFF=NLOC-J2-3
            IF (NDIFF.LE.0 .AND. NDIFF.GT.-2) THEN
               J2=J2+NDIFF
               NCOL=NCOL+NDIFF
            ENDIF
            WRITE (6,202) ('--------',J=J1,J2)
            NCOLS=J2-J1+1
            FMT='(6X,  (4X),''Locus and sample size'')'
            WRITE (FMT(5:6),'(I2)') MAX(NCOLS-2, 1)
            WRITE (6,FMT)
            J1P1=J1+1
            WRITE (6,203) ('--------',J=J1P1,J2)
            WRITE (6,204) (LOCNAM(J),J=J1,J2)
            WRITE (6,205) (N(J),J=J1,J2)
            WRITE (6,206) ('--------',J=J1,J2)
            WRITE (6,207)
            DO 504 I=IMIN,IMAX
               IF (NLET.EQ.1) THEN
                  WRITE (6,209) LETTER(I),(FREQ(I,J),J=J1,J2)
               ELSE
                  WRITE (6,208) I,(FREQ(I,J),J=J1,J2)
               ENDIF
504         CONTINUE
            WRITE (6,210) (HBI(J),J=J1,J2)
            WRITE (6,211) (HUNB(J),J=J1,J2)

            IF (NDATYP .NE. 3) WRITE (6,212) (HDC(J),J=J1,J2)
            WRITE (6,206) ('--------',J=J1,J2)
            IF (J2.NE.NLOC) THEN
               J1=J1+NCOL
               GO TO 5
            ENDIF
            WRITE (6,213) HETBIA(K),SEHBIA(K),HETUNB(K),SEHUNB(K)
            IF (NDATYP .NE. 3) WRITE (6,214) HETDIR(K),SEHDIR(K)
            WRITE (6,215) AVEALL(K),SEALL(K)
            WRITE (6,216) POLY95(K),POLY99(K),POLYNO(K)
         ENDIF
500   CONTINUE

*  Summary of genetic variability in all populations:

      CALL PAGSKP
      WRITE (6,217) NLOC
      WRITE (6,218)
      DO 505 K=1,NOTU
         IF (HTYPE.NE.2) THEN
            HET=HETUNB(K)
            SEHET=SEHUNB(K)
         ELSE
            HET=HETBIA(K)
            SEHET=SEHBIA(K)
         ENDIF
         IF ((PCRIT-1).LE.0) THEN
            POLY=POLYNO(K)
         ELSE IF ((PCRIT-1).EQ.0) THEN
            POLY=POLY99(K)
         ELSE IF ((PCRIT-1).GT.0) THEN
            POLY=POLY95(K)
         ENDIF
         IF (NDATYP.NE.3) THEN
            WRITE (6,219) K,POPNAM(K),AVEN(K),AVEALL(K),POLY,
     +         HETDIR(K),HET
            WRITE (6,220) SEN(K),SEALL(K),SEHDIR(K),SEHET
         ELSE
            WRITE (6,221) K,POPNAM(K),AVEN(K),AVEALL(K),POLY,HET
            WRITE (6,222) SEN(K),SEALL(K),SEHET
         ENDIF
505   CONTINUE
      WRITE (6,223)

*  Footnotes:

      IF (PCRIT.GT.0) THEN
         WRITE (6,225) PLEVEL(PCRIT)
      ELSE
         WRITE (6,224)
      ENDIF
      IF (HTYPE.EQ.1) THEN
         WRITE (6,226)
      ELSE
         WRITE (6,227)
      ENDIF
      IF (NDATYP.EQ.3) WRITE (6,228)
      RETURN

200   FORMAT (//////T3,76('*')/T3,'*',T78,'*'/T3,'*',2X,A72,
     +'*'/T3,'*',T78,'*'/T3,
     +'*  Genetic variability analysis',
     +T78,'*'/T3,'*',T78,'*'/T3,'*  BIOSYS-1   Release 1.7',3X,A12,3X,
     +A12,T78,'*'/T3,'*',T78,'*'/T3,76('*')//////)
201   FORMAT (' Allele frequencies and genetic variability measures'/
     +1X,51('*')//6X,'Population: ',A16,' (',A4,')')
202   FORMAT (////' ------',15A8)
203   FORMAT(10X,'-----',14A8)
204   FORMAT (7X,15(3X,A5))
205   FORMAT (' Allele',I7,14I8)
206   FORMAT (' ------',15A8)
207   FORMAT (' ')
208   FORMAT (3X,I2,2X,15F8.3)
209   FORMAT (4X,A1,2X,15F8.3)
210   FORMAT (/' H',5X,15F8.3)
211   FORMAT (' H(unb)',15F8.3)
212   FORMAT (' H(D.C.)',F7.3,14F8.3)
213   FORMAT (/////' Mean heterozygosity per locus (biased estimate) ='
     +,F6.3,' (S.E.',F6.3,')'//' Mean heterozygosity per locus (unbia'
     +,'sed estimate) =',F6.3,' (S.E.',F6.3,')')
214   FORMAT (/' Mean heterozygosity per locus (direct-count estimate',
     +') =',F6.3,' (S.E.',F6.3,')')
215   FORMAT (/' Mean number of alleles per locus =',F5.2,' (S.E.',
     +F5.2,')')
216   FORMAT (/' Percentage of loci polymorphic (0.95 criterion) =',
     +F6.2//' Percentage of loci polymorphic (0.99 criterion) =',
     +F6.2//' Percentage of loci polymorphic (no criterion) =',
     +F6.2//////)
217   FORMAT (' Genetic variability at',I3,' loci in all populations'/
     +1X,49('*')//6X,'(standard errors in parentheses)')
218   FORMAT (////1X,79('-')/T61,'Mean heterozygosity'/T22,
     +'Mean sample',2X,'Mean no.',4X,'Percentage',4X,19('-')/T23,
     +'size per  of alleles',4X,'of loci',T62,'Direct-',3X,'HdyWbg'/
     +T7,'Population',T25,'Locus',T34,'per locus',2X,' polymorphic*',
     +T63,'count',T71,'expected**'/1X,79('-'))
219   FORMAT (/1X,I3,'. ',A16,F7.1,F11.1,T49,F5.1,T58,2F10.3)
220   FORMAT (T25,'(',F4.1,')',T37,'(',F3.1,')',T59,2(3X,'(',F5.3,')'))
221   FORMAT (//1X,I3,'. ',A16,F7.1,F11.1,T49,F5.1,T64,'***',T73,
     +F5.3)
222   FORMAT (T25,'(',F4.1,')',T37,'(',F3.1,')',T72,'(',F5.3,')')
223   FORMAT (/1X,79('-'))
224   FORMAT (/5X,'* A locus is considered polymorphic if more than ',
     +'one allele was detected')
225   FORMAT (/5X,'* A locus is considered polymorphic if the ',
     +'frequency'/' of the most common allele does not exceed',F5.2)
226   FORMAT (/4X,'** Unbiased estimate (see Nei, 1978)')
227   FORMAT (/4X,'** Biased estimate (see Nei, 1978)')
228   FORMAT (/3X,'*** Direct-count heterozygosity cannot be ',
     +'calculated'/' from allele frequency data alone')
300   FORMAT (' WARNING: Illegal value for ',A5,']',I5,
     +   '(reset to default value)'///)
      END
