*=======================================================================*
*                                                                       *
*                   S T E P   W R I G H T   7 8                         *
*                                                                       *
*           F-statistics as presented by Wright (1978)                  *
*                                                                       *
*  Calculates hierarchical and non-hierarchical F-statistics using      *
*  formulation of Wright (1978).  Unlike step FSTAT, step WRIGHT78      *
*  operates only from gene frequencies, therefore F(is) and F(it)       *
*  cannot be computed.  (Note that step WRIGHT78 must be used for       *
*  F-statistic calculation if DATYP=3.)  Also, step Wrright78provides   *
*  for correction of mean squares for sampling error, which is not      *
*  provided in step FSTAT.                                              *
*                                                                       *
*  Options:                                                             *
*     NOCORR            : do not correct among demes mean square for    *
*                            sampling error (default is to perform this *
*                            correction)                                *
*     NOHCHY ("NOHRCHY"): do not perform hierarchical analysis.  treat  *
*                            each population (deme) as a subdivision of *
*                            the total set of populations.              *
*                                                                       *
*  Subprograms required by step WRIGHT 78: WRIGHT  READER  NONHRF       *
*                                                                       *
*=======================================================================*

      SUBROUTINE WRIGHT
     +   (FREQ2,QSUM,VC,F,MINI2,MAXI2,N2,IHIER,HNUM,NSUM,PRES,
     +    LOCNAM,MA,ML,MT)

      CHARACTER INTST*64,REALST*8,LOGST*8,LOCNAM*5,HNAME*10,
     +   LNAME*10,DATE*12,TIME*12,TITLE*72,FMT*80,LINE*80,LETTER
      REAL FREQ2,QSUM,VC,F,SUM,MSQ,REALVL
      INTEGER MINI2,MAXI2,N2,IHIER,NSUM,HNUM
      LOGICAL PRES,NOCORR,NOHCHY,LOGVL
      COMMON/ADDED/VCSUM,VCTOT,FSUM,FTOT,ONEWRD
      COMMON/GBPARM/NOTU,NOTU2,NLOC,NALL,NLET,CRT,NDATYP
      COMMON/HIER1/NOHIER,NLEVEL,NROW,NHIER(4)
      COMMON/HIER/HNAME(4)
      COMMON/SYMB/LETTER(26)
      COMMON/CHARV/LINE,DATE,TIME,TITLE,FMT
      DIMENSION LNAME(5),INTST(1),LOGST(3),LOCNAM(ML),FREQ2(MA,MT),
     +    QSUM(MT,3),VC(5,5,MA),F(5,5,MA),SUM(5),MINI2(MT),MAXI2(MT),
     +    N2(MT),IHIER(MT,3),NSUM(MT,3),HNUM(MT),PRES(MA),VCSUM(5,5),
     +    VCTOT(5,5),FSUM(5,5),FTOT(5,5),REALST(1),INTVL(1),REALVL(1),
     +    LOGVL(3)
      LOGICAL FULSKP
      COMMON/PAGES/FULSKP
      DATA UNDEF/-99.E9/,ZERO/1.E-6/,
     +   INTST/'NOPARM'/,LOGST/'NOCORR','NOHRCHY','END'/

      CALL PAGSKP
      WRITE (6,200) TITLE,DATE,TIME

*  Defaults:

      INTVL(1)=0
      LOGVL(1)=.FALSE.
      LOGVL(2)=.FALSE.
      LOGVL(3)=.FALSE.

      CALL READER (1,0,3,INTST,REALST,LOGST,INTVL,REALVL,LOGVL,IERROR)
      IF (IERROR.EQ.1) RETURN
      NOCORR=LOGVL(1)
      IF (NOCORR) WRITE (6,201)
      NOHCHY=LOGVL(2)
      IF (NOHCHY) THEN
         WRITE (6,203)
         CALL NONHRF (FREQ2,MINI2,MAXI2,N2,LOCNAM,PRES,NOCORR,
     +                MA,ML,MT)
         RETURN
      ELSE
         WRITE (6,202)
         IF (NOHIER.NE.0) THEN
            WRITE (6,300)
            RETURN
         ENDIF
         REWIND 14
         READ (14) IHIER,HNUM
      ENDIF
      NLEVP1=NLEVEL+1
      NLEVP2=NLEVEL+2
      TOTDEN=0.
      L2=NLEVP2
      DO 500 L1=1,NLEVP1
         L2=L2-1
         LNAME(L2)=HNAME(L1)
500   CONTINUE
      LNAME(NLEVP2)='TOTAL'

      DO 501 L1=1,NLEVP1
         DO 502 L2=L1+1,NLEVP2
*           VCTOT(L1,L2)=0.
502      CONTINUE
501   CONTINUE

      REWIND 16
      DO 503 J=1,NLOC
         READ (16) FREQ2,MINI2,MAXI2,N2
         DO 504 L1=1,5
            DO 505 L2=L1,5
               VCSUM(L1,L2)=0.
505         CONTINUE
504      CONTINUE

*        Determine which alleles are present and catch invariant loci:

         NALLS=0
         DO 506 I=1,NALL
            PRES(I)=.FALSE.
            DO 507 K=1,NOTU
               IF (FREQ2(I,K).GT.ZERO) THEN
                  PRES(I)=.TRUE.
                  NALLS=NALLS+1
                  GO TO 506
               ENDIF
507         CONTINUE
506      CONTINUE
         IF (NALLS.NE.1) THEN
            CALL PAGSKP
            WRITE (6,204) LOCNAM(J)
            SUMDEN=0.
            SUMSE=0.
            DO 508 I=1,NALL
               IF (PRES(I)) THEN
               SUMQ=0.
               SUMQSQ=0.
               SESQ=0.
               DO 509 L=1,NLEVEL
                  KSTOP=NHIER(L)
                  DO 510 K=1,KSTOP
                     QSUM(K,L)=0.
                     NSUM(K,L)=0
510               CONTINUE
509            CONTINUE
               DO 511 K=1,NOTU
                  Q=FREQ2(I,K)
                  SUMQ=SUMQ+Q
                  SUMQSQ=SUMQSQ+Q*Q
                  SESQ=SESQ+Q*(1.-Q)/(2*N2(K))
                  DO 512 L=1,NLEVEL
                     LNUM=IHIER(HNUM(K),L)
                     QSUM(LNUM,L)=QSUM(LNUM,L)+Q
                     NSUM(LNUM,L)=NSUM(LNUM,L)+1
512               CONTINUE
511            CONTINUE
               QT=SUMQ/NOTU
               SESQ=SESQ/NOTU
               SUMSE=SUMSE+SESQ
               SUM(1)=SUMQ*SUMQ/NOTU
               DO 513 L=1,NLEVEL
                  SUML=0.
                  DO 514 K=1,NHIER(L)
                     IF (NSUM(K,L).GT.0)
     +                  SUML=SUML+QSUM(K,L)**2/NSUM(K,L)
514               CONTINUE
                  SUM(L+1)=SUML
513            CONTINUE
               SUM(NLEVP2)=SUMQSQ
               NHIER(NLEVP1)=NOTU
               PRVMSQ=SESQ
               IF (NOCORR) PRVMSQ=0.
               L=NLEVP2
               DO 515 LROW=1,NLEVP1
                  L=L-1
                  SSQ=SUM(L+1)-SUM(L)
                  IF (L.NE.1) THEN
                     IDF=NHIER(L)-NHIER(L-1)
                  ELSE
                     IDF=NHIER(1)
                  ENDIF
                  MSQ=SSQ/IDF
                  DIFF=MSQ-PRVMSQ
                  IF (L.EQ.NLEVP1) DIFF=AMAX1(0.,DIFF)
                  PRVMSQ=MSQ
                  VC(LROW,LROW+1,I)=DIFF*NHIER(L)/NOTU
                  VCSUM(LROW,LROW+1)=VCSUM(LROW,LROW+1) +
     +                   VC(LROW,LROW+1,I)
                  VCTOT(LROW,LROW+1)=VCTOT(LROW,LROW+1) +
     +                   VC(LROW,LROW+1,I)
515            CONTINUE
               DO 516 L1=1,NLEVEL
                  DO 517 L2=L1+2,NLEVP2
                     VC(L1,L2,I)=VC(L1,L2-1,I)+VC(L2-1,L2,I)
                     VCSUM(L1,L2)=VCSUM(L1,L2)+VC(L1,L2,I)
                     VCTOT(L1,L2)=VCTOT(L1,L2)+VC(L1,L2,I)
517               CONTINUE
516            CONTINUE
               DENOM=QT*(1.-QT)
               SUMDEN=SUMDEN+DENOM
               DO 518 L=1,NLEVP1
                  F(L,NLEVP2,I)=VC(L,NLEVP2,I)/DENOM
518            CONTINUE
               DO 519 L1=1,NLEVEL
                  FL1T=F(L1,NLEVP2,I)
                  DO 520 L2=L1+1,NLEVP1
                     FL2T=F(L2,NLEVP2,I)
                     IF (ABS(1.-FL2T).GT.ZERO) THEN
                        F(L1,L2,I)=(FL1T-FL2T)/(1.-FL2T)
                     ELSE
                        F(L1,L2,I)=UNDEF
                     ENDIF
520               CONTINUE
519            CONTINUE
               ENDIF
508         CONTINUE
            TOTDEN=TOTDEN+SUMDEN

*           Totals for j(th) locus:

            DO 521 L=1,NLEVP1
               FSUM(L,NLEVP2)=VCSUM(L,NLEVP2)/SUMDEN
521         CONTINUE
            DO 522 L1=1,NLEVEL
               FL1T=FSUM(L1,NLEVP2)
               DO 523 L2=L1+1,NLEVP1
                  FL2T=FSUM(L2,NLEVP2)
                  IF (ABS(1.-FL2T).GT.ZERO) THEN
                     FSUM(L1,L2)=(FL1T-FL2T)/(1.-FL2T)
                  ELSE
                     FSUM(L1,L2)=UNDEF
                  ENDIF
523            CONTINUE
522         CONTINUE

*           Print table:

            DO 524 L1=1,NLEVP1
               DO 525 L2=L1+1,NLEVP2
                  WRITE (6,205) LNAME(L1),LNAME(L2)
                  DO 526 I=1,NALL
                     IF (PRES(I)) THEN
                        IF (NLET.NE.1) THEN
                           IF (F(L1,L2,I).NE.UNDEF) THEN
                              WRITE (6,206) I,VC(L1,L2,I),
     +                           F(L1,L2,I)
                           ELSE
                              WRITE (6,207) I,VC(L1,L2,I)
                           ENDIF
                        ELSE
                           IF (F(L1,L2,I).NE.UNDEF) THEN
                              WRITE (6,208) LETTER(I),VC(L1,L2,I),
     +                                      F(L1,L2,I)
                           ELSE
                              WRITE (6,209) LETTER(I),VC(L1,L2,I)
                           ENDIF
                        ENDIF
                     ENDIF
526               CONTINUE
                  IF (FSUM(L1,L2).NE.UNDEF) THEN
                     WRITE (6,210) VCSUM(L1,L2),FSUM(L1,L2)
                  ELSE
                     WRITE (6,211) VCSUM(L1,L2)
                  ENDIF
525            CONTINUE
524         CONTINUE
            WRITE (6,212) SUMDEN
         ENDIF
503   CONTINUE

*  Totals for all loci:

      CALL PAGSKP
      WRITE (6,213)
      DO 527 L=1,NLEVP1
         FTOT(L,NLEVP2)=VCTOT(L,NLEVP2)/TOTDEN
527   CONTINUE
      DO 528 L1=1,NLEVEL
         FL1T=FTOT(L1,NLEVP2)
         DO 529 L2=L1+1,NLEVP1
            FL2T=FTOT(L2,NLEVP2)
            IF (ABS(1.-FL2T).GT.ZERO) THEN
               FTOT(L1,L2)=(FL1T-FL2T)/(1.-FL2T)
            ELSE
               FTOT(L1,L2)=UNDEF
            ENDIF
529      CONTINUE
528   CONTINUE
      DO 530 L1=1,NLEVP1
         DO 531 L2=L1+1,NLEVP2
            IF (FTOT(L1,L2).NE.UNDEF) THEN
               WRITE (6,214) LNAME(L1),LNAME(L2),VCTOT(L1,L2),
     +                       FTOT(L1,L2)
            ELSE
               WRITE (6,215) LNAME(L1),LNAME(L2),VCTOT(L1,L2)
            ENDIF
531      CONTINUE
530   CONTINUE
      WRITE (6,216)
      RETURN

200   FORMAT (//////T3,76('*')/T3,'*',T78,'*'/T3,'*',2X,A72,
     +'*'/T3,'*',T78,'*'/T3,
     +'*  Wright (1978) F-statistics',
     +T78,'*'/T3,'*',T78,'*'/T3,'*  BIOSYS-1   Release 1.7',3X,A12,3X,
     +A12,T78,'*'/T3,'*',T78,'*'/T3,76('*')//)
201   FORMAT (/' Correction for sampling error in demes not performed')
202   FORMAT (/' Hierarchical analysis'//////)
203   FORMAT (/' Non-hierarchical analysis'//////)
204   FORMAT (' Hierarchical F-statistics'/1X,25('*')//5X,'Locus: ',A5
     +////1X,66('-')/7X,'Comparison'/1X,22('-'),T43,'Variance',T64,
     +'F'/5X,'X',T19,'Y',T28,'Allele',T43,'component',T65,'XY'/
     +1X,66('-'))
205   FORMAT (/1X,A10,'- ',A10)
206   FORMAT (T29,I3,2X,F17.5,F17.3)
207   FORMAT (T29,I3,2X,F17.5,13X,'0/0')
208   FORMAT (T31,A1,2X,F17.5,F17.3)
209   FORMAT (T31,A1,2X,F17.5,13X,'0/0')
210   FORMAT (T29,'Total',F17.5,F17.3)
211   FORMAT (T29,'Total',F17.5,13X,'0/0')
212   FORMAT (1X,66('-')//' Total limiting variance = ',F10.5//////)
213   FORMAT (' Variance components and F-statistics combined across '
     +,'loci'/1X,57('*')////1X,56('-')/7X,'Comparison'/1X,22('-'),T33,
     +'Variance',T54,'F'/5X,'X',T19,'Y',T33,'component',T55,'XY'/
     +1X,56('-') )
214   FORMAT (1X,A10,'- ',A10,F17.5,F17.3)
215   FORMAT (1X,A10,'- ',A10,F17.5,13X,'0/0')
216   FORMAT (1X,56('-')////)
300   FORMAT (/' ERROR: Step HIERARCHY has not been performed or ',
     +'previously failed')
      END


      SUBROUTINE NONHRF
     +   (FREQ2,MINI2,MAXI2,N2,LOCNAM,PRES,NOCORR,MA,ML,MT)

*  Calculates non-hierarchical F-statistics using formulation of
*  Wright (1978)

      CHARACTER LETTER,LOCNAM*5
      REAL FREQ2
      INTEGER MINI2,MAXI2,N2
      LOGICAL PRES,NOCORR
      COMMON/GBPARM/NOTU,NOTU2,NLOC,NALL,NLET,CRT,NDATYP
      COMMON/SYMB/LETTER(26)
      DIMENSION LOCNAM(ML),FREQ2(MA,MT),MINI2(MT),MAXI2(MT),N2(MT),
     +   PRES(MA)
      LOGICAL FULSKP
      COMMON/PAGES/FULSKP
      DATA ZERO/1.E-6/

      CALL PAGSKP
      WRITE (6,200)
      TOTSE=0.
      TOTDEN=0.
      TOTVDT=0.
      NL=0

      REWIND 16
      DO 500 J=1,NLOC
         READ (16) FREQ2,MINI2,MAXI2,N2
         NALLS=0
         DO 501 I=1,NALL
            PRES(I)=.FALSE.
            DO 502 K=1,NOTU
               IF (FREQ2(I,K).GT.ZERO) THEN
                  PRES(I)=.TRUE.
                  NALLS=NALLS+1
                  GO TO 501
               ENDIF
502         CONTINUE
501      CONTINUE
         IF (NALLS.NE.1) THEN
            NL=NL+1
            WRITE (6,201) LOCNAM(J)
            SUMSE=0.
            SUMVDT=0.
            SUMDEN=0.
            DO 503 I=1,NALL
               IF (PRES(I)) THEN
                  SUMQ=0.
                  SUMQSQ=0.
                  SESQ=0.
                  DO 504 K=1,NOTU
                     Q=FREQ2(I,K)
                     SUMQ=SUMQ+Q
                     SUMQSQ=SUMQSQ+Q*Q
                     SESQ=SESQ+Q*(1.-Q)/(2*N2(K))
504               CONTINUE
                  QT=SUMQ/NOTU
                  SESQ=SESQ/NOTU
                  SUMSE=SUMSE+SESQ
                  SSQ=SUMQSQ-SUMQ*SUMQ/NOTU
                  VDT=SSQ/NOTU
                  IF (.NOT.NOCORR) VDT=VDT-SESQ
                  VDT=MAX(0.,VDT)
                  SUMVDT=SUMVDT+VDT
                  DENOM=QT*(1.-QT)
                  SUMDEN=SUMDEN+DENOM
                  FDT=VDT/DENOM
                  IF (NLET.NE.1) THEN
                     WRITE (6,202) I,QT,SESQ,VDT,DENOM,FDT
                  ELSE
                     WRITE (6,203) LETTER(I),QT,SESQ,VDT,DENOM,FDT
                  ENDIF
               ENDIF
503         CONTINUE
            FDTLOC=SUMVDT/SUMDEN
            WRITE (6,204) SUMSE,SUMVDT,SUMDEN,FDTLOC
            TOTSE=TOTSE+SUMSE
            TOTVDT=TOTVDT+SUMVDT
            TOTDEN=TOTDEN+SUMDEN
         ENDIF
500   CONTINUE
      AVEFDT=TOTVDT/TOTDEN
      AVESE=TOTSE/NL
      AVEVDT=TOTVDT/NL
      AVEDEN=TOTDEN/NL
      WRITE (6,205) AVESE,AVEVDT,AVEDEN,AVEFDT
      RETURN
*
200   FORMAT (' Non-hierarchical F-statistics'/1X,29('*')////1X,65('-')/
     +T19,'Mean',T29,'Sampling',T41,'Actual',T51,'Limiting',T63,
     +'F'/1X,'LOCUS',2X,'Allele',2X,'frequency',3X,'variance',
     +3X,'variance',3X,'variance',T64,'DT'/1X,65('-'))
201   FORMAT (/1X,A5)
202   FORMAT (8X,I3,2X,4F11.5,F9.3)
203   FORMAT (10X,A1,2X,4F11.5,F9.3)
204   FORMAT (8X,'Total',T21,'--- ',3F11.5,F9.3)
205   FORMAT (1X,65('-')/1X,'Average',T21,'----',3F11.5,F9.3/1X,
     +65('-')///)
      END
